<?php
/*
 * Example Connect
 *
 * Copyright (c) 2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of Example Connect and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Example Account.
 *
 * @package GhExampleConnect
 * @subpackage core
 * @category Example Connect
 *
 * @author GREYHOUND Software GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2019 GREYHOUND Software GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhExampleAccount extends GhAddonAccount
{
  /**
   * Addon.
   * @var GhExampleAddon
   */
  protected $_addon;


  /**
   * Konstruktor.
   *
   * @param GhAddon $addon    Addon
   * @param int     $id       Account-ID
   * @param array   $settings Account-Einstellungen
   */
  public function __construct($addon, $id, $settings)
  {
    parent::__construct($addon, $id, $settings);

    //TODO Initialisierung
  }

  /**
   * Diese Methode muss überladen werden und true liefern falls der Account
   * gültig ist (z.B. über alle notwendigen Einstellungen für die Zugangsdaten
   * zum Drittsystem verfügt).
   *
   * @return boolean
   */
  public function isValid()
  {
    //TODO Prüfen, ob alle notwendigen Einstellungen gesetzt sind:
    if ($this->getSetting('ApiUrl') && $this->getSetting('ApiUser') && $this->getSetting('ApiPassword')) {
      return true;
    } else {
      return false;
    }
  }

  /**
   * Erzeugt ein Datenobjekt, das Aufträge und Kunden repräsentiert.
   *
   * @param array $data Auftrags- und Kundendaten (Addon-spezifisch)
   * @return GhAddonData
   */
  public function newOrderData($data)
  {
    return new GhExampleData($this->_addon, $this, $data);
  }

  /**
   * Sucht bei Example nach einem Suchbegriff und liefert Kunden und
   * Bestellungen.
   *
   * @throws GhAddonEmptySearchException falls keine Datensätze gefunden wurden
   *
   * @param string $searchTerm Suchbegriff
   * @return GhAddonData
   */
  public function searchBySearchTerm($searchTerm, $params = null)
  {
    $fields = array();

    if (preg_match('/^[^@]*@[^\.]*\.[^\s]*$/', $searchTerm)) {
      // Nach E-Mail Adresse suchen:
      $data = $this->searchByEmail($searchTerm);
    } else {
      // Nach Suchbegriff suchen (z.B. Nachname, Stadt, PLZ):
      $data = null;
      $orders = array();

      if (preg_replace('/[^0-9]/', '', $searchTerm) == $searchTerm) {
        // Nur Ziffern:
        //TODO Nach Suchbegriffen suchen, die nur Ziffern enthalten (Auftragsnr, Kundennr, Rechnungsnr, PLZ)
        $fields = array('Auftragsnr.', 'Kundennr.', 'Rechnungsnr.', 'Postleitzahl');
      } elseif (preg_match('/[0-9]+/', $searchTerm)) {
        // Ziffern und Buchstaben:
        //TODO Nach Suchbegriffen suchen, die Ziffern und Buchstaben enthalten (Rechnungsnr, Straße, PLZ)
        $fields = array('Rechnungsnr.', 'Straße', 'Postleitzahl');
      } else {
        // Nur Buchstaben und/oder Sonderzeichen:
        //TODO Nach Suchbegriffen suchen, die nur Buchstaben und Sonderzeichen enthalten (Name, Straße, Stadt)
        $fields = array('Name', 'Straße', 'Stadt');
      }

      if (!empty($orders)) {
        $data = new GhExampleData($this->_addon, $this, $orders);

        // Gefundene Auftragsnummer auswählen:
        foreach ($data->getOrders() as $order) {
          if ($order->OrderNumber == $searchTerm) {
            $data->setSelectedOrderId($order->ID);
            break;
          }
        }
      }
    }

    if (!$data) {
      throw new GhAddonEmptySearchException($searchTerm, $fields, $searchTerm);
    } else {
      return $data;
    }
  }

  /**
   * Sucht nach einer E-Mail Adresse und liefert Kunden und Aufträge.
   *
   * @throws GhAddonEmptySearchException falls keine Datensätze gefunden wurden
   *
   * @param string $email E-Mail Adresse
   * @return GhAddonData
   */
  public function searchByEmail($email, $params = null)
  {
    $orders = array();
    //TODO Nach E-Mail Adresse suchen

    if (empty($orders)) {
      throw new GhAddonEmptySearchException($email, array('E-Mail'), $email);
    } else {
      return new GhExampleData($this->_addon, $this, $orders);
    }
  }

  /**
   * Sucht nach einem eBay Namen und liefert Kunden und Aufträge.
   *
   * @throws GhAddonEmptySearchException falls keine Datensätze gefunden wurden
   *
   * @param string $ebayName eBay Name
   * @return GhAddonData
   */
  public function searchByEbayName($ebayName, $params = null)
  {
    //TODO Nach eBay-Name suchen (falls nicht möglich, dann kann die Funktion entfernt werden)

    return null;
  }

  /**
   * Sucht nach einer Amazon Order-ID und liefert Kunden und Aufträge.
   *
   * @param string $amazonId Amazon Order-ID
   * @param array $params Alle Suchparameter
   * @return GhAddonData
   */
  public function searchByAmazonId($amazonId, $params = null)
  {
    //TODO nach Amazon Order-ID suchen (falls nicht möglich, dann kann die Funktion entfernt werden)
    return null;
  }

  /**
   * Sucht nach Kunden-IDs und liefert Kunden und Aufträge.
   *
   * @throws GhAddonEmptySearchException falls keine Datensätze gefunden wurden
   *
   * @param array $customerIds Kunden-IDs
   * @return GhAddonData
   */
  public function searchByCustomerIds($customerIds, $params = null)
  {
    if(!is_array($customerIds)) {
      $customerIds = array($customerIds);
    }

    $orders = array();
    //TODO Nach Kunden-IDs suchen

    if (empty($orders)) {
      throw new GhAddonEmptySearchException(is_array($customerIds) ? $customerIds[0] : $customerIds, array('Kundennr.'), is_array($customerIds) ? $customerIds[0] : $customerIds);
    } else {
      return new GhExampleData($this->_addon, $this, $orders);
    }
  }

  /**
   * Sucht nach Bestell-IDs und liefert Kunden und Aufträge.
   *
   * @throws GhAddonEmptySearchException falls keine Datensätze gefunden wurden
   *
   * @param array $orderIds Bestell-IDs
   * @return GhAddonData
   */
  public function searchByOrderIds($orderIds, $params = null)
  {
    if (!is_array($orderIds)) {
      $orderIds = array($orderIds);
    }

    $orders = array();
    //TODO Nach Auftrags-IDs suchen

    if (empty($orders)) {
      throw new GhAddonEmptySearchException(is_array($orderIds) ? $orderIds[0] : $orderIds, array('Auftragsnr.'), is_array($orderIds) ? $orderIds[0] : $orderIds);
    } else {
      return new GhExampleData($this->_addon, $this, $orders);
    }
  }

  /**
   * Liefert eine URL, die einen Kunden im Webbrowser oder einer Drittanwendung öffnet.
   *
   * @param GhAddonOrder $order Auftrag
   * @return string
   */
  public function getCustomerUrl($order)
  {
    //TODO URL zurück liefern, über die ein Kundendatensatz in Example angezeigt werden kann. Falls das nicht möglich ist, dann kann diese Funktion entfernt werden.
    return null;
  }

  /**
   * Liefert eine URL, die einen Auftrag im Webbrowser oder einer Drittanwendung öffnet.
   *
   * @param GhAddonOrder $order Auftrag
   * @return string
   */
  public function getOrderUrl($order)
  {
    //TODO URL zurück liefern, über die ein Auftragsdatensatz in Example angezeigt werden kann. Falls das nicht möglich ist, dann kann diese Funktion entfernt werden.
    return null;
  }

  /**
   * Liefert eine URL, die eine Rechnung im Webbrowser oder einer Drittanwendung öffnet.
   *
   * @param GhAddonOrder $order Auftrag
   * @return string
   */
  public function getInvoiceUrl($order)
  {
    //TODO URL zurück liefern, über die eine Rechnung in Example angezeigt werden kann. Falls das nicht möglich ist, dann kann diese Funktion entfernt werden.
    return null;
  }

  /**
   * Liefert eine URL, die einen Artikel im Webbrowser oder einer Drittanwendung öffnet.
   *
   * @param GhAddonOrderItem $orderItem Bestellposition
   * @param GhAddonOrder $order Auftrag
   * @return string
   */
  public function getProductUrl($orderItem, $order)
  {
    //TODO URL zurück liefern, über die ein Artikel in Example angezeigt werden kann. Falls das nicht möglich ist, dann kann diese Funktion entfernt werden.
    return null;
  }
}