<?php
/*
 * Example Connect
 *
 * Copyright (c) 2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of Example Connect and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Example Connect Addon.
 *
 * @package GhExampleConnect
 * @subpackage core
 * @category Example Connect
 *
 * @author GREYHOUND Software GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2019 GREYHOUND Software GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhExampleAddon extends GhAddon
{
  /**
   * Name des Addons bzw. des Drittanbieters.
   * @var string
   */
  protected $_addonName = 'exampleConnect';

  /**
   * Name des Systems des Drittanbieters.
   * @var string
   */
  protected $_remoteServiceName = 'Example';

  /**
   * Prefix für den Identifier von Extensions.
   * @var string
   */
  protected $_extensionIdentifierPrefix = 'ExampleConnect';

  /**
   * Versionsnummer des Addons.
   * @see $_addonBuild
   * @var string
   */
  protected $_addonVersion = '1.0.0';

  /**
   * Build-Nummer des Addons.
   * Die Build-Nummer wird unter anderem an die URL für eingebundene Javascript
   * Dateien als GET-Parameter angehängt, um sicherzustellen, dass bei neuen
   * Versionen die Scripte neu abgerufen werden (und nicht aus dem Cache
   * geladen werden).
   * @see $_addonVersion
   * @var string
   */
  protected $_addonBuild = 100;

  /**
   * Icon des Addons.
   * @var string
   */
  protected $_addonIcon = 'lib/images/icon.png';

  /**
   * Tooltip-Text für das Suchfeld.
   * @var string
   */
  protected $_searchTooltip = 'In Example nach Auftragsnr., Kundennr., E-Mail, etc. suchen.';

  /**
   * Name/Identifier des benutzerdefinierten Felds für die Kunden-ID des Drittanbieters.
   * @var string
   */
  protected $_customerIdUserFieldName = 'ExampleUserID';

  /**
   * Beschreibung des benutzerdefinierten Felds für die Kunden-ID des Drittanbieters.
   * @var string
   */
  protected $_customerIdUserFieldDescription = 'Example Kunden-ID';

  /**
   * Produkt-ID des Addons.
   * @var string
   */
  protected $_addonProductId = 'E68E34F79BD34A9610214DEED172F788';

  /**
   * E-Mail Adresse des Maintainers des Addons.
   * //TODO E-Mail Adresse, zu der Debug-Logs geschickt werden können
   * @var string
   */
  protected $_maintainerEmail = 'develop@greyhound-software.com';

  /**
   * Name des Maintainers des Addons.
   * //TODO E-Mail Name, zu dem Debug-Logs geschickt werden können
   * @var string
   */
  protected $_maintainerName = 'GREYHOUND Entwicklung';


  /**
   * Prüft Verbindungsdaten und versucht, eine Verbindung zum Drittsystem herzustellen.
   * Es wird eine Erfolgsmeldung zurückgeliefert oder eine Exception geworfen.
   *
   * @throws GhAddonConnectionException falls keine Verbindung hergestellt werden konnte
   *
   * @param array $settings Account-Einstellungen für die Verbindung
   * @return string
   */
  public function checkConnection($settings)
  {
    $account = new GhExampleAccount($this, $settings['id'], $settings);

    if (!$account->isValid()) {
      throw new GhAddonConnectionException('Die Verbindungsdaten sind unvollständig oder fehlerhaft');
    }

    try {
      //TODO Verbindungstest durchführen (prüfen, ob mit den Zugangsdaten eine gültige Verbindung aufgebaut werden kann)
      $account->searchByEmail('ghCheckConnection@greyhound-software.com');
    } catch (Exception $exception) {
      if (!($exception instanceof GhAddonEmptySearchException)) {
        throw new GhAddonConnectionException('Die Verbindung konnte nicht hergestellt werden: ' . $exception->getMessage());
      }
    }

    return 'Die Verbindung wurde erfolgreich hergestellt.';
  }

  /**
   * Liefert einen Example Account.
   *
   * @param int   $id       Account-ID
   * @param array $settings Account-Einstellungen
   * @return GhAddonAccount
   */
  protected function _createAccount($id, $settings)
  {
      return new GhExampleAccount($this, $id, $settings);
  }

  /**
   * Liefert die Account-Einstellungs-Optionen des Addons.
   *
   * @param array $settings  aktuelle Einstellungen des Addons
   * @param int   $accountId aktuelle Account-ID
   * @return array
   */
  public function getAccountOptions($settings = null, $accountId = null)
  {
    //TODO Konfigurations-Optionen pro Account (z.B. Zugangsdaten):
    $options = array(
      array(
        'caption' => 'Bezeichnung:',
        'property' => 'Caption',
        'help' => 'Die Bezeichnung kann beliebig gewählt werden und dient lediglich der Unterscheidung mehrerer Accounts.'
      ),
      array(
        'caption' => 'Aktiv:',
        'property' => 'Activated',
        'type' => 'checkbox'
      ),
      array(
        'caption' => 'API URL:',
        'property' => 'ApiUrl',
        'help' => 'Tragen Sie hier die Example URL ein.'
      ),
      array(
        'caption' => 'Benutzer:',
        'property' => 'ApiUser',
        'help' => 'Tragen Sie hier den Example Benutzernamen ein.'
      ),
      array(
        'caption' => 'Kennwort:',
        'property' => 'ApiPassword',
        'type' => 'password',
        'help' => 'Tragen Sie hier das Example Kennwort ein.'
      )
    );

    // Account: Empfänger:
    // Falls der Empfänger-Filter auf einen anderen Wert als '*' (Wildcard) eingestellt wird, so wird dieser Account
    // nur verwendet, wenn die Empfänger-Adresse eines GREYHOUND Elements auf diesen Filter passt. So können z.B. für
    // unterschiedliche E-Mail Empfänger verschiedene Accounts eingerichtet werden.
    $i = 0;
    $recipients = array();
    $recipient = GhAddonData::safeGet($settings, 'Recipient' . ($i++));

    while ($recipient) {
      if (strlen(trim($recipient)) > 0) {
        $recipients[] = $recipient;
      }

      $recipient = GhAddonData::safeGet($settings, 'Recipient' . ($i++));
    }

    if (empty($recipients)) {
      $recipients[] = '*';
    }

    $options[] = array(
      'caption' => 'Empfänger:',
      'property' => 'recipients',
      'configProperty' => false,
      'type' => 'textarea',
      'value' => implode("\r\n", $recipients),
      'height' => '40px',
      'help' => 'Es werden nur Example Daten für die hier eingetragenen Empfängeradressen abgerufen (Pro Zeile ist ein Empfänger erlaubt. Platzhalter wie z.B. * oder ? können verwendet werden).'
    );

    // Button zum Testen der Verbindung anbieten:
    $options[] = array(
      'caption' => 'Verbindung testen',
      'property' => 'action',
      'type' => 'button',
      'value' => 'checkConnection'
    );

    return $options;
  }

  /**
   * Liefert die Voreinstellungen für einen neuen Account.
   *
   * @return array
   */
  public function getNewAccountSettings()
  {
    //TODO Voreinstellungen für einen neu angelegten Account
    return array(
      'Caption' => 'Neue Verbindung',
      'Activated' => '-1',
      'ApiUrl' => '',
      'ApiUser' => '',
      'ApiPassword' => '',
      'ValidateSslCert' => '-1',
      'Recipient0' => '*'
    );
  }

  /**
   * Liefert die globalen Einstellungs-Optionen des Addons.
   *
   * @param array $settings aktuelle Einstellungen des Addons
   * @return array
   */
  public function getGlobalOptions($settings = null)
  {
    $taskNumberPrefixes = array((object)array('ID' => 0, 'Name' => 'nicht verwenden', 'icon' => 'forbidden2'));

    foreach (array('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'U', 'V', 'W', 'X', 'Y', 'Z') as $prefix) {
      $taskNumberPrefixes[] = (object)array('ID' => $prefix, 'Name' => $prefix . '-xxxxxxxxxx-xx', 'icon' => 'barcode');
    }

    $options = array(
      array(
        'caption' => 'Präfix für Vorgangsnummern:',
        'property' => 'TaskNumberPrefix',
        'type' => 'combotree',
        'options' => $taskNumberPrefixes,
        'help' => 'Um eingehende Nachrichten zum gleichen Example Auftrag automatisch zu einem Vorgang zusammen zu fassen, kann diese Funktion aktiviert und das gewünschte Präfix ausgewählt werden.'
      ),
      array(
        'caption' => 'Vorgangsnummern automatisch zuordnen:',
        'property' => 'TaskNumberAutoAssign',
        'type' => 'checkbox',
        'help' => 'Wenn diese Option aktiviert ist werden die Vorgangsnummern von Nachrichten, zu denen ein Example Auftrag gefunden wird, automatisch dem Auftrag zugeordnet. Ist die Option deaktiviert, so kann diese Zuordnung nur manuell erfolgen.'
      )
    );

    $userFields = array((object)array('ID' => 0, 'Description' => 'nicht verwenden', 'icon' => 'forbidden2'));
    $blHaveCreatedField = false;

    foreach (rpc_userfieldslib::newRpcUserFields($this->getGreyhoundClient())->GetList() as $userField) {
      if (GhRpcUtils::inSet($userField->Kinds, rpc_itemslib::ikContact) && ($userField->DataType == rpc_userfieldslib::ufdtString || $userField->DataType == rpc_userfieldslib::ufdtInteger || $userField->DataType == rpc_userfieldslib::ufdtInt64)) {
        $userField->icon = 'cube_green';
        $userFields[] = $userField;

        if ($userField->Name == $this->_customerIdUserFieldName) {
          $blHaveCreatedField = true;
        }
      }
    }

    $options[] = array(
      'caption' => 'Feld für Kunden-ID:',
      'property' => 'CustomerIDUserField',
      'type' => 'combotree',
      'options' => new GhRpcTree($userFields, 'Description'),
      'help' => 'Sofern bereits ein Benutzerdefiniertes Feld in GREYHOUND angelegt wurde, kann dieses Feld hier ausgewählt werden. Alternativ kann dieses hier angelegt werden. Sofern keine automatische Kontaktsynchronisierung in GREYHOUND eingerichtet wurde, sind hier keine Angaben notwendig.'
    );

    if (!$blHaveCreatedField) {
      $options[] = array(
        'caption' => 'Benutzerdef. Feld anlegen',
        'property' => 'action',
        'type' => 'button',
        'value' => 'createUserIdUserField'
      );
    }

    $textBlocks = array(
      (object)array('ID' => 0, 'Name' => 'nicht verwenden', 'icon' => 'forbidden2'),
      (object)array('ID' => -1, 'Name' => 'Alle Textbausteine', 'icon' => 'text')
    );

    try {
      foreach (rpc_textblockslib::newRpcTextBlocks($this->getGreyhoundClient())->GetList(false) as $textBlock) {
        $textBlock->icon = 'text';
        $textBlocks[] = $textBlock;
      }
    } catch (Exception $exception) {
    }

    $options[] = array(
      'caption' => 'Schnellzugriff für Textbausteine:',
      'property' => 'TextBlockRoot',
      'type' => 'combotree',
      'options' => $textBlocks,
      'help' => 'Hier kann der Basistextbaustein ausgewählt werden, welcher direkt über das Addon erreichbar sein soll. Damit entfällt das Wechseln des Karteireiters zwischen Addon und Textbausteinen.'
    );

    $options[] = array(
      'caption' => 'CC an Auftrags-E-Mail:',
      'property' => 'CustomerCC',
      'type' => 'checkbox',
      'help' => 'Beim Antworten wird eine Kopie (CC) der Nachricht an die E-Mailadresse des Kunden gesendet, welche im Auftrag steht.'
    );

    return $options;
  }

  /**
   * Liefert die erweiterten Einstellungs-Optionen des Addons.
   *
   * @param array $settings aktuelle Einstellungen des Addons
   * @return array
   */
  public function getExtendedOptions($settings = null)
  {
    return array(
      array(
        'caption' => 'Timeout (Sekunden):',
        'property' => 'ReadTimeout',
        'type' => 'number',
        'default' => 10,
        'help' => 'Bei besonders langsamen Example Verbindungen kann es notwendig sein den Timeout-Wert anzuheben, damit Verbindungsdaten angezeigt werden.'
      ),
      array(
        'caption' => 'Debug-Modus:',
        'property' => 'DebugMode',
        'type' => 'checkbox',
        'label' => '(Nur nach Absprache aktivieren)',
        'help' => 'Zur besseren Fehleranalyse kann der Debug-Modus aktiviert werden. Dadurch werden detailliertere Informationen bei Problemen angezeigt. Zusätzlich wird ein Protokoll des ' . $this->_remoteServiceName . ' Datenabrufs in eine Datei geschrieben, die über die Schaltfläche "Debug-Log versenden" an die GREYHOUND Entwicklungsabteilung gesendet werden kann. Dieses Debug-Log darf nur über sichere E-Mail Verbindungen (SSL/TLS) versendet werden, da es personenbezogene Daten enthalten kann.'
      ),
      array(
        'caption' => 'Debug-Log versenden',
        'property' => 'action',
        'type' => 'button',
        'value' => 'sendDebugLog',
        'help' => 'Öffnet eine neue E-Mail an die GREYHOUND Entwicklungsabteilung, in der das Fehlerprotokoll des Addons enthalten ist. Das Debug-Log darf nur über sichere E-Mail Verbindungen (SSL/TLS) versendet werden, da es personenbezogene Daten enthalten kann.'
      )
    );
  }

  /**
   * Liefert die Template-Variablen und ihre Bezeichnung / Kurzbeschreibung.
   *
   * @return array
   */
  public function getTemplateVariableDescriptions()
  {
    //TODO Alle nicht verfügbaren Variablen/Daten auskommentieren oder löschen:
    // Diese Liste bestimmt sowohl die Anzeige der Hilfe-Erweiterung für das Addon, als auch die Auswahlboxen
    // in den Addon-Einstellungen (Datenübernahme, Vorschauzeile).
    return array(
      'Order.OrderNumber' => 'Bestellnummer',
      'Order.InvoiceNumber' => 'Rechnungsnummer',
      'Order.CustomerNumber' => 'Kundennummer',
      'Order.Date' => 'Bestelldatum',
      'Order.Status' => 'Status des Auftrags',
      'Order.Platform' => 'Verkaufsplattform',
      'Order.PlatformUsername' => 'Benutzer der Verkaufsplattform',
      'Order.TotalAmount' => 'Gesamtbetrag des Auftrags',
      'Order.OpenAmount'=> 'Offener Betrag des Auftrags',
      'BillingAddress.Company' => 'Rechnungsadresse: Firma',
      'BillingAddress.Salutation' => 'Rechnungsadresse: Anrede',
      'BillingAddress.Sex.Number' => 'Rechnungsadresse: Geschlecht',
      'BillingAddress.Title' => 'Rechnungsadresse: Titel',
      'BillingAddress.Name' => 'Rechnungsadresse: Name',
      'BillingAddress.Firstname' => 'Rechnungsadresse: Vorname',
      'BillingAddress.Lastname' => 'Rechnungsadresse: Nachname',
      'BillingAddress.Letter.Identifier' => 'Rechnungsadresse: Adresse',
      'BillingAddress.Letter.Street' => 'Rechnungsadresse: Straße',
      'BillingAddress.Letter.AdditionalInfo' => 'Rechnungsadresse: Zusatzinfo',
      'BillingAddress.Letter.ZipCode' => 'Rechnungsadresse: Postleitzahl',
      'BillingAddress.Letter.City' => 'Rechnungsadresse: Stadt',
      'BillingAddress.Letter.Province' => 'Rechnungsadresse: Bundesstaat/Provinz',
      'BillingAddress.Letter.Country' => 'Rechnungsadresse: Land',
      'BillingAddress.Letter.Country.ISO2' => 'Rechnungsadresse: Ländercode',
      'BillingAddress.Email' => 'Rechnungsadresse: E-Mail Adresse',
      'BillingAddress.Phone' => 'Rechnungsadresse: Telefonnummer',
      'BillingAddress.Mobile' => 'Rechnungsadresse: Mobiltelefonnummer',
      'BillingAddress.Fax' => 'Rechnungsadresse: Faxnummer',
      'BillingAddress.Birthday' => 'Rechnungsadresse: Geburtstag',
      'ShippingAddress.Company' => 'Lieferadresse: Firma',
      'ShippingAddress.Salutation' => 'Lieferadresse: Anrede',
      'ShippingAddress.Sex.Number' => 'Lieferadresse: Geschlecht',
      'ShippingAddress.Title' => 'Lieferadresse: Titel',
      'ShippingAddress.Name' => 'Lieferadresse: Name',
      'ShippingAddress.Firstname' => 'Lieferadresse: Vorname',
      'ShippingAddress.Lastname' => 'Lieferadresse: Nachname',
      'ShippingAddress.Letter.Identifier' => 'Lieferadresse: Adresse',
      'ShippingAddress.Letter.Street' => 'Lieferadresse: Straße',
      'ShippingAddress.Letter.AdditionalInfo' => 'Lieferadresse: Zusatzinfo',
      'ShippingAddress.Letter.ZipCode' => 'Lieferadresse: Postleitzahl',
      'ShippingAddress.Letter.City' => 'Lieferadresse: Stadt',
      'ShippingAddress.Letter.Province' => 'Lieferadresse: Bundesstaat/Provinz',
      'ShippingAddress.Letter.Country' => 'Lieferadresse: Land',
      'ShippingAddress.Letter.Country.ISO2' => 'Lieferadresse: Ländercode',
      'ShippingAddress.Email' => 'Lieferadresse: E-Mail Adresse',
      'ShippingAddress.Phone' => 'Lieferadresse: Telefonnummer',
      'ShippingAddress.Mobile' => 'Lieferadresse: Mobiltelefonnummer',
      'ShippingAddress.Fax' => 'Lieferadresse: Faxnummer',
      'ShippingAddress.Birthday' => 'Lieferadresse: Geburtstag',
      'Payment.Method' => 'Zahlungsart',
      'Payment.Costs' => 'Kosten der Zahlungsart',
      'Payment.Date' => 'Zahlungsdatum',
      'Payment.TransactionID' => 'Transaktions-ID der Zahlung',
      'Payment.TermOfPayment' => 'Zahlungsziel',
      'Shipping.Provider' => 'Versanddienstleister',
      'Shipping.Costs' => 'Versandkosten',
      'Shipping.Date' => 'Versanddatum',
      'Shipping.Tracking.Code' => 'Sendungsnummer',
      'Shipping.Tracking.Codes' => 'Alle Sendungsnummern',
      'Shipping.Tracking.URL' => 'Sendungsverfolgungs-URL',
      'Shipping.Tracking.URLs' => 'Alle Sendungsverfolgungs-URLs',
      'Shipping.Tracking.Count' => 'Anzahl der Sendungsnummern',
      'Shipping.Tracking.Code1' => 'Erste Sendungsnummer',
      'Shipping.Tracking.URL1' => 'Erste Sendungsverfolgungs-URL',
      'Shipping.Tracking.Code2' => 'Zweite Sendungsnummer',
      'Shipping.Tracking.URL2' => 'Zweite Sendungsverfolgungs-URL',
      'Shipping.Tracking.Code3' => 'Dritte Sendungsnummer',
      'Shipping.Tracking.URL3' => 'Dritte Sendungsverfolgungs-URL'
    );
  }

  /**
   * Liefert den Pfad der Debug-Log Datei.
   *
   * @return string
   */
  public function getDebugLogFile()
  {
    return dirname(__FILE__) . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'debug.txt';
  }
}