<?php
/*
 * Example Connect
 *
 * Copyright (c) 2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of Example Connect and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Ergebnisobjekt für Example Aufrufe.
 *
 * @package GhExampleConnect
 * @subpackage core
 * @category Example Connect
 *
 * @author GREYHOUND Software GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2019 GREYHOUND Software GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhExampleData extends GhAddonData
{
  /**
   * App.
   * @var GhExampleAddon
   */
  protected $_addon;

  /**
   * Account.
   * @var GhExampleAccount
   */
  protected $_account;

  /**
   * Kunden.
   * @var GhAddonCustomer[]
   */
  protected $_customers;

  /**
   * Bestellungen.
   * @var GhAddonOrder[]
   */
  protected $_orders;


  /**
   * Konstruktor.
   *
   * @param GhExampleAddon $addon Addon
   * @param GhExampleAccount $account Account
   * @param array $data Daten eines Example Aufrufs
   */
  public function __construct($addon, $account, $data = null)
  {
    $this->_addon = $addon;
    $this->_account = $account;
    $this->_data = $data;
  }

  /**
   * Liefert alle Kunden dieser Abfrage.
   *
   * @return GhAddonCustomer[]
   */
  public function getCustomers()
  {
    if (!is_array($this->_customers)) {
      $this->_customers = array();

      if (is_array($this->_data)) {
        // Liste der Käufer aufbauen:
        foreach ($this->_data as $orderData) {
          //TODO Kunden-ID:
          $customerId = GhAddonData::safeGet($orderData, 'customerId');

          if (!isset($this->_customers[$customerId])) {
            //TODO Rechnungsanschrift des Kunden:
            $billingAddressData = $this->_getAddressData(GhAddonData::safeGet($orderData, 'billingAddress', 'object'), $orderData);

            $customer = new GhAddonCustomer();
            $customer->ID = $customerId;
            //TODO Falls Kunden->ID != Kundennummer, dann bei CustomerNumber die Kundennummer hinterlegen:
            $customer->CustomerNumber = $customer->ID;
            $customer->Name = GhAddonData::safeGet($billingAddressData, 'Name');
            $customer->Email = GhAddonData::safeGet($billingAddressData, 'Email');
            $customer->Address = GhCoreLocale::getAddressLines($billingAddressData);
            $customer->Data = $orderData;
            $customer->Description = implode('<br />', $customer->Address);
            $customer->Description .= '<br /><br /><table>';
            $customer->Description .= '<tr><td>Kundennr.:</td><td>' . $customerId;

            $customer->Description .= '</td></tr>';

            if ($customer->Email) {
              $customer->Description .= '<tr><td>E-Mail:</td><td>' . $customer->Email . '</td></tr>';
            }

            if ($customer->Phone) {
              $customer->Description .= '<tr><td>Telefon:</td><td>' . $customer->Phone . '</td></tr>';
            }

            $customer->Description .= '</table>';

            $this->_customers[$customerId] = $customer;
          }
        }

        usort($this->_customers, array($this, 'compareCustomers'));
      }
    }

    return $this->_customers;
  }

  /**
   * Liefert alle Bestellungen dieser Abfrage.
   *
   * @param string $customerId Kunden-ID, falls nur Bestellungen dieses Kunden geliefert werden sollen
   * @return GhAddonOrder[]
   */
  public function getOrders($customerId = null)
  {
    if (!is_array($this->_orders)) {
      $this->_orders = array();

      if (is_array($this->_data)) {
        // Liste der Bestellungen aufbauen:
        foreach ($this->_data as $orderData) {
          //TODO Auftrags-ID:
          $orderId = GhAddonData::safeGet($orderData, 'orderId');

          if (!isset($this->_orders[$orderId])) {
            $order = new GhAddonOrder();
            $this->_orders[] = $order;

            $order->ID = $orderId;
            //TODO Falls Auftrags-ID != Auftragsnummer, dann bei OrderNumber die Auftragsnummer hinterlegen:
            $order->OrderNumber = $order->ID;
            $order->Data = $orderData;
            $order->TaskNumberID = $order->ID;

            //TODO Rechnungsnummer:
            $order->InvoiceNumber = GhAddonData::safeGet($orderData, 'invoiceId');

            //TODO Kunden-ID:
            $order->CustomerID = GhAddonData::safeGet($orderData, 'customerId');
            //TODO Falls Kunden-ID != Kundennummer, dann bei CustomerNumber die Kundennummer hinterlegen:
            $order->CustomerNumber = $order->CustomerID;

            //TODO Auftragsdatum:
            $order->Date = $this->_getDate(GhAddonData::safeGet($orderData, 'date'));

            //TODO Gesamtbetrag:
            $order->TotalAmount = GhAddonData::safeGet($orderData, 'totalAmount', 'float');

            //TODO Offener / unbezahlter Betrag:
            $order->OpenAmount = GhAddonData::safeGet($orderData, 'openAmount', 'float');
            //$order->OpenAmount = $order->TotalAmount - GhAddonData::safeGet($orderData, 'paidAmount', 'float');

            //TODO Währung:
            $order->Currency = GhAddonData::safeGet($orderData, 'currency');

            if ($order->Currency == 'EUR') {
              $order->Currency = '€';
            } elseif ($order->Currency == 'USD') {
              $order->Currency = '$';
            }

            // OrderInfo befüllen:
            //TODO Rechnungsanschrift:
            $order->BillingAddress = $this->_getAddressData(GhAddonData::safeGet($orderData, 'billingAddress', 'object'), $orderData);

            $order->Name = $order->Date->getFormatted('d.m.Y') . ', Nr: ' . $order->OrderNumber;
            $order->Description = 'Bestellung ' . $order->OrderNumber . ' vom ' . $order->Date->getFormatted('d.m.Y');

            if ($order->InvoiceNumber) {
              $order->Name .= ', ReNr: ' . $order->InvoiceNumber;
              $order->Description .= '<br />ReNr: ' . $order->InvoiceNumber;
            }
        
            $order->Name .= ', ' . GhAddonData::safeGet($order, 'BillingAddress/Name');
            $order->Description .= '<br /><br />' . implode('<br />', GhCoreLocale::getAddressLines($order->BillingAddress));

            $order->Description .= '<br />';

            if ($order->TotalAmount) {
              $tmpOrderAmount = GhCoreLocale::formatCurrency($order->TotalAmount, $order->Currency);
              $order->Name .= ', ' . $tmpOrderAmount;
              $order->Description .= '<br />Rechnungsbetrag: ' . $tmpOrderAmount . '<br />';
            }
          }
        }

        usort($this->_orders, array($this, 'compareOrders'));
      }
    }

    // Aufträge aller Kunden:
    if (!$customerId) {
      return $this->_orders;
    }

    // Aufträge eines bestimmten Kunden:
    $orders = array();

    foreach ($this->_orders as $order) {
      if ($order->CustomerID == $customerId) {
        $orders[] = $order;
      }
    }

    return $orders;
  }

  /**
   * Liefert eine Bestellung anhand ihrer ID.
   *
   * @param string $orderId ID der Bestellung
   * @return GhAddonOrder
   */
  public function getOrder($orderId)
  {
    /* @var GhAddonOrder $order */
    $order = parent::getOrder($orderId);

    if (!$order || $order->HasDetails) {
      return $order;
    }

    $order->HasDetails = true;

    /* @var GhAddonOrder $order */
    $orderData = $order->Data;

    if (!$orderData) {
      return $order;
    }
    
    //TODO Rechnungsanschrift:
    $order->BillingAddress = $this->_getAddressData(GhAddonData::safeGet($orderData, 'billingAddress', 'object'), $orderData);
    //TODO Lieferanschrift:
    $order->ShippingAddress = $this->_getAddressData(GhAddonData::safeGet($orderData, 'shippingAddress', 'object'), $orderData);

    // Bestellnummer:
    $orderNumberHtml = $this->_getOrderNumberHtml($order);

    if ($orderNumberHtml) {
      $order->OrderInfo[] = (object)array('Icon' => 'shoppingcart', 'Caption' => 'Bestellnr.:', 'Value' => $orderNumberHtml);
    }

    // Rechnungsnummer:
    $invoiceNumberHtml = $this->_getInvoiceNumberHtml($order);

    if ($invoiceNumberHtml) {
      $order->OrderInfo[] = (object)array('Icon' => 'document_certificate', 'Caption' => 'Rechnung:', 'Value' => $invoiceNumberHtml);
    }

    // Bestelldatum:
    $order->OrderInfo[] = (object)array('Icon' => 'calendar', 'Caption' => 'Datum:', 'Value' => GhCoreStrings::attributeValue($order->Date->getFormatted('d.m.Y')));

    // Bestellstatus:
    //TODO Auftragsstatus:
    $order->Status = GhAddonData::safeGet($orderData, 'status');
    $order->OrderInfo[] = (object)array('Icon' => '', 'Caption' => 'Status:', 'Value' => GhCoreStrings::attributeValue($order->Status));

    // Kundennummer:
    $customerNumberHtml = $this->_getCustomerNumberHtml($order);

    if ($customerNumberHtml) {
      $order->OrderInfo[] = (object)array('Icon' => 'businessman', 'Caption' => 'Kunde:', 'Value' => $customerNumberHtml);
    }

    // Telefon, Fax and E-Mail:
    if ($order->BillingAddress['Phone'] && $order->BillingAddress['Phone'] != '-') {
      $fixedPhone = GhCoreLocale::getPhoneNumber($order->BillingAddress);

      if ($fixedPhone) {
        $phone = '<a href=\'callto:' . rawurlencode($fixedPhone) . '\'>' . $order->BillingAddress['Phone'] . '</a>';
      } else {
        $phone = $order->BillingAddress['Phone'];
      }

      $order->OrderInfo[] = (object)array('Icon' => 'telephone', 'Caption' => 'Telefon:', 'Value' => GhCoreStrings::attributeValue($phone));
    }

    if ($order->BillingAddress['Fax']) {
      $order->OrderInfo[] = (object)array('Icon' => 'fax', 'Caption' => 'Fax:', 'Value' => GhCoreStrings::attributeValue($order->BillingAddress['Fax']));
    }

    if ($order->BillingAddress['Email']) {
      $order->OrderInfo[] = (object)array('Icon' => 'mail', 'Caption' => 'E-Mail:', 'Value' => '<a href="#" onclick="ghAddonSidebarPage.commAction(\'newEmail\');return false;">' . $order->BillingAddress['Email'] . '</a>');
    }

    // Rechnungsadresse:
    $billingLines = GhCoreLocale::getAddressLines($order->BillingAddress);

    if (!empty($billingLines)) {
      $order->OrderInfo[] = (object)array('Icon' => 'mail2', 'Caption' => array('Rechnungs-', 'adresse:'), 'Value' => $billingLines);
    }

    // Lieferadresse:
    $shippingLines = GhCoreLocale::getAddressLines($order->ShippingAddress);

    if (!empty($shippingLines)) {
      $order->OrderInfo[] = (object)array('Icon' => 'house', 'Caption' => array('Liefer-', 'adresse:'), 'Value' => $shippingLines);
    }

    // Zahlung:
    $paymentInfo = array();
    $paymentAmount = GhCoreLocale::formatCurrency($order->TotalAmount, $order->Currency);

    if ($order->OpenAmount > 0) {
      $paymentAmount .= ' <span style="color:#880000;">(' . GhCoreLocale::formatCurrency($order->OpenAmount, $order->Currency) . ' offen)</span>';
    } elseif($order->OpenAmount < 0) {
      $paymentAmount .= ' <span style="color:#880000;">(' . GhCoreLocale::formatCurrency(-$order->OpenAmount, $order->Currency) . ' zu viel bezahlt)</span>';
    } else {
      $paymentAmount .= ' <span style="color:#008800;">(bezahlt)</span>';
    }

    $paymentInfo[] = $paymentAmount;
    
    //TODO Zahungsart:
    $order->PaymentMethod = trim(GhAddonData::safeGet($orderData, 'paymentMethod'));

    if ($order->PaymentMethod) {
      $paymentInfo[] = $order->PaymentMethod;
    }

    //TODO Zahlungsziel:
    if (trim(GhAddonData::safeGet($orderData, 'dueDate'))) {
      $paymentInfo[] = trim(GhAddonData::safeGet($orderData, 'dueDate'));
    }

    $order->OrderInfo[] = (object)array('Icon' => 'currency_euro', 'Caption' => 'Zahlung:', 'Value' => $paymentInfo);

    // Versand:
    //TODO Versandkosten:
    $order->ShippingCosts = GhAddonData::safeGet($orderData, 'shippingCost', 'float');

    $shippingInfo = array(GhCoreLocale::formatCurrency($order->ShippingCosts, $order->Currency));

    //TODO Versanddienstleister:
    $order->ShippingProvider = GhAddonData::safeGet($orderData, 'shippingProvider');

    if ($order->ShippingProvider) {
      $shippingInfo[] = $order->ShippingProvider;
    }

    $order->OrderInfo[] = (object)array('Icon' => 'package', 'Caption' => 'Versand:', 'Value' => $shippingInfo);

    $trackingInfo = array();

    //TODO Sendungsnummern / Tracking:
    foreach (GhAddonData::getArray($orderData, 'tracking') as $package) {
      $packageNumber = GhAddonData::safeGet($package, 'trackingId');
      $shippingDate = $this->_getDate(GhAddonData::safeGet($package, 'shippingDate'));

      $trackingUrl = $this->_addon->getTrackingUrl($packageNumber, $order->ShippingProvider, GhAddonData::safeGet($order, 'ShippingAddress/Letter.ZipCode') ? GhAddonData::safeGet($order, 'ShippingAddress/Letter.ZipCode') : GhAddonData::safeGet($order, 'BillingAddress/Letter.ZipCode'));
      $order->Tracking[] = array('Code' => $packageNumber, 'Url' => $trackingUrl, 'Provider' => $order->ShippingProvider, 'Date' => $shippingDate);

      if ($trackingUrl) {
        $str = '<a href=\'' . $this->_addon->getBrowserLink($trackingUrl) . '\'>' . $packageNumber . '</a>';
      } else {
        $str = $packageNumber;
      }
      
      if ($shippingDate) {
        $str .= ' (' . $shippingDate->getFormatted('d.m.Y') . ')';
      }
      
      $trackingInfo[] = $str;
    }

    if (!empty($trackingInfo)) {
      $order->OrderInfo[] = (object)array('Icon' => '', 'Caption' => 'Sendungsnr.:', 'Value' => $trackingInfo);
    }

    // Kurzbeschreibung und Langbeschreibung (für Listen/Tooltips):
    $order->Name = $order->Date->getFormatted('d.m.Y') . ', Nr: ' . $order->OrderNumber;
    $order->Description = 'Bestellung ' . $order->OrderNumber . ' vom ' . $order->Date->getFormatted('d.m.Y');

    if ($order->InvoiceNumber) {
      $order->Name .= ', ReNr: ' . $order->InvoiceNumber;
      $order->Description .= '<br />ReNr: ' . $order->InvoiceNumber;
    }

    $order->Name .= ', ' . GhAddonData::safeGet($order->BillingAddress, 'Name');
    $order->Description .= '<br /><br />' . implode('<br />', GhCoreLocale::getAddressLines($order->BillingAddress));

    $order->Description .= '<br />';

    if ($order->TotalAmount) {
      $tmpOrderAmount = GhCoreLocale::formatCurrency($order->TotalAmount, $order->Currency);
      $order->Name .= ', ' . $tmpOrderAmount;
      $order->Description .= '<br />Rechnungsbetrag: ' . $tmpOrderAmount . '<br />';
    }

    // OrderTextInfo befüllen:

    // Kommentar:
    //TODO Anmerkungen/Kommentare:
    $comment = trim(GhAddonData::safeGet($orderData, 'comments'));

    if ($comment) {
      $order->OrderTextInfo[] = (object)array(
        'Icon' => 'message',
        'Caption' => 'Kommentar:',
        'Value' => $comment
      );
    }

    // Bestellte Artikel / Auftragspositionen:
    //TODO Bestellpositionen:
    foreach (GhAddonData::getArray($orderData, 'items') as $orderItemData) {
      $orderItem = new GhAddonOrderItem();
      $order->OrderItems[] = $orderItem;
      //TODO Artikel-ID:
      $orderItem->ID = GhAddonData::safeGet($orderItemData, 'itemId');
      //TODO Falls Artikel->ID != Artikelnummer, dann bei ItemNumber die Artikelnummer hinterlegen:
      $orderItem->ItemNumber = $orderItem->ID;
      //TODO Artikelname/-bezeichnung:
      $orderItem->Name = GhCoreStrings::attributeValue('<span title=\'' . trim(str_replace('\'', '\\\'', GhAddonData::safeGet($orderItemData, 'name'))) . '\'>' . GhAddonData::safeGet($orderItemData, 'name') . '</span>');
      //TODO Anzahl:
      $orderItem->Quantity = GhAddonData::safeGet($orderItemData, 'quantity', 'float');
      //TODO Einzelpreis:
      $orderItem->UnitPrice = GhAddonData::safeGet($orderItemData, 'unitPrice', 'float');
      //TODO Gesamtpreis der Position (Einzelpreis x Anzahl):
      $orderItem->TotalPrice = GhAddonData::safeGet($orderItemData, 'totalAmount', 'float');
      //$orderItem->TotalPrice = $orderItem->Quantity * $orderItem->UnitPrice;
      //TODO Währung der Bestellposition:
      $orderItem->Currency = $order->Currency;

      $productNumberHtml = $this->_getProductNumberHtml($orderItem, $order);

      if ($productNumberHtml) {
        $orderItem->Description[] = 'Anr: ' . $productNumberHtml;
      }

      $order->Description .= '<br />' . $orderItem->Quantity . ' x ' . $orderItem->Name;
    }

    // Bestellung merken:
    foreach ($this->_orders as $key => $tmpOrder) {
      if ($tmpOrder->ID == $order->ID) {
        $this->_orders[$key] = $order;
        break;
      }
    }

    return $order;
  }

  /**
   * Liefert Adressdaten in einem einheitlichen Format.
   *
   * @param array $addressData Adressdaten
   * @param array $orderData Auftragsdaten
   * @return array
   */
  protected function _getAddressData($addressData, $orderData)
  {
    $data = GhCoreLocale::getEmptyAddress();

    //TODO Adressdaten:
    $data['Company'] = trim(GhAddonData::safeGet($addressData, 'company'));
    $data['Salutation'] = trim(GhAddonData::safeGet($addressData, 'salutation'));
    $data['Title'] = trim(GhAddonData::safeGet($addressData, 'title'));
    $data['Firstname'] = trim(GhAddonData::safeGet($addressData, 'firstName'));
    $data['Lastname'] = trim(GhAddonData::safeGet($addressData, 'lastName'));
    $data['Letter.Street'] = trim(GhAddonData::safeGet($addressData, 'street') . ' ' . GhAddonData::safeGet($addressData, 'houseNo'));
    $data['Letter.AdditionalInfo'] = trim(GhAddonData::safeGet($addressData, 'additionalInfo'));
    $data['Letter.ZipCode'] = trim(GhAddonData::safeGet($addressData, 'postalCode'));
    $data['Letter.City'] = trim(GhAddonData::safeGet($addressData, 'city'));
    $data['Letter.Province'] = trim(GhAddonData::safeGet($addressData, 'province'));
    $data['Letter.CountryInfo.ISO2'] = trim(GhAddonData::safeGet($addressData, 'countryCode'));
    $data['Email'] = GhAddonData::safeGet($addressData, 'email');
    $data['Phone'] = GhAddonData::safeGet($addressData, 'phone');
    $data['Fax'] = GhAddonData::safeGet($addressData, 'fax');
    $data['Mobile'] = GhAddonData::safeGet($addressData, 'mobile');

    switch ($data['Salutation']) {
      case 'Frau':
      case 'Miss':
      case 'Mrs':
        $data['Sex.Number'] = GhCoreLocale::SEX_FEMALE;
        break;
      
      case 'Herr':
      case 'Mr':
        $data['Sex.Number'] = GhCoreLocale::SEX_MALE;
        break;
    }

    $data['Birthday'] = new GhRpcDateTime(GhAddonData::safeGet($addressData, 'birthday'));
  
    $data = GhCoreLocale::fixAddress($data);

    return $data;
  }
}