/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

// Kompatiblität mit älteren Browsern:
if(!Array.isArray) { Array.isArray = function(arg) { return Object.prototype.toString.call(arg) === '[object Array]'; }; }

// Enums
var RpcItemState = {
  isOpen: 0,
  isNew: 1,
  isQuestion: 2,
  isAnswer: 3,
  isDone: 4,
  isForward: 5,
  isDraft: 6
};

var RpcItemKind = {
  ikEmail: 0,
  ikFax: 1,
  ikLetter: 2,
  ikShortMessage: 3,
  ikCall: 4,
  ikAppointment: 5,
  ikTask: 6,
  ikNote: 7,
  ikContact: 8,
  ikFile: 9
};

var RpcItemFlag = {
  ifRead: 1,
  ifDeleted: 2,
  ifEscalated: 4,
  ifSpam: 8,
  ifFailure: 16,
  ifWaiting: 32,
  ifUnsent: 64,
  ifAttachment: 128,
  ifWholeday: 256,
  ifRecurrence: 512,
  ifUnclassifiable: 1024,
  ifProtocol: 2048,
  ifRemarks: 4096,
  ifUseHtml: 8192,
  ifEml: 16384,
  ifDeliveryNotification: 32768
};

var ghSortOrder = {
  descendingId: -3,
  descendingAndId: -2,
  descending: -1,
  none: 0,
  ascending: 1,
  ascendingAndId: 2,
  ascendingId: 3
};

// Hilfsfunktionen

// Liefert CrossBrowser XMLHttpRequest zurück
function getXHR(a) {for(a=0;a<4;a++)try{return a?new ActiveXObject([,"Msxml2","Msxml3","Microsoft"][a]+".XMLHTTP"):new XMLHttpRequest}catch(e){}};

// Wrapper für Debug-Ausgaben
function debug(obj) {try { console.log(arguments); } catch(e) {}};

// Extrahiert Key/Value aus Hash
function getHashValue(key, _default) {
  var v = location.hash.match(key+'=([^&|$]+)');
  return (v===null) ? (_default||'') : decodeURIComponent(v.pop());
};

// Trimfunktion
function trim(txt) {
  return txt ? txt.replace(/^[\s\uFEFF]+|[\s\uFEFF]+$/g,'') : txt;
};

// Entfernt den 'px'-Suffix aus einem String
function trimPx(str) {
  var l = str.length;

  if(l >= 2 && str[l - 2].toLowerCase() == 'p' && str[l - 1].toLowerCase() == 'x')
    return trim(str.substr(0, l - 2));
  else
   return str;
};

// Array indexOf für IE <= 8:
if (!Array.prototype.indexOf) {
  Array.prototype.indexOf = function(obj, start) {
    var i;

    for (i = start || 0; i < this.length; i++) {
      if (this[i] === obj) {
        return i;
      }
    }

    return -1;
  };
}

function addCssClass(obj, className) {
  if (obj && ((' ' + obj.className + ' ').indexOf(' ' + className + ' ') == -1)) {
      obj.className = trim(obj.className + ' ' + className);
  }
}

function removeCssClass(obj, className) {
  var s;

  if (obj) {
    s = trim((' ' + obj.className + ' ').replace(' ' + className + ' ', ' '));

    if (obj.className != s) {
      obj.className = s;
    }
  }
}

function normalizePhoneNumber(number) {
  number = number || '';

  if(number.substr(0, 1) == '+')
    number = '+' + number.substr(1).replace(/[^0-9]/g, '');
  else if(number.substr(0, 2) == '00')
    number = '+' + number.substr(2).replace(/[^0-9]/g, '');
  else {
    number = number.replace(/[^0-9]/g, '');

    if(number.length > 0 && number.substr(0, 1) == '0')
      number = '+49' + number.substr(1);
  }

  return number;
};

// Funktion zum sicheren escapen
function htmlspecialchars(v) {
  return v ? (v.toString()||'').replace(/[&<>]/g, function (c) { return "&" + {"&": "amp", "<": "lt", ">": "gt"}[c] + ";" }) : '';
};

function createJsonError(message) {
  var code = -32700;

  if(arguments.length > 1)
    code = arguments[1];

  return {"jsonrpc": "2.0", 'success': false, 'result': null, 'error': {'code': code, 'message': message}};
};

// Hash in Objekt wandeln:
function extractHashValues() {
  var hash = {};
  var str = location.hash;

  if(str && str[0]=="#")
    str = str.substr(1);

  var arr = str.split("&");

  while(arr.length) {
    var tmp = arr.shift().split("=",2);
    var key = tmp[0];

    if(key.substr(key.length - 2) == "[]") // Als array behandeln!
    {
      key = key.substr(0, key.length - 2);

      if(!hash[key])
        hash[key] = [];

      hash[key].push( decodeURIComponent(tmp[1]) );
    }
    else
    {
      hash[key] = decodeURIComponent(tmp[1]);
    }
  }

  return hash;
};

// Funktion um default-optionen zu setzen
function createOptions(a,b,c) {
  var k, r = {};
  if(a)for(k in a) r[k]=a[k];
  if(b)for(k in b) r[k]=b[k];
  if(c)for(k in c) r[k]=c[k];
  return r;
};

function addEvent(obj, evt, fnc) {
 if (obj.addEventListener) {
    obj.addEventListener(evt, fnc, false);
    return true;
  }
  // Microsoft model
  else if (obj.attachEvent) {
    return obj.attachEvent('on' + evt, fnc);
  } 
};

function removeEvent(obj, evt, fnc) {
 if (obj.removeEventListener) 
    obj.removeEventListener (evt,fnc,false);
 if (obj.detachEvent)
    obj.detachEvent ('on'+evt,fnc); 
}

function createInput($parent, options) {
  var $wrapper, $border, $input, $placeholder, option;
  options = options || {};

  $wrapper = document.createElement('DIV');
  $wrapper.className = "ghInput";
  $border = document.createElement('DIV');
  $border.className = "border";
  $input = document.createElement('INPUT');
  addEvent($input, 'focus', function() {
    addCssClass($wrapper, 'ghFocus');
  });
  addEvent($input, 'blur', function() {
    removeCssClass($wrapper, 'ghFocus');
  });

  // IE hat erst ab 10 "placeholder"; Wir kümmern uns daher selbst darum
  if (options.placeholder) {
    $placeholder = document.createElement('span');
    $placeholder.className = "placeholder";
    $placeholder.innerHTML = options.placeholder || '';
    delete(options.placeholder);
    $border.appendChild($placeholder);
    $input.checkPlaceholder = function() { $placeholder.style.display = $input.value ? 'none' : ''; };
    addEvent($input, 'focus', function() { $placeholder.style.display = 'none'; });
    addEvent($input, 'blur', function() { if ($input.value=='') $placeholder.style.display=''; });
  } else {
    $input.checkPlaceholder = function(){};
  }
  
  if (!options['type']) {
    options['type'] = 'text';
  }

  for (option in options) {
    $input[option] = options[option];
  }

  $input.checkPlaceholder();

  $wrapper.appendChild($border);
  $border.appendChild($input);
  $parent.appendChild($wrapper);

  return $input;
};

function ghText($container, options) {
  this.getText = getText;
  this.setText = setText;

  this.domNode = null;

  var _self = this;

  (function constructor() {
    _self.domNode = document.createElement('DIV');
    _self.domNode.className = 'ghText';

    if (options.className) {
      _self.domNode.className += ' ' + options.className;
    }

    if (options.text) {
      _self.domNode.innerHTML = options.text;
    }

    $container.appendChild(_self.domNode);
  })();

  function getText() {
    return _self.domNode.innerHTML;
  }

  function setText(text) {
    _self.domNode.innerHTML = text;
  }
}

function ghTable($container, options) {
  this.addCell = addCell;
  this.domNode = null;
  this.columns = [{}, {}];
  this.components = {};
  this.currentRowNode = null;
  this.currentCellNode = null;

  var _self = this;
  var _currentColumnIndex = 0;

  (function constructor() {
    _self.domNode = document.createElement('TABLE');
    _self.domNode.className = 'ghTable';

    if (options.className) {
      _self.domNode.className += ' ' + options.className;
    }

    if (options.columns) {
      _self.columns = options.columns;
    }

    $container.appendChild(_self.domNode);
  })();

  function addCell() {
    if (!_self.currentRowNode || _currentColumnIndex >= _self.columns.length - 1) {
      _self.currentRowNode = document.createElement('TR');
      _self.domNode.appendChild(_self.currentRowNode);
      _currentColumnIndex = 0;
    } else {
      _currentColumnIndex++;
    }

    _self.currentCellNode = document.createElement('TD');
    _self.currentRowNode.appendChild(_self.currentCellNode);

    return _self.currentCellNode;
  }
}

function ghInput($container, options) {
  this.focus = focus;
  this.setValue = setValue;
  this.getValue = getValue;
  this.setEnabled = setEnabled;
  this.setTitle = setTitle;
  this.setVisible = setVisible;

  this.domNode = null;
  this.inputNode = null;
  this.onValueChange = null;

  var _self = this;
  var _enabled = true;
  var _visible = true;
  var _$border;
  var _$placeholder;

  (function constructor() {
    options = createOptions({
      type: 'text',
      value: '',
      domAttributes: {}
    }, options);

    _self.domNode = document.createElement('DIV');
    _self.domNode.className = 'ghInput';
    _$border = document.createElement('DIV');
    _$border.className = 'border';
    _self.inputNode = document.createElement('INPUT');

    if (options.className) {
      _self.domNode.className += ' ' + options.className;
    }

    if (options.title) {
      _self.domNode.title = options.title;
    }

    // IE hat erst ab Version 10 "placeholder"; Wir kümmern uns daher selbst darum:
    if (options.placeholder) {
      _$placeholder = document.createElement('span');
      _$placeholder.className = 'placeholder';
      _$placeholder.innerHTML = options.placeholder || '';
      _$border.appendChild(_$placeholder);
      _self.inputNode.checkPlaceholder = function() { _$placeholder.style.display = (_self.inputNode.value ? 'none' : 'inline'); };
      addEvent(_$placeholder, 'click', function() {
        _$placeholder.style.display = 'none';

        if (_self.inputNode.focus) {
          _self.inputNode.focus();
        }
      });
    } else {
      _self.inputNode.checkPlaceholder = function(){};
    }

    addEvent(_self.inputNode, 'focus', function() {
      addCssClass(_self.domNode, 'ghFocus');

      if (_$placeholder) {
        _$placeholder.style.display = 'none';
      }

      if (options.selectOnFocus) {
        _self.inputNode.select();
      }
    });

    addEvent(_self.inputNode, 'blur', function() {
      removeCssClass(_self.domNode, 'ghFocus');

      if (_$placeholder && _self.inputNode.value == '') {
        _$placeholder.style.display = 'inline';
      }
    });

    if (options.disabled) {
      setEnabled(false);
    }

    _self.inputNode.setAttribute('type', options.type);

    for (var option in options.domAttributes) {
      _self.inputNode.setAttribute(option, options.domAttributes[option]);
    }

    _self.inputNode.setAttribute('value', options.value);

    _self.inputNode.checkPlaceholder();

    _self.domNode.appendChild(_$border);
    _$border.appendChild(_self.inputNode);
    $container.appendChild(_self.domNode);

    addEvent(_self.inputNode, 'input', onValueChange);
    addEvent(_self.inputNode, 'propertychange', onValueChange);
    addEvent(_self.inputNode, 'change', onValueChange);

    if (options.onValueChange) {
      _self.onValueChange = options.onValueChange;
    }
  })();

  function setValue(value) {
    _self.inputNode.value = value;
    _self.inputNode.checkPlaceholder();
    onValueChange();
  };

  function getValue() {
    return _self.inputNode.value;
  };

  function setEnabled(enabled) {
    if(enabled != _enabled) {
      _enabled = enabled;
      _self.domNode.className = (_self.domNode.className || '').replace(/\s?disabled\s?/, '') + (enabled ? '' : ' disabled');
      _self.inputNode.readOnly = !enabled;
    }
  };

  function setVisible(visible) {
    if(visible != _visible) {
      _visible = visible;
      _self.domNode.style.display = (visible ? 'block' : 'none');
    }
  };

  function setTitle(title) {
    _self.domNode.title = title;
  }

  function focus() {
    _self.inputNode.focus();
  }

  function onValueChange() {
    if(_enabled && _self.onValueChange) {
      _self.onValueChange(_self.inputNode.value);
    }
  };
};

// Textarea
function ghTextArea($container, options) {
  this.focus = focus;
  this.setValue = setValue;
  this.getValue = getValue;
  this.setEnabled = setEnabled;
  this.setTitle = setTitle;
  this.setVisible = setVisible;

  this.domNode = null;
  this.textAreaNode = null;
  this.onValueChange = null;

  var _self = this;
  var _enabled = true;
  var _visible = true;
  var _$placeholder;

  (function constructor() {
    options = createOptions({
      type: 'text',
      value: '',
      domAttributes: {}
    }, options);

    _self.domNode = document.createElement('DIV');
    _self.domNode.className = 'textarea' + (options.className ? ' ' + options.className : '');
    _self.textAreaNode = document.createElement('TEXTAREA');

    if (options.title) {
      _self.domNode.title = options.title;
    }

    // IE hat erst ab Version 10 "placeholder"; Wir kümmern uns daher selbst darum:
    if (options.placeholder) {
      _$placeholder = document.createElement('span');
      _$placeholder.className = 'placeholder';
      _$placeholder.innerHTML = options.placeholder || '';
      _self.domNode.appendChild(_$placeholder);
      _self.textAreaNode.checkPlaceholder = function() { _$placeholder.style.display = (_self.textAreaNode.value ? 'none' : 'inline'); };
    } else {
      _self.textAreaNode.checkPlaceholder = function(){};
    }

    addEvent(_self.textAreaNode, 'focus', function() {
      addCssClass(_self.domNode, 'ghFocus');

      if (_$placeholder) {
        _$placeholder.style.display = 'none';
      }

      if (options.selectOnFocus) {
        _self.textAreaNode.select();
      }
    });

    addEvent(_self.textAreaNode, 'blur', function() {
      removeCssClass(_self.domNode, 'ghFocus');

      if (_$placeholder && _self.textAreaNode.value == '') {
        _$placeholder.style.display = 'inline';
      }
    });

    if (options.disabled) {
      setEnabled(false);
    }

    for (var option in options.domAttributes) {
      _self.textAreaNode.setAttribute(option, options.domAttributes[option]);
    }

    _self.textAreaNode.setAttribute('value', options.value);

    _self.textAreaNode.checkPlaceholder();

    _self.domNode.appendChild(_self.textAreaNode);
    $container.appendChild(_self.domNode);

    addEvent(_self.textAreaNode, 'input', onValueChange);
    addEvent(_self.textAreaNode, 'propertychange', onValueChange);
    addEvent(_self.textAreaNode, 'change', onValueChange);

    if (options.onValueChange) {
      _self.onValueChange = options.onValueChange;
    }
  })();

  function setValue(value) {
    _self.textAreaNode.setAttribute('value', value);
    _self.textAreaNode.checkPlaceholder();
    onValueChange();
  };

  function getValue() {
    return _self.textAreaNode.value;
  };

  function setEnabled(enabled) {
    if(enabled != _enabled) {
      _enabled = enabled;
      _self.domNode.className = (_self.domNode.className || '').replace(/\s?disabled\s?/, '') + (enabled ? '' : ' disabled');
      _self.textAreaNode.readOnly = !enabled;
    }
  };

  function setVisible(visible) {
    if(visible != _visible) {
      _visible = visible;
      _self.domNode.style.display = (visible ? 'block' : 'none');
    }
  };

  function setTitle(title) {
    _self.domNode.title = title;
  }

  function focus() {
    _self.textAreaNode.focus();
  }

  function onValueChange() {
    if(_enabled && _self.onValueChange)
      _self.onValueChange(_self.textAreaNode.value);
  };
}

// Checkbox-Control
function ghCheckbox($container, parent, options) {
  this.setChecked = setChecked;
  this.isChecked = isChecked;
  this.getId = getId;

  this.onChange = null;
  this.domNode = null;

  var _self = this;
  var _checked = false;
  var _$checkbox;
  var _$input;
  var _$image;

  (function constructor() {
    options = createOptions({
      checked: false
    }, options);

    _checked = options.checked;

    if (options.onChange) {
      _self.onChange = options.onChange;
    }

    _$checkbox = document.createElement('DIV');
    _self.domNode = _$checkbox;
    _$checkbox.className = 'ghCheckbox' + (_checked ? ' ghChecked' : '');
    $container.appendChild(_$checkbox);
    _$input = document.createElement('INPUT');
    _$input.setAttribute('type', 'checkbox');

    if (options.id) {
      _$input.setAttribute('id', options.id);
    }

    if (_checked) {
      _$input.setAttribute('checked', 'checked');
    }

    _$input.style.display = 'none';
    addEvent(_$input, 'change', function() {
      setChecked(this.checked);
    });
    _$checkbox.appendChild(_$input);
    _$image = document.createElement('LABEL');
    _$image.setAttribute('tabindex', '0');
    // Kein "for" Attribut für das Label, da die Events explizit abgehandelt werden (sonst funktioniert die Checkbox im IE 8 nicht).
    addEvent(_$image, 'keypress', function(e) {
      if (e.keyCode == 32) {
        _$input.checked = _$input.checked ? false : true;
        setChecked(_$input.checked);
      }
    });
    addEvent(_$image, 'click', function() {
      _$input.checked = _$input.checked ? false : true;
      setChecked(_$input.checked);
    });
    addEvent(_$image, 'focus', function() {
      addCssClass(_$checkbox, 'ghFocus');
    });
    addEvent(_$image, 'blur', function() {
      removeCssClass(_$checkbox, 'ghFocus');
    });
    _$checkbox.appendChild(_$image);
  })();

  function setChecked(checked) {
    _checked = !!checked;

    if (_checked) {
      addCssClass(_$checkbox, 'ghChecked');
    } else {
      removeCssClass(_$checkbox, 'ghChecked');
    }

    if (typeof _self.onChange == 'function') {
      _self.onChange(_checked);
    }
  }

  function isChecked() {
    return _checked;
  }

  function getId() {
    return _$checkbox.id;
  }
}

function ghButton($container, options) {
  this.domNode = null;
  this.onClick = function(){};
  this.click = click;
  this.setEnabled = setEnabled;
  this.setVisible = setVisible;
  this.setCaption = setCaption;
  this.setIcon = setIcon;

  var _self = this;
  var _enabled = true;
  var _visible = true;
  var _$icon;

  (function constructor() {
    options = createOptions({}, options);

    _self.domNode = document.createElement('DIV');
    _self.domNode.className = 'button' + (options.className ? ' ' + options.className : '');
    $container.appendChild(_self.domNode);

    if (options.caption) {
      setCaption(options.caption);
    }

    _$icon = document.createElement('SPAN');
    _self.domNode.appendChild(_$icon);

    _self.setIcon(options.icon);

    if (options.disabled) {
      setEnabled(false);
    }

    if (options.onClick) {
      _self.onClick = options.onClick;
    } else if (options.onclick) {
      _self.onClick = options.onclick;
    }

    addEvent(_self.domNode, 'click', click);
  })();

  function click() {
    if (_enabled && _self.onClick) {
      _self.onClick();
    }
  };

  function setEnabled(enabled) {
    if(enabled != _enabled) {
      _enabled = enabled;
      _self.domNode.className = (_self.domNode.className || '').replace(/\s?disabled\s?/, '') + (enabled ? '' : ' disabled');
    }
  };

  function setVisible(visible) {
    if(visible != _visible) {
      _visible = visible;
      _self.domNode.style.display = (visible ? 'inline-block' : 'none');
    }
  };

  function setCaption(caption) {
    _self.domNode.innerHTML = caption;
  };

  function setIcon(icon) {
    if(icon) {
      _$icon.className = 'icon ghGuiDefaultIcons ' + icon;
      _$icon.style.display = 'inline';
    }
    else {
      _$icon.style.display = 'none';
      _$icon.className = '';
    }
  };
};

// Liefert Position und Grösse eines Elements innerhalb eines Fensters. Berücksichtigt Scrollpositionen.
function axGetRect(obj) {
    if(!obj) {return null;}
    var curleft = 0;
    var curtop  = 0;
    var curwidth=obj.offsetWidth;
    var curheight=obj.offsetHeight;
    if (obj.offsetParent) {
        curleft = obj.offsetLeft - obj.scrollLeft;
        curtop = obj.offsetTop - obj.scrollTop;
        while (obj = obj.offsetParent) {
            curleft += obj.offsetLeft - obj.scrollLeft;
            curtop += obj.offsetTop - obj.scrollTop;
        }
    }
    return {left: curleft, top: curtop, width: curwidth, height: curheight};
};


// ghTooltip
// $element = DOM-Element auf der Tooltip gebunden werden soll.
function ghTooltip( $element, content, options ) {
  options = options || {};
  options.openmode = options.openmode || 'bottom';

  this.close = hide;

  var _self = this;
  var _count = 0, hOpener;
  var _clicked;
  var _$tooltip, _$arrow, _$content, _$contentWrapper;

  (function constructor() {
    var i, arr;
    _$tooltip = document.createElement('DIV');
    _$tooltip.className = 'ghTooltip ' + options.openmode;
    _$arrow = document.createElement('DIV');
    _$arrow.className = 'arrow';
    _$contentWrapper = document.createElement('DIV');
    _$contentWrapper.className = 'contentWrapper';
    _$content = document.createElement('DIV');
    _$content.className = 'content';
    _$tooltip.appendChild(_$arrow);
    _$tooltip.appendChild(_$contentWrapper);
    _$contentWrapper.appendChild(_$content);

    $element.onmouseover = mouseover;
    $element.onmouseout = mouseout;
    $element.onclick = onClick;
    _$content.onclick = onClick;
  
    // Content bearbeiten
    if(Array.isArray(content)) {
      arr = content;
      content = '';

      for(i = 0; i < content.length; i++)
        content += arr[i] + '<br>';
    }

    if(content) {
      content = content.replace(/\<style[\s\S]*\<\/style[\s\S]*?\>/igm, '');  // stylesheets entfernen falls vorhanden
      // Falls <body> dann nur content dazwischen
      content = trim((content.match(/\<body[\s\S]*?\>([\s\S]*)\<\/body/i) || []).pop() || content);
    }
  })();

  function show() {
    var c, e, i, mid, lft, offset;
    _$content.innerHTML = trim(content);

    if(!content || !trim(content.replace(/\<br\>/igm,'\n').replace(/&nbsp;/g," ").replace(/\<[^>]*\>/g,'')))
      return;

    document.body.appendChild( _$tooltip );

    // Position setzen:
    _$tooltip.style.left  = "5px";
    _$tooltip.style.right = "5px";
    _$tooltip.style.top   = "0px";
    _$tooltip.style.maxWidth = document.body.offsetWidth - 20 + "px";              
    c = axGetRect( (options.item ? options.item : $element) );
    e = _$contentWrapper.offsetWidth;
    i = c.left + c.width / 2 - e / 2;
    _$tooltip.style.right = "";

    switch (options.openmode) {
        case "top":
            mid = c.left + c.width/2;
            lft = Math.max(10, _$arrow.offsetWidth/2 + (mid - _$tooltip.offsetWidth/2) - Math.max(0, mid + _$tooltip.offsetWidth/2 - e));
            _$tooltip.style.left = lft + "px";
            _$tooltip.style.top = c.top - _$tooltip.offsetHeight + "px";
            break;

        case "bottom":
            mid = c.left + c.width/2;
            lft = Math.max(10, _$arrow.offsetWidth/2 + (mid - _$tooltip.offsetWidth/2) - Math.max(0, mid + _$tooltip.offsetWidth/2 - e));
            _$tooltip.style.left = lft + "px";
            _$arrow.style.left = c.left - parseInt(_$tooltip.style.left,10) + c.width/2 + "px";
            offset = 3 + c.top + c.height;
            _$tooltip.style.top = offset + "px";
            _$tooltip.style.height = (document.body.clientHeight - offset - 25) + 'px';
            break;
       
        case "left":
            _$tooltip.style.left = Math.max(0, c.left - e) - _$arrow.offsetWidth + "px";
            _$tooltip.style.top = c.top + c.height / 2 - _$tooltip.offsetHeight / 2 + "px";
            break;

        case "right":
            _$tooltip.style.left = c.left + c.width + "px";
            _$tooltip.style.top = c.top + (c.height / 2) - (_$tooltip.offsetHeight / 2) + "px";
            break;
    }
  }

  function hide() {
    clearTimeout(hOpener);
    _$content.innerHTML = '';
    try { _$tooltip.parentNode.removeChild( _$tooltip ) } catch(e) {};
  }

  function mouseover() {
    _count++;
    clearTimeout(hOpener);
    hOpener = setTimeout(checkOpen, 500);
  }

  function mouseout() {
    _count--;
    clearTimeout(hOpener);
    hOpener = setTimeout(checkOpen, 50);
  }

  function onClick() {
    if (_clicked) {
      _clicked = false;
      hide();
    } else {
      _clicked = true;
      show();
    }
  }

  function checkOpen() {
    if (!_clicked) {
      return _count ? show() : hide();
    }
  }
};

// Order-Details-Renderer
function ghOrderDetails($container) {
  
  this.clear       = clear;
  this.render      = render;
  this.addEntry    = addEntry;
  this.addColumn   = addColumn;
  this.getHeight   = getHeight;
  this.setColumnLabel = setColumnLabel;

  var _self = this;
  var _columns = [];
  var _rowdata = {};
  var _$table;
  var _$tableHead;
  var _$tableBody;
  var _tooltips = [];
  var _tooltipObjs = [];

  function clear() {
    $container.innerHTML = '';
    for(var i=0; i<_columns.length;i++)
    {
      _columns[i]['$column'].innerHTML = '';
      _columns[i]['data'] = [];
      _columns[i]['left'] = 0;
      _columns[i]['html'] = [];
    }

    _tooltips = [];
    _tooltipObjs = [];
  };

  function getHeight() {
    return $container.offsetHeight;
  };

  // Design der Spalten wird hier abgelegt.
  function addColumn( key, obj ) {
    var row = {
      '$column':  document.createElement('DIV'),
      'key':      key,
      'width':    obj.width || 'auto',
      'fixed':    obj.fixed || false,
      'type':     obj.type  || 'string',
      'label':    obj.label || '',
      'data':     [],
      'html':     []
    };

    _columns.push( row );
    _rowdata[row.key] = row; // Für schnelleren Zugriff

    row["$column"].style.width = row.width;
    row["$column"].className = "column";
  };

  function setColumnLabel(key, label) {
    if(_rowdata[key])
      _rowdata[key].label = label;
  };

  function addEntry(obj, isSelected) {
    // Objektdaten auf Zeilen mappen. "value" könnte ein Array sein - deshalb zuerst ermitteln wieviele Zeilen hinzugefügt werden müssen!
    var iLines = 1, a, i, key, v;

    for (key in _rowdata) {
      if (Array.isArray(obj[key]) && obj[key].length > iLines) {
        iLines = obj[key].length;
      }
    }
    
    for (key in _rowdata) {
      if (Array.isArray(obj[key])) {
        a = obj[key];
      } else {
        a = [ obj[key] ];
      }

      for (i = 0; i < iLines; i++) {
        v = a[i] || '';
        _rowdata[key]["data"].push(v);
        _rowdata[key]["html"].push(getRepresentation(v, _rowdata[key]["type"]));
      }
    }

    for(i=0; i<iLines; i++)
      _tooltips.push(obj['Description'] || '');
  };

  function render() {
    var i, j;

    $container.innerHTML = '<div class="ghGridview"></div>';
    _$table = document.createElement('TABLE');
    _$table.setAttribute('cellspacing', '0');
    _$table.setAttribute('cellpadding', '0');
    $container.firstChild.appendChild(_$table);

    _$tableHead = document.createElement('THEAD');
    _$table.appendChild(_$tableHead);
    _$tableBody = document.createElement('TBODY');
    _$table.appendChild(_$tableBody);

    var $row = document.createElement('TR');
    _$tableHead.appendChild($row);

    for(i = 0; i < _columns.length; i++) {
      var $header = document.createElement('TH');
      $header.innerHTML = '<th class="head row">' + htmlspecialchars(_columns[i].label) + '</th>';
      $row.appendChild($header);
    }

    if(_columns.length > 0) {
      var $cell = null;

      for(j = 0; j < _columns[0]['html'].length; j++) {
        $row = document.createElement('TR');
        $row.className = j % 2 ? 'even' : 'odd';
        _$tableBody.appendChild($row);

        for(i = 0; i < _columns.length; i++) {
          $cell = document.createElement('TD');
          $cell.innerHTML = _columns[i]['html'][j];
          $cell.className = 'col' + i;
          $row.appendChild($cell);
        }

        if((j < _tooltips.length) && (_tooltips[j]))
          _tooltipObjs.push( new ghTooltip($cell, _tooltips[j], {'item': $cell, 'openmode': 'top'}) );
      }
    }
  };

  function getRepresentation(v, t) {
    switch(t) {
      case 'icon':
        if(v.indexOf(' ') > 0) // Icon-Angaben mit Leerzeichen verwenden ein anderes Icon-Set
          return '<td><div class="icon ' + htmlspecialchars(v) + '" ></div></td>';
        else
          return '<td><div class="icon ghGuiDefaultIcons ' + htmlspecialchars(v) + '" ></div></td>';

      case 'html':
        return '<td>' + v + '</td>';
       
      case 'string':
      default:
        return '<td>' + htmlspecialchars(v) + '</td>';
    }
  };
};



// Splitview-Control (einfache Ausführung)
function ghSplitter( $container ) {

  this.addPanel = addPanel;
  this.removePanel = removePanel;
  this.getPanels = getPanels;
  this.resize = resize;
  this.onSplitterMoved = null;
  this.setPanelVisible = setPanelVisible;

  var _self = this;
  var _panels = [];
  var _currentTarget, _startY, _startH;
  var _$splitter, _$thumb, _$panel;

  (function constructor() {
    _$splitter = document.createElement('DIV');
    _$splitter.className = 'ghSplitter';
    $container.appendChild( _$splitter );
  })();

  function setPanelVisible( pPanel, isVisible )
  {
    if (isVisible)  {
      pPanel.style.display = "";

      if (pPanel.className.indexOf(' noSpacing') == -1 && pPanel.previousSibling) {
        pPanel.previousSibling.style.display = "";
      }
    } else {
      pPanel.style.display = "none";

      if (pPanel.className.indexOf(' noSpacing') == -1 && pPanel.previousSibling) {
        pPanel.previousSibling.style.display = "none";
      }
    }
  }

  // Fügt einen Anzeigebereich hinzu; Liefert DOM-Element zurück.
  function addPanel( opts ) {
    var $row, $cell, $space;

    if(_$splitter.lastChild) {
      _$splitter.lastChild.style.height="";
    }

    if(_$splitter.childNodes.length && (!opts || !opts['noSpacing'])) {
      $space = document.createElement('DIV');
      $space.className = 'cellspacing' + ((opts && opts["resizable"]) ? ' resizable' : '');
      $space.innerHTML = '<div></div>';
      _$splitter.appendChild( $space );

      if (opts && opts["resizable"]) {
        $space.onmousedown = _dragstart;
      }
    }

    $row = document.createElement('DIV');
    $row.className = 'panel' + ((opts && opts["class"]) ? (' '+opts["class"]) : '') + (opts && opts['noSpacing'] ? ' noSpacing' : '');
    _$splitter.appendChild( $row );

    // Letzte Element bekommt default die max. verfügbare Höhe
    //$row.style.height = "100%";

    _panels.push({panel: $row, options: opts, height: opts.height ? opts.height : 0});

    return $row;
  };

  // Entfernt ein Panel
  function removePanel( $panel ) {
    if($panel && $panel.parentNode == $container) {
      $container.removeChild($panel);

      for(var i = 0; i < _panels.length; i++) {
        if(_panels[i].panel == $panel) {
          _panels.splice(i, 1);
          break;
        }
      }
    }
  };

  // Liefert alle Panel-Objekte.
  function getPanels() {
    return _panels;
  }

  function _getVariableHeight() {
    var i, height, nrSplitters, maxHeight = _$splitter.offsetHeight;

    nrSplitters = _panels.length - 1;

    for (i = 0; i < _panels.length; i++) {
      if (_panels[i].height <= 0) {
        height = _panels[i].panel.offsetHeight;
        maxHeight -= height;

        // Unsichtbare Panels haben keine Splitter:
        if (height <= 0 && nrSplitters > 0) {
          nrSplitters -= 1;
        }
      }
    }

    maxHeight -= nrSplitters * 8; // Splitter-Höhen abziehen

    return Number(maxHeight);
  };

  function resize() {
    var i, maxHeight = _getVariableHeight();

    var remainingHeight = maxHeight;
    var lastPanel = null;

    for (i = 0; i < _panels.length; i++) {
      var panel = _panels[i];
      lastPanel = panel;

      if (panel.height > 0) {
        var height = Math.floor(maxHeight * Number(panel.height));

        if (height > remainingHeight) {
          height = remainingHeight;
        }

        remainingHeight -= height;

        panel.panel.style.height = height + 'px';
      }
    }

    if (lastPanel && remainingHeight > 0) {
      lastPanel.panel.style.height = (Number(trimPx(lastPanel.panel.style.height)) + remainingHeight) + 'px';
    }
  };

  function _dragstart(evt) {
    evt = evt || event;
    _currentTarget = evt.target || evt.srcElement;
    _startY = evt.pageY||evt.clientY;
    _startH = _currentTarget.previousSibling.offsetHeight;

    document.onmousemove = _dragmove;
    document.onmouseup = _dragstop;
    return false;
  };

  function _dragstop(evt) {
    evt = evt || event;
    document.onmousemove = null;
    document.onmouseup = null;
    _currentTarget = null;

    if (typeof _self.onSplitterMoved === 'function') {
      _self.onSplitterMoved(_panels);
    }
  };

  function _dragmove(evt) {
    evt = evt || event;
    var height = Math.min(Math.max(0, _startH - _startY + (evt.pageY||evt.clientY)), document.body.offsetHeight - _currentTarget.previousSibling.offsetTop - _currentTarget.offsetHeight);
    var i, panel = null, nextPanel = null;

    for(i = 0; i < _panels.length; i++) {
      if(_panels[i].panel == _currentTarget.previousSibling) {
        panel = _panels[i];

        if(i < _panels.length - 1)
          nextPanel = _panels[i + 1];
        break;
      }
    }

    if(panel) {
      var oldHeight = panel.panel.offsetHeight;
      var maxHeight = _getVariableHeight();
      panel.height = height / maxHeight; // Geänderte anteilige Größe merken
      panel.panel.style.height = height + 'px';

      if(nextPanel) {
        var nextPanelHeight = nextPanel.panel.offsetHeight + oldHeight - height;
        nextPanel.height = nextPanelHeight / maxHeight; // Geänderte anteilige Größe merken
        nextPanel.panel.style.height = nextPanelHeight + 'px';
      }
    }
  };
};


// Dropdown-Control
function ghDropdown( $container, parent, options ) {
  
  this.clear    = clear;
  this.addEntry = addEntry;
  this.addMenuItem = addMenuItem; // aus kompatibilitätsgründen
  this.setSelected = setSelected;
  this.getSelected = getSelected;
  this.setRenderer = setRenderer;
  this.setVisible  = setVisible;
  this.getVisible  = getVisible;
  this.toggleVisible = toggleVisible;
  this.setInactive = setInactive;
  this.showPopup = _showPopup;
  this.hidePopup = _hidePopup;
  this.blur        = blur;
  this.layoutChanged = _layoutChanged;

  this.onChange = null;
  this.sortOrder = ghSortOrder.ascending;    // a-z
  this.domNode = null;
  this.dropdownDomNode = null;

  var _self = this;
  var _open=false;
  var _renderer;
  var _firstID;
  var _listOfEntries=[];
  var _inactive = false;

  var _$dropdown, _$title, _$arrow, _$content, _$display;


  (function constructor() {
    options = createOptions({
      buttonMode: false,        // true falls die Darstellung die aktuelle Auswahl reflektieren soll (select-verhalten; ansonsten dropdown-button)
      iconInRepresentation: false
    }, options);

    _$dropdown = document.createElement('DIV');
    _self.domNode = _$dropdown;
    _$dropdown.className = 'ghDropdown ' + (options.buttonMode ? 'buttonMode' : '');
    $container.appendChild( _$dropdown );

    addEvent(_$dropdown, 'mousedown', function() { 
      _$dropdown.className = (_$dropdown.className || '').replace(/\s?active\s?/,'') + ' active';
      document.onmouseup = function() {
        _$dropdown.className = (_$dropdown.className || '').replace(/\s?active\s?/,'');
        document.onmouseup = null;
      };
    });

    _$display = document.createElement('DIV');
    _$display.className = 'entry representation' + (options.iconInRepresentation ? '':' noIcon');
    _$dropdown.appendChild(_$display);

    _$title = document.createElement('INPUT');
    _$title.className = 'title';
    _$title.readOnly = true;
    _$dropdown.appendChild(_$title);

    _$arrow = document.createElement('SPAN');
    _$arrow.className = 'arrow';
    _$dropdown.appendChild(_$arrow);

    _$content = document.createElement('DIV');
    _self.dropdownDomNode = _$content;
    _$content.className = 'ghDropdownContainer ' + (options.buttonMode ? 'buttonMode' : '');

    // IE-Patch um zu verhindern das ein blur-event aufgerufen wird wenn gescrollt wird.
    _$content.onmousedown = function(e) {
      _$title.onblur = null;
      setTimeout(function() {
        _$title.focus();
        _$title.onblur = _hidePopup;
      },100)
    };

    if(options.buttonMode)
    {
      _$arrow.onclick = toggleVisible;

      _$display.onclick = function(e) {
        if(_self.onChange && !_inactive && (!_listOfEntries[0].data || !_listOfEntries[0].data.inactive))
          _self.onChange(0, _listOfEntries[0].id, _listOfEntries[0].data);
      };

      _$dropdown.onselectstart = function() { return false }; // prevents textselection
    }
    else
      _$dropdown.onclick = toggleVisible;

    setRenderer(defaultRenderer);

  })();

  function blur(bForceClose) {
    _$title.blur();

    if(bForceClose)
      _hidePopup();
  };

  function setRenderer( renderClass ) {
    _renderer = new renderClass(_$content, _self, options);
    // Check interface
    var a, i = "onChange|render|setSelected|getSelected|addEntry|getRepresentation|clear".split("|");

    while(a = i.pop())
      if(!(a in _renderer))
        throw('Missing interface "'+a+'" for renderer!');

    _renderer.onChange = _valueChanged;
  };

  function addMenuItem(label, icon, id, tooltip, inactive) {
    addEntry(id, {value: id, label: label, icon: "ghGuiDefaultIcons "+icon, tooltip: tooltip, inactive: !!inactive}, false);
  };

  function addEntry(id, data, isSelected) {
    _listOfEntries.push({id: id, data: data});
    _renderer.addEntry(id, data, isSelected);

    if(isSelected && data && data.inactive)
      _$dropdown.className = _$dropdown.className.replace(/\s?dijitDisabled\s?/, '') + ' dijitDisabled';

    if(_listOfEntries.length == 1 || isSelected)
      _$display.innerHTML = _renderer.getRepresentation(id); // default den 1. eintrag für darstellung verwenden oder den zuletzt selektierten
  };

  function setSelected(id) {
    _renderer.setSelected(id);
  };

  function getSelected() {
    return _renderer.getSelected();
  };

  function clear() {
    _renderer.clear();
    _firstID = undefined;
    _listOfEntries=[];
  };

  function _hidePopup() {
    if(!_open)
      return;

    _open = false;
    removeCssClass(_$dropdown, 'ghOpened');

    if(_renderer)
      _renderer.hide();

    _$content.parentNode.removeChild(_$content);

    setTimeout(function() {
      if(options.buttonMode)
        _$arrow.onclick = toggleVisible;
      else
        _$dropdown.onclick = toggleVisible;
    },100); // clickhandler erst nach 100ms wieder ermöglichen sodass ggf ein "onblur"-event verarbeitet wurde.

    window.onresize = null;
  };

  function _layoutChanged() {
    var dropDownRect, contentRect, maxWidth, maxHeight, maxHeightBottom, maxHeightTop, parent;
    _$content.style.height = '';
    _$content.style.width  = '';
    _$content.style.overflow = '';
    maxWidth = document.body.offsetWidth;
    maxHeight = document.body.offsetHeight;

    dropDownRect = axGetRect(_$dropdown);
    contentRect = axGetRect(_$content);

    // Falls das Drop-Down in einem gescrollten Bereich ist müssen wir seine top-Position korrigieren (diese bezieht sich ja nicht auf die sichtbare Position):
    parent = _$dropdown.parentNode;

    while(parent) {
      if(parent.scrollTop > 0) {
        dropDownRect.top -= parent.scrollTop;
        break;
      }

      parent = parent.parentNode;
    }

    _$content.style.width = Math.min(maxWidth - dropDownRect.left, Math.max(_$content.offsetWidth, _$dropdown.offsetWidth - 2)) + 'px';
    _$content.style.left = dropDownRect.left + 'px';

    // Clippen falls Inhalt zu groß:
    maxHeightBottom = maxHeight - dropDownRect.top - dropDownRect.height;
    maxHeightTop = dropDownRect.top - 1;

    if(contentRect.height <= maxHeightBottom) { // drop-down nach unten öffnen:
      _$content.style.top = (dropDownRect.top + dropDownRect.height) + 'px';
      _$content.style.height = contentRect.height + 'px';
    }
    else if(contentRect.height <= maxHeightTop) { // drop-down nach oben öffnen:
      _$content.style.top = (dropDownRect.top - contentRect.height) + 'px';
      _$content.style.height = contentRect.height + 'px';
    }
    else if (maxHeightTop > maxHeightBottom) { // drop-down verkleinern und nach oben öffnen:
      _$content.style.top = '0px';
      _$content.style.height = maxHeightTop + 'px';
    }
    else { // drop-down verkleinern und nach unten öffnen:
      _$content.style.top = (dropDownRect.top + dropDownRect.height) + 'px';
      _$content.style.height = maxHeightBottom + 'px';
    }

    _$content.style.overflow = 'auto';
  };

  function _showPopup() {
    if(_open)
      return;

    _open = true;
    _$content.style.zIndex = ((10000 * ghDialogLayer) + 10100); // Sicherstellen, dass die Drop-Downs immer über allen Dialogen zu sehen sind.
    addCssClass(_$dropdown, 'ghOpened');

    document.body.appendChild(_$content);

    _renderer.render(_$content);

    _$title.focus();
    _$title.onblur = _hidePopup;

    if(options.buttonMode)
      _$arrow.onclick = null;
    else
      _$dropdown.onclick = null;

    _layoutChanged();
    window.onresize = _layoutChanged;
    _$content.scrollTop = 0;
  }

  function toggleVisible(e) {
    e=e||event;

    if(_open)
      _hidePopup();
    else
      _showPopup();

    if(e.stopPropagation)
      e.stopPropagation();

    e.cancelBubble=true;
    return false;
  };

  function _valueChanged(idx, id, data) {
    if(!options.buttonMode || !_$display.innerHTML)
      _$display.innerHTML = _renderer.getRepresentation(id);

    if(_self.onChange && !_inactive)
      _self.onChange(idx, id, data);
  };

  function setVisible( isVisible ) {
    $container.style.display = isVisible ? "" : "none";
  }

  function getVisible() {
    return !!$container.offsetParent; // wenn null dann ist element nicht sichtbar.
  }

  function setInactive(inactive) {
    if(inactive != _inactive) {
      _inactive = inactive;

      if(inactive) {
        _$dropdown.className = _$dropdown.className.replace(/\s?dijitDisabled\s?/, ' ') + ' dijitDisabled';
      }
      else {
        _$dropdown.className = _$dropdown.className.replace(/\s?dijitDisabled\s?/, ' ');
      }
    }
  }

  // Standard-Renderer-Klasse
  // Muss folgende Methoden implementieren:
  // render, setSelected, getSelected, addEntry, getRepresentation, hide
  function defaultRenderer($container, parent, options) {
    
    this.clear       = clear;
    this.render      = render;
    this.setSelected = setSelected;
    this.getSelected = getSelected;
    this.addEntry    = addEntry;
    this.getRepresentation = getRepresentation; // Liefert eine HTML-Zeile
    this.onChange = null;
    this.hide = hide;

    var _sort=[];
    var _data={};
    var _selectedId;
    var _tooltips=[];
    var _self = this;

    function clear() {
      _sort=[];
      _data={};
      _selectedId=undefined;
      $container.innerHTML = '';

      while(_tooltips.length)
        _tooltips.pop().close();
    };

    function hide() {
      var i;

      for(i = 0; i < _tooltips.length; i++)
        _tooltips[i].close();
    };

    function addEntry(id, data, isSelected) {
      if(typeof data == 'string') 
        data = {name: data, icon: null};

      _data[id] = data;
      _sort.push(id);

      if (typeof _selectedId == "undefined" || isSelected)
        setSelected(id);
    };

    function getSelected() {
      return _selectedId;
    }

    function setSelected(id) {
      _selectedId = id;
    };

    function render() {
      var i, id, $entry, $icon, $title;
      
      $container.innerHTML = '';
      for(i = (options.buttonMode ? 1 : 0); i < _sort.length; i++) {
        id = _sort[i];
        $entry = document.createElement("DIV");
        $entry.className = "entry " + ((i%2) ? 'odd' : 'even') + ((id==_selectedId) ? ' selected' : '') + ((_data[id].inactive) ? ' inactive dijitDisabled' : '');
        (function(idx, id) {
          $entry.onmousedown = function() {
            if(_data[id].inactive) return;
            setSelected(id);
            if(_self.onChange) _self.onChange(idx, id, _data[id]); // liefert index!
            parent.blur(true);
          }
        })(i, id);
        $entry.innerHTML = getRepresentation(id);
        $container.appendChild($entry);
    
        if (_data[id].tooltip)
          _tooltips.push( new ghTooltip($entry, _data[id].tooltip, {item: $entry.firstChild}) );

      }
    };

    function getRepresentation(id) {
      return '<span class="title'+(_data[id].icon ? ' withIcon':'')+'">' + 
      (_data[id].icon ? ('<span class="icon ' + _data[id].icon + '"></span>') : '' ) +
      '&nbsp;' +
      (_data[id].label||'').replace(/[&<>]/g, function (c) { return "&" + {"&": "amp", "<": "lt", ">": "gt"}[c] + ";" }) + '</span>';
    };
  };
};



// Treeview-Control
function ghTreeview( $container, parent, options ) {
  
  var _self = this;
  var _lineNo=0;
  var _data = {};
  var _root = {id:-1, childs: []};
  var _rows = [];
  var _selectedId;
  var _stacked=false;
  var _$treeview;
  var _tooltips=[];

  this.addEntry    = addEntry;
  this.clear       = clear;
  this.render      = render;
  this.setSelected = setSelected;
  this.getSelected = getSelected;
  this.getRepresentation = getRepresentation;
  this.onChange = null;
  this.hide = hide;

  (function constructor() {
    _$treeview = document.createElement('DIV');
    _$treeview.className = 'ghTreeview';
    $container.appendChild( _$treeview );
  })();

  function clear() {
    _rows = [];
    _data = {};
    _root = {id:-1, childs: []};
    _selectedId = undefined;
    _lineNo = 0;
    _stacked = false;
    _$treeview.innerHTML = '';

    while(_tooltips.length)
      _tooltips.pop().close();
  };

  function hide() {
    var i;

    for(i = 0; i < _tooltips.length; i++)
      _tooltips[i].close();
  };

  function addEntry(id, data, isSelected) {
    _data[id] = data;
    _data[id].childs = [];
    _data[id].id = id;

    if(data.parent==id) data.parent=null;

    _data[id].Closed = (data.parent!==null);

    _stacked=false;
    if (typeof _selectedId == "undefined" || isSelected)
      setSelected(id);

  };


  function render() {
    var i, x, row, depth, key, node;
    var $entry, $em, $toggle, $span, $repr;

    _$treeview.innerHTML = '';
    _lineNo=0;

    while(_tooltips.length)
      _tooltips.pop().close();

    // Schachteln
    if(!_stacked) {
      for(key in _data)
        _data[key].childs = [];

      for(key in _data) {
        if(_data[key].parent && _data[_data[key].parent])
          _data[_data[key].parent].childs.push(_data[key]);
        else
          _root.childs.push(_data[key]);
      }

      // Ausgewähltes Element und seine Eltern aufklappen:
      if(_selectedId) {
        node = _data[_selectedId];

        while(node) {
          node.Closed = false;

          if(node.parent)
            node = _data[node.parent];
          else
            node = null;
        }
      }

      _stacked=true;
    }

    _rows=[];
    buildTree(_root, 0);
    // Baum aufbauen und einhängen:
    for(i = 0; i < _rows.length; i++) {
      row = _rows[i][0];
      depth = _rows[i][1];

      $entry = document.createElement("DIV");
      $entry.className = 'entry ' + ((++_lineNo % 2) ? 'even':'odd') + ((row.id == _selectedId) ? ' selected' : '') + ((row.inactive) ? ' inactive' : '');
      (function(idx, rowData) {
        $entry.onmousedown = function(e) {
          if(rowData.inactive) return;
          setSelected(rowData.id);
          if(_self.onChange) _self.onChange(idx, rowData.id, rowData);
          while(_tooltips.length) _tooltips.pop().close();
          parent.blur(true);
        }
      })(i, row);

      row.Element = $entry;

      // Icons aufbauen
      for(x = 0; x <= depth; x++) {
        // Korrekte Linien zeichnen:
        if(x==depth) {
          $em = document.createElement("EM");
          $em.className = "line" + _rows[i][2];

          // Childs?
          if(row.childs.length) {
            $toggle = document.createElement("EM");
            $toggle.className = row.Closed ? 'plus' : 'minus';
            (function(rowData) {
              $toggle.onmousedown = function(ev) {
                rowData.Closed = !rowData.Closed;
                render();

                if(ev && ev.stopPropagation)
                  ev.stopPropagation();

                if(parent.layoutChanged)
                  parent.layoutChanged();

                return false;
              }
            })(row);
            $em.appendChild($toggle);
          }
          
          $entry.appendChild($em);

        } else {
          $em = document.createElement("EM");
          $em.className = "line3";
          $entry.appendChild($em);
        }
      }

      // Representation (Davor war nur Geplänkel):
      $repr = document.createElement("DIV");
      $repr.innerHTML = getRepresentation(row.id);
      $entry.appendChild($repr);

      if (row.tooltip)
        _tooltips.push( new ghTooltip($entry, row.tooltip, {item: $repr.firstChild}) );

      _$treeview.appendChild( $entry );
    }
  };

  function addRow(data, depth, state) {
    _rows.push([data, depth, state]);
  };

  function buildTree(node, depth) {
    if (node.Closed || !node.childs || !node.childs.length)
      return;

    if(parent.sortOrder == ghSortOrder.ascending) {
      node.childs.sort(function(a,b) {
        if(a.id == 0) return -1; else if(b.id == 0) return 1;
        var va = (a.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        var vb = (b.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        return parent.sortOrder * (va == vb ? 0 : va < vb ? -1 : 1);
      });
    }
    else if(parent.sortOrder == ghSortOrder.descending) {
      node.childs.sort(function(a,b) {
        if(a.id == 0) return -1; else if(b.id == 0) return 1;
        var va = (a.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        var vb = (b.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        return va == vb ? 0 : va < vb ? 1 : -1;
      });
    }
    else if(parent.sortOrder == ghSortOrder.ascendingAndId) {
      node.childs.sort(function(a,b) {
        if(a.id == 0) return -1; else if(b.id == 0) return 1;
        var va = (a.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        var vb = (b.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        return a.id == 0 ? -1 : b.id == 0 ? 1 : a.id == -1 ? -1 : b.id == -1 ? 1 : va == vb ? 0 : va < vb ? -1 : 1;
      });
    }
    else if(parent.sortOrder == ghSortOrder.descendingAndId) {
      node.childs.sort(function(a,b) {
        if(a.id == 0) return -1; else if(b.id == 0) return 1;
        var va = (a.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        var vb = (b.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        return a.id == 0 ? -1 : b.id == 0 ? 1 : a.id == -1 ? -1 : b.id == -1 ? 1 : va == vb ? 0 : va < vb ? 1 : -1;
      });
    }
    else if(parent.sortOrder == ghSortOrder.ascendingId) {
      node.childs.sort(function(a,b) {
        if(a.id < b.id) return -1; else if(b.id < a.id) return 1;
        var va = (a.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        var vb = (b.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        return va == vb ? 0 : va < vb ? 1 : -1;
      });
    }
    else if(parent.sortOrder == ghSortOrder.descendingId) {
      node.childs.sort(function(a,b) {
        if(a.id < b.id) return 1; else if(b.id < a.id) return -1;
        var va = (a.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        var vb = (b.label||'').replace(/ä/g,"a").replace(/ö/g,"o").replace(/ü/g,"u").replace(/Ä/g,"A").replace(/Ö/g,"O").replace(/Ü/g,"U").replace(/ß/g,"ss");
        return va == vb ? 0 : va < vb ? -1 : 1;
      });
    }

    for(var i = 0; i < node.childs.length; i++) {
      var child = node.childs[i];
      addRow(child, depth, (i==0 && depth==0) ? 0 : (i==node.childs.length-1) ? 2 : 1) ;
      buildTree(child, 1 + depth);
    }
  };
  
  function _clearSelected() {
    for(var row in _data) {
      if(_data[row].Element)
        _data[row].Element.className = _data[row].Element.className.replace(/\sselected/,'');
    }
  };

  function setSelected(id) {
    _clearSelected();
    _selectedId = id;
    if(_data[id].Element)
      _data[id].Element.className = _data[id].Element.className.replace(/\sselected/,'') + ' selected';
  };

  function getSelected() {
    return _selectedId;
  };

  function getRepresentation(id) {
    return '<span class="title'+(_data[id].icon ? ' withIcon':'')+'">' + 
      (_data[id].icon ? ('<em class="icon ' + _data[id].icon + '"></em>') : '') +
      '&nbsp;' +
      (_data[id].label||'').replace(/[&<>]/g, function (c) { return "&" + {"&": "amp", "<": "lt", ">": "gt"}[c] + ";" }) + '</span>';
  };
};


// Verbindung zum Ajax-Handler:

function ghAddonRpc() {
  var _self = this;
  var _host;
  var _port;
  var _protocol;
  var _token;
  var _uniqueClientId;
  var _xhr;
  var _uniqueRpcId = 0;

  this.settingsTimestamp = 0;
  this.onAddonSettingsChanged = null;
  this.onError = [];
  this.url = 'ajax.php';

  this.call = call;
  this.getRpcId = getRpcId;
  this.setSettings = setSettings;
  this.getHost = getHost;

  function setSettings(settings) {
    _host = settings['ghhost'];
    _port = settings['ghport'];
    _protocol = settings['ghprotocol'];
    _token = settings['ghtoken'];
    _uniqueClientId = settings['ghclientid'];
  };

  function getRpcId() {
    return _uniqueRpcId;
  };

  function getHost() {
    return _host;
  }

  function call(method, params, onSuccess, onError, canAbort, uncache) {
    var xhr, request;
    
    xhr = getXHR();

    // Falls bereits ein abrechbarer Request auf Antwort wartet, wird dieser abgebrochen und ein neuer gestartet:
    if(canAbort)
    {
      if (_xhr)
      {
        _xhr.onreadystatechange = null;
        _xhr.abort();
      }

      _xhr = xhr;
    }

    xhr.onreadystatechange = function() {
      // Default Ausgabe auf Konsole falls keine callbacks übergeben:
      onSuccess = onSuccess || function(){};
      var _onError = function(response, callParams){
        for(var i = 0; i < _self.onError.length; i++)
          if(_self.onError[i](response, callParams))
            return;

        if(onError)
          onError(response, callParams);
      };

      if(xhr.readyState != 4)
        return;

      if(xhr.status != 200)
        return _onError({"jsonrpc": "2.0", "result": null, "error": {code: -32603, message: "Internal error (" + xhr.status + " " + xhr.statusText + ")"}}, {method: method, params: params, onSuccess: onSuccess, onError: onError, canAbort: canAbort});

      // Versuche Daten auszuwerten:
      try {
        var response = JSON.parse(xhr.responseText);

        if (response && 'result' in response)
          return onSuccess(response, {method: method, params: params, onSuccess: onSuccess, onError: onError, canAbort: canAbort});
        else if (response && response.error) {
          // Falls sich die Einstellungen des Addons zwischenzeitlich geändert haben werden diese neu abgerufen, der Zeitstempel wird solange auf 0 gesetzt, da sonst immer nur die Exception zurück kommt:
          if(response.error.name && response.error.name == 'GhAddonSettingsChangedException') {
            var newSettingsTimestamp = response.error.settingsTimestamp;
            _self.settingsTimestamp = 0;

            _self.call('getAddonSettings', [], function(response) {
              if(response && response.result) {
                // Wenn die Einstellungen abgerufen werden konnten, wird der neue Zeitstempel gesichert, ggf. der Callback aufgerufen und dann der ursprüngliche Request erneut durchgeführt:
                _self.settingsTimestamp = newSettingsTimestamp;

                //Key-Value Paare in Objekt umwandeln:
                var fixedSettings = {};

                for(var i = 0; i < response.result.length; i++)
                  fixedSettings[response.result[i].key] = response.result[i].value;

                response.result = fixedSettings;

                if(_self.onAddonSettingsChanged)
                  _self.onAddonSettingsChanged(response.result);

                _self.call(method, params, onSuccess, _onError, canAbort);
              }
              else
                _onError(response, {method: method, params: params, onSuccess: onSuccess, onError: onError, canAbort: canAbort});
            }, function(response) {
              _onError(response, {method: method, params: params, onSuccess: onSuccess, onError: onError, canAbort: canAbort});
            }, false);
          }
          else
            return _onError(response, {method: method, params: params, onSuccess: onSuccess, onError: onError, canAbort: canAbort});
        }
        else
          return _onError({"jsonrpc": "2.0", "result": null, "error": {code: -32700, message: "Parse error"}}, {method: method, params: params, onSuccess: onSuccess, onError: onError, canAbort: canAbort});
      } catch(ex) {
        return _onError({"jsonrpc": "2.0", "result": null, "error": {code: -32700, message: "RPCCall: " + ex.message + " ("+method+")" + xhr.responseText, data: ex}}, {method: method, params: params, onSuccess: onSuccess, onError: onError, canAbort: canAbort});
      }
    };

    request = {
      "id": ++_uniqueRpcId,
      "jsonrpc": "2.0",
      "method": method,
      "params": params
    };

    if (uncache) {
      request.uncache = true;
    }

    xhr.open('POST', _self.url + "?ghhost="+encodeURIComponent(_host)+"&ghport="+encodeURIComponent(_port)+"&ghprotocol="+encodeURIComponent(_protocol)+"&ghtoken="+encodeURIComponent(_token)+"&ghclientid="+encodeURIComponent(_uniqueClientId)+"&ghsettingstimestamp="+_self.settingsTimestamp, true);
    xhr.setRequestHeader('Content-Type', 'application/json; charset=UTF-8');
    xhr.send(JSON.stringify(request));
  };
}


// Addon-Account:
function ghAddonAccount(id, settings, rpc) {
  // Public Interface
  this.setSettings = setSettings;
  this.refresh = refresh;

  // Public Attributes
  this.id = ~~id;
  this.rpc = rpc;
  this.settings = {};
  this.templates = [];
  this.afterRefresh = null;

  // Privates
  var _self = this;

  (function constructor() {
    refresh(settings);
  })();

  function setSettings(settings) {
    _self.settings = {};

    for (var key in settings) {
      _self.settings[key] = settings[key];
    }
  };

  function refresh(settings) {
    setSettings(settings);
    refreshTemplates();
  };

  function refreshTemplates() {
    rpc.call("getTemplates", [_self.id],
      function(response) {
        _self.templates = response.result || [];

        if (_self.afterRefresh) {
          _self.afterRefresh(_self);
        }
      },
      undefined,
      false
    );
  };
};


// Addon Datenhaltung:
function ghAddonDataManager(rpc) {
  var _self = this;
  var _lastSearch;

  // Öffentliche Eigenschaften:
  this.accounts = []; // Addon Accounts
  this.account = null; // Aktueller Account
  this.email = '';
  this.origEmail = '';
  this.eBayName = '';
  this.rpc = rpc;
  this.sendHash = false; // true: vollständigen Hash an jede Suche mit anhängen

  // Öffentliche Methoden:
  this.search = search;
  this.repeatSearch = repeatSearch;
  this.setAccountId = setAccountId;
  this.invalidateData = invalidateData;
  this.isValidRecipient = isValidRecipient;
  this.setSettings = setSettings;

  this.onAccountChanged = null;



  // Setzt eine neue AccountID (falls im Backend mehrere Anbindungen konfiguriert wurden).
  function setAccountId(id) {
    var i;

    for (i = 0; i < _self.accounts.length; i++) {
      if (id == _self.accounts[i].id) {
        _self.account = _self.accounts[i];

        if (typeof _self.onAccountChanged == 'function') {
          _self.onAccountChanged(id);
        }
        
        break;
      }
    }
  };

  /**
   * Führt eine Suche durch.
   * Die letzte Suchanfrage wird gespeichert und ggf. das Ergebnis aus dem Cache geholt.
   *
   * @param {string} rpcMethod Suchmethode
   * @param {object} rpcParams Suchparameter
   * @param {object} hash Aufrufparameter (Hash) der Webseite
   * @param {function} onSuccess Callback bei Erfolg (Parameter: response, rpcMethod, rpcParams)
   * @param {function} onError Callback bei Fehler (Parameter: response, rpcMethod, rpcParams)
   * @param {function} onSearch Callback der aufgerufen wird, wenn der Request abgeschickt wird (Parameter: rpcMethod, rpcParams)
   * @param {function} onSearchTerm Callback der aufgerufen wird, wenn der Suchbegriff festgelegt wurde (Parameter: searchTerm)
   * @param {boolean} uncache Verwirft ein ggf. zuvor gecachtes Ergebnis für diese Suche
   */
  function search(rpcMethod, rpcParams, hash, onSuccess, onError, onSearch, onSearchTerm, uncache) {
    if (!_self.account) {
      throw 'Es wurde keine gültige Verbindung konfiguriert.';
    }

    _lastSearch = [_self.account.id, rpcMethod, rpcParams, {itemId: hash.itemID, itemKind: hash.itemKind, itemClassified: hash.itemClassified}];

    if (_self.sendHash) {
      _lastSearch.push(hash);
    }

    // Feedback über den Beginn der Suche:
    if (onSearch) {
      onSearch(rpcMethod, rpcParams);
    }

    _self.rpc.call('search', _lastSearch, function(response, rpcMethod, rpcParams) {
        if(onSearchTerm && response && response.result && response.result.searchedTerm)
          onSearchTerm(response.result.searchedTerm);

        if (onSuccess) {
          onSuccess(response, rpcMethod, rpcParams);
        }
      }, function(response, rpcMethod, rpcParams) {

        if (onSearchTerm && response && response.error && response.error.searchTerm) {
          onSearchTerm(response.error.searchTerm);
        }

        if (onError) {
          onError(response, rpcMethod, rpcParams);
        }
      },
      true, uncache);
  };

  function repeatSearch(accountId, onSuccess, onError, onSearch) {
    _self.account.id = accountId;
    _lastSearch[0] = accountId;

    // Feedback über den Beginn der Suche:
    if (onSearch) {
      onSearch(_lastSearch[1], _lastSearch[2]);
    }

    _self.rpc.call('search', _lastSearch, onSuccess, onError, true);
  };

  // Prüft ob recipients-Liste für diese Anfrage zuständig ist.
  function isValidRecipient(recipients, filters) {
    if (!recipients||!recipients.length) {
      return true;
    }

    if (!Array.isArray(recipients)) {
      recipients = [recipients];
    }

    if (!filters) {
      return false;
    }

    for (var i = 0; i < recipients.length; i++) {
      var recipient = recipients[i];

      for (var j = 0; j < filters.length; j++) {
        if (recipient.match(filters[j])) {
          return true;
        }
      }
    }

    return false;
  };

  /**
   * Bereits vorhandene Daten als ungültig markieren und anhand der Suchparameter neu anfragen.
   * Die letzte Suchanfrage wird gespeichert und ggf. das Ergebnis aus dem Cache geholt.
   *
   * @param {object} hash Aufrufparameter (Hash) der Webseite
   * @param {function} onSuccess Callback bei Erfolg (Parameter: response, rpcMethod, rpcParams)
   * @param {function} onError Callback bei Fehler (Parameter: response, rpcMethod, rpcParams)
   * @param {function} onSearchTerm Callback der aufgerufen wird, wenn der Suchbegriff festgelegt wurde (Parameter: searchTerm)
   * @param {function} onSearch Callback der aufgerufen wird, wenn der Request abgeschickt wird (Parameter: rpcMethod, rpcParams)
   * @param {boolean} uncache Verwirft ein ggf. zuvor gecachtes Ergebnis für diese Suche
   */
  function invalidateData(hash, onSuccess, onError, onSearchTerm, onSearch, uncache) {
    var itemState = hash.itemState;
    var recipient = trim(hash.itemRecipient);
    var recipients = [];
    var itemEmailName = hash.itemEmailName;
    var outgoingEmail = (itemState == RpcItemState.isNew || itemState == RpcItemState.isAnswer || itemState == RpcItemState.isForward || itemState == RpcItemState.isDraft);
    var index1, index2, name, domain, i;

    if (hash.itemFlags && (hash.itemFlags & RpcItemFlag.ifSpam)) {
      onError({error: {message: 'Für Spam-Nachrichten werden keine Daten abgerufen.', level: 'info'}});
      return;
    }

    // Der initiale Request erfolgt immer über die Mailadresse. Bei Service-Mails, existiert i.d.R. eine fallback-adresse welche zuvor im GHClient aus dem Mailtext extrahiert wurde.
    _self.email      = (hash.itemReplyToEmail ? hash.itemReplyToEmail : hash.itemEmail) || '';
    _self.origEmail  = (hash.itemOrigEmail ? hash.itemOrigEmail : _self.email) || '';

    if (outgoingEmail) {
      // Eingehende Mail/Absender ist das was wir suchen
      recipients.push(_self.email);
      _self.email = recipient;
      itemEmailName = hash.itemRecipientName;
    } else {
      if (recipient) {
        recipients.push(recipient);
      }

      if (hash['itemCC[]']) {
        for (var e = 0; e < hash['itemCC[]'].length; e++) {
          recipients.push(trim(hash['itemCC[]'][e]));
        }
      }
    }

    // Falls das Addon aufggerufen wurde bevor die Verbindungen hinterlegt wurden, haben wir hier noch keine Zugangsdaten und sollte diese neu laden:
    if (_self.accounts.length == 0) {
      _self.rpc.call("getAddonSettings", [],
        function(response){
          if (response && response.result) {
            //Key-Value Paare in Objekt umwandeln:
            var fixedSettings = {};

            for (var i = 0; i < response.result.length; i++) {
              fixedSettings[response.result[i].key] = response.result[i].value;
            }

            response.result = fixedSettings;

            setSettings(response.result);
            if (_self.accounts.length != 0) {
              return location.reload(true); // muss ein reload der seite sein um die gui neu aufzubauen!
            } else {
              onError({error: {message: 'Es wurden keine gültigen Verbindungsdaten hinterlegt, oder die Verbindungen sind inaktiv.'}});
            }
          }
        },
        function(err) {
          var host;

          host = _self.rpc.getHost();
          host = host ? ' (Verwendeter Host/IP: ' + host + ')' : '';
          onError({error: {message: 'Es konnte keine Verbindung zum GREYHOUND Server hergestellt werden. Bitte überprüfen Sie die Einstellung "Öffentlicher Host/IP" des GREYHOUND Servers.' + host}});
        },
        false
      );
      return;
    }

    // eBayName
    _self.eBayName = hash.eBayName || '';
    _self.eBayName = _self.eBayName.replace(/[^a-zA-Z0-9\.\*\-_]/g, '');

    if (_self.email && _self.email.indexOf('@members.ebay.') > 0 && !_self.eBayName && itemEmailName) {
      _self.eBayName = itemEmailName.replace(/\+/g, ' ').replace(/[ \-]*eBay[ \-]*/g, '');
    }

    var eBayNameSeparator = _self.eBayName.indexOf(':');

    if (eBayNameSeparator >= 0) {
      _self.eBayName = _self.eBayName.substring(eBayNameSeparator + 1);
    } else {
      eBayNameSeparator = _self.eBayName.indexOf(' - ');

      if (eBayNameSeparator >= 0) {
        _self.eBayName = _self.eBayName.substring(eBayNameSeparator + 3);
      }
    }

    _self.eBayName = _self.eBayName.replace(/[^a-zA-Z0-9\.\*\-_]/g, '');

    if (!_self.email && hash.contactEmail) {
      _self.email = hash.contactEmail;
    }

    // Service-E-Mail Adressen anullieren (interessant ist nur die Kunden-Adresse):
    if (_self.email == 'service@paypal.de' || _self.email.indexOf('@members.ebay.') > 0) {
      _self.email = '';
    }

    if (_self.email.indexOf('ebay@ebay.') == 0 && _self.eBayName) {
      _self.email = '';
    }

    index1 = _self.email.indexOf('@marketplace.amazon.');

    if (index1 > 0) {
      name = _self.email.substring(0, index1);
      domain = _self.email.substring(index1);
      index2 = name.indexOf('+');

      if (index2 > 0) {
        _self.email = name.substring(0, index2) + domain;
      }
    }

    // Falls keine E-Mail Adresse bzw. kein eBay-Name (wo kommt der her?) vorliegt wird als Fallback auf einen regulären Ausdruck über den Inhalt zurück gegriffen:
    if (!_self.email && hash.emailFallback && hash.emailFallback.indexOf('@')> 0) {
      _self.email = hash.emailFallback;
    }

    // Default den ersten Account verwenden
    if (_self.accounts.length == 1) {
      setAccountId(_self.accounts[0].id);
    }

    // Da mehrere Zugänge konfiguriert wurden, suchen wir hier den besten heraus (abhängig vom vermeintlichen Absender)
    var matchingAccount = false;

    // Alle Accounts durchgehen und schauen welche ggf. passen könnten!
    for (i = 0; i < _self.accounts.length; i++) {
      if (isValidRecipient(recipients, _self.accounts[i].settings.Recipients)) {
        matchingAccount = true;
        setAccountId(_self.accounts[i].id);
        break;
      }
    }

    if (!_self.account || _self.account.id === null) {
      onError({error: {message: 'Es wurden keine gültigen Zugänge hinterlegt.'}});
      return;
    }

    if (!matchingAccount) {
      onError({error: {message: 'Für diesen Empfänger wurden keine Daten abgerufen.', level: 'info'}});
      return;
    }

    var ignoreFrom = _self.accounts[i].settings.IgnoreFrom;

    if (_self.email && ignoreFrom) {
      ignoreFrom = ignoreFrom.split('\r\n');

      for (i = 0; i < ignoreFrom.length; i++) {
        if (ignoreFrom[i].length > 0 && _self.email.match(ignoreFrom[i].replace('?', '.').replace('*', '.*'))) {
          onError({error: {message: 'Für diesen Absender wurden keine Daten abgerufen.', level: 'info'}});
          return;
        }
      }
    }

    // Das was letztendlich gesucht wurde auch in Suchmaske eintragen:
    if (onSearchTerm) {
      onSearchTerm(hash.previewSearchTerm || _self.email || _self.eBayName);
    }

    // Suche ausführen:
    var searchParams = {};

    if (_self.email) {
      searchParams.email = _self.email;
    }

    if (_self.eBayName) {
      searchParams.ebay = _self.eBayName;
    }

    if (hash.itemTaskNumber && hash.itemTaskNumber.length > 0 && hash.itemTaskNumber[0] != 'T') {
      searchParams.taskNumber = hash.itemTaskNumber;
    }

    // Amazon Bestellnummern:
    if (hash.amazonID) {
      searchParams.amazon = hash.amazonID;
    }

    return searchResult = _self.search('search', searchParams, hash, onSuccess, onError, onSearch, onSearchTerm, uncache);
  };

  /**
   * Setzt die Einstellungen und erzeugt die Accounts aus den Einstellungen.
   *
   * @param {object} settings Einstellungen
   * @param {function} onAccountRefreshed Callback für aktualisierte Accounts
   */
  function setSettings(settings, onAccountRefreshed) {
    var key, tmp, id, val, accountData = {};

    for (key in settings) {
      val = settings[key];

      if (key.match(/^Account\.(\d+)\.(.*)$/)) {
        tmp = key.match(/^Account\.(\d+)\.(.*)$/);
        id  = tmp[1];
        key = tmp[2];

        if (!accountData[id]) {
          accountData[id] = {Activated: false, Caption: 'Neue Verbindung', id: id, Recipients: []}; // Default erstellen
        }

        if (key.match(/^Recipient\d+$/)) {
          // Filter in reguläre Ausdrücke wandeln:
          val = val.replace(/\./g, '\\.').replace(/[\?\*\+\(\)\[\]]/g, function (c) { return {'?': '.', '*': '.*', '+': '\\+', '(': '\\(', ')': '\\)', '[': '\\[', ']': '\\]'}[c] });
          accountData[id].Recipients.push(new RegExp('^' + val + '$'));
        } else {
          accountData[id][key] = val;
        }
      }
    }

    // Accountdaten durchgehen; inaktive löschen und leere Recipientsfilter behandeln
    _self.accounts = [];
    for (key in accountData) {
      if (!(~~accountData[key].Activated)) {
        continue;
      }

      if (!accountData[key].Recipients.length) {
        accountData[key].Recipients.push( new RegExp('^.*$') );
      }

      var account = new ghAddonAccount(key, accountData[key], _self.rpc);

      if (onAccountRefreshed) {
        account.afterRefresh = onAccountRefreshed;
      }

      _self.accounts.push(account);
    }

    if (_self.accounts.length > 0) {
      _self.setAccountId(_self.accounts[0].id);
    }
  };
};


// Dialog:
var ghDialogLayer = 0;

function ghDialog(options) {
  // Öffentliche Methoden:
  this.show = show;
  this.hide = hide;
  this.setTitle = setTitle;
  this.dispose = dispose;

  // Öffentliche Eigenschaften:
  this.$node = null;
  this.$content = null;
  this.buttons = {};

  var _self = this;
  var _$titleBar, _$content, _$statusBar, _$okButton;
  var _$opacityLayer;
  var _layer = 0;
  var _absolute = false;

  (function constructor(options) {
    var $statusDivider, button, i;

    options = createOptions({}, options);

    _self.$node = document.createElement("DIV");
    _self.$node.className = "ghDialog";
    _self.$node.style.display = "none";

    if (options.absolute) {
      _absolute = true;
      _self.$node.style.position = 'absolute';
      _self.$node.style.top = (options.margins && options.margins.top ? options.margins.top + 'px' : '12px');
      _self.$node.style.right = (options.margins && options.margins.right ? options.margins.right + 'px' : '12px');
      _self.$node.style.bottom = (options.margins && options.margins.bottom ? options.margins.bottom + 'px' : '12px');
      _self.$node.style.left = (options.margins && options.margins.left ? options.margins.left + 'px' : '12px');
      _self.$node.style.width = 'inherit';
      _self.$node.style.maxWidth = 'inherit';
      _self.$node.style.maxHeight = 'inherit';
    }

    if (options.width) {
      _self.$node.style.width = options.width;
    }

    _$titleBar = document.createElement("DIV");
    _$titleBar.className = "ghDialogTitleBar";
    _self.$node.appendChild(_$titleBar);

    if (options.title) {
      _$titleBar.innerHTML = htmlspecialchars(options.title);
    }

    _self.$content = document.createElement("DIV");
    _self.$content.className = "ghDialogContent";
    _self.$node.appendChild(_self.$content);

    if (options.message) {
      _self.$content.innerHTML = htmlspecialchars(options.message);
    }

    _$statusBar = document.createElement("DIV");
    _$statusBar.className = "statusbar";
    _self.$node.appendChild(_$statusBar);

    $statusDivider = document.createElement("DIV");
    $statusDivider.className = "statusdivider";
    _$statusBar.appendChild($statusDivider);

    if (options.absolute) {
      _self.$content.style.overflowY = 'auto';
      _self.$content.style.position = 'absolute';
      _self.$content.style.top = '29px';
      _self.$content.style.right = '0';
      _self.$content.style.bottom = '43px';
      _self.$content.style.left = '0';

      _$statusBar.style.position = 'absolute';
    }

    if (options.buttons && options.buttons.length) {
      for (i = options.buttons.length - 1; i >= 0; i--) {
        button = new ghButton(_$statusBar, {
          caption: options.buttons[i].label,
          onClick: options.buttons[i].onClick || options.buttons[i].onclick
        });

        if (options.buttons[i].id) {
          _self.buttons[options.buttons[i].id] = button;
        }
      }
    } else {
      button = new ghButton(_$statusBar, {
        caption: 'OK',
        onClick: function() { _self.hide(); }
      });
      _self.buttons.ok = button;
    }

    _$opacityLayer = document.createElement("DIV");
    _$opacityLayer.className = "ghDialogOpacityLayer";
    _$opacityLayer.style.display = "none";
    document.body.appendChild(_$opacityLayer);
    document.body.appendChild(_self.$node);
  })(options);

  function show(title, message) {
    if (title !== undefined) {
      _$titleBar.innerHTML = htmlspecialchars(title);
    }

    if (message !== undefined) {
      _self.$content.innerHTML = htmlspecialchars(message);
    }

    _layer = ++ghDialogLayer;
    _self.$node.style.zIndex = (10000 * _layer) + 1;

    if (!_absolute) {
      _self.$node.style.left = 0;
    }

    _$opacityLayer.style.zIndex = (10000 * _layer);
    _$opacityLayer.style.display = "block";
    _self.$node.style.display = "block";

    if (!_absolute) {
      _self.$node.style.left = (document.body.offsetWidth/2 - _self.$node.offsetWidth/2) + "px";
    }
  };

  function hide() {
    ghDialogLayer--;
    _self.$node.style.display = "none";
    _$opacityLayer.style.display = "none";
  };

  function dispose() {
    document.body.removeChild(_self.$node);
    document.body.removeChild(_$opacityLayer);
  }

  function setTitle(title) {
    _$titleBar.innerHTML = title;
  }
};
