/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

// GREYHOUND Addon Funktionen
greyhound.ghAddon = new function () {
  this.tooltipDelay = 500; // Millisekunden
  this.tooltipTimeout = null;
  this.tooltipItem = null;
  this.tooltipEvent = null;
  this.tooltipAnchor = null;

  /**
   * Diese Methode bereitet ein Tooltip für die Anzeige vor.
   * Das Tooltip wird erst nach einer Verzögerung dargestellt.
   * @param event Event Objekt
   */
  this.prepareTooltip = function(event)
  {
    if(greyhound.ghAddon.tooltipTimeout)
      window.clearTimeout(greyhound.ghAddon.tooltipTimeout);

    greyhound.ghAddon.tooltipTimeout = null;
    greyhound.ghAddon.tooltipAnchor = null;
    greyhound.ghAddon.tooltipItem = null;

    // Tooltip für Tree- und Grid-Elemente:
    if(event.type && event.type == "mouseover" && event.toElement)
    {
      var widget;

      if(event.cellIndex == 2)
      {
        widget = dijit.getEnclosingWidget(event.toElement);
        var item = widget.grid.getItem(event.rowIndex);

        greyhound.ghAddon.tooltipAnchor = event.toElement;

        if(item && !item.isEmpty && (item.Description || item.tooltip))
          greyhound.ghAddon.tooltipItem = item;
      }
      else
      {
        widget = dijit.getEnclosingWidget(event.toElement);
        greyhound.ghAddon.tooltipAnchor = event.toElement;

        if(widget.item && !widget.item.isEmpty && (widget.item.Description || widget.item.tooltip))
          greyhound.ghAddon.tooltipItem = widget.item;
      }
    }

    if(greyhound.ghAddon.tooltipItem)
    {
      greyhound.ghAddon.tooltipEvent = event;
      greyhound.ghAddon.tooltipTimeout = window.setTimeout("greyhound.ghAddon.showTooltip()", greyhound.ghAddon.tooltipDelay);
    }
  };

  /**
   * Diese Methode zeigt ein vorbereitetes Tooltip an.
   */
  this.showTooltip = function()
  {
    if(!greyhound.ghAddon.tooltipEvent || !greyhound.ghAddon.tooltipItem || !greyhound.ghAddon.tooltipAnchor)
      return;

    var tooltipText;
    var tooltipPosition = "below";

    if(greyhound.ghAddon.tooltipItem.Description)
      tooltipText = greyhound.ghAddon.tooltipItem.Description;
    else if(greyhound.ghAddon.tooltipItem.tooltip)
      tooltipText = greyhound.ghAddon.tooltipItem.tooltip;

    if(greyhound.ghAddon.tooltipItem.DescriptionPosition)
      tooltipPosition = greyhound.ghAddon.tooltipItem.DescriptionPosition;

    dijit.showTooltip("<div class=\"GhAddonTooltip\">" + tooltipText + "</div>", greyhound.ghAddon.tooltipAnchor, [tooltipPosition]);
  };

  /**
   * Diese Methode verbirgt ein zuvor angezeigtes Tooltip.
   */
  this.hideTooltip = function()
  {
    if(greyhound.ghAddon.tooltipTimeout)
      window.clearTimeout(greyhound.ghAddon.tooltipTimeout);

    greyhound.ghAddon.tooltipTimeout = null;

    if(greyhound.ghAddon.tooltipAnchor)
      dijit.hideTooltip(greyhound.ghAddon.tooltipAnchor);

    greyhound.ghAddon.tooltipAnchor = null;
  };
}();
