/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

// Ajax-Funktionen für Addons
(function(ghAddonAjax, $) {

  ghAddonAjax.url = 'ajax.php';
  ghAddonAjax.successCallback = null;
  ghAddonAjax.errorCallback = null;

  ghAddonAjax.init = function() {
    window.bind('hashchange', _onHashChange);
    _onHashChange();
  };

  ghAddonAjax.getLocationHash = function() {
    var hash = window.location.hash;

    if(hash == undefined || hash.length == 0)
      return "";

    hash = decodeURIComponent(hash);

    if(hash.charAt(0) == '#')
      hash = hash.substr(1);

    return hash;
  };

  ghAddonAjax.getLocationHashParams = function() {
    return ghAddonAjax.getParamsFromString(ghAddonAjax.getLocationHash());
  };

  ghAddonAjax.getParamsFromString = function(str) {
    var params = {};

    if(str == undefined || str.length < 1)
      return params;

    var parts = str.split('&');

    for(var i=0; i<parts.length; i++) {
      var part = parts[i];
      var splitPos = part.indexOf('=');

      if(splitPos > 0)
        params[part.substr(0, splitPos)] = part.substr(splitPos + 1);
      else
        params[part] = true;
    }

    return params;
  };

  var _onHashChange = function() {
    var url = ghAddonAjax.url + "?page=" + ghAddonPage + "&" + ghAddonAjax.getLocationHash();
    _ajaxCall(url, ghAddonAjax.successCallback, ghAddonAjax.errorCallback);
  };

  var _ajaxCall = function(url, success, error) {
    var settings = {
      contentType: 'application/json',
      type: 'GET',
      timeout: ghAddonTimeout,
      success: function(response) {
        if(response.error && error)
          error(response.error);
        else if(response.result && success)
          success(response.result);
      },
      error: function(response) {
        if(error) {
          if(response.error)
            error(response.error);
          else
            error({code: 500, message: 'Unknown error reported by php proxy', name: 'Exception'});
        }
      }
    };

    $.ajax(url, settings);
  };

})(ghAddonAjax = function() {}, jQuery);

document.ready(function(){
  ghAddonAjax.init();
});
