<?php
/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Datenklasse für Kundendaten.
 *
 * @package GhAddonFramework
 * @subpackage core
 * @category PHP Addon Framework
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2019 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhAddonCustomer
{
  /**
   * ID des Kunden.
   * @var int
   */
  public $ID;

  /**
   * ID zur Anzeige (z.B. Kundennummer).
   * @var string
   */
  public $CustomerNumber;

  /**
   * Bezeichnung / Listendarstellung des Kunden.
   * @var string
   */
  public $Name;

  /**
   * Icon für die Listendarstellung des Kunden.
   * @var string
   */
  public $Icon = 'businessman';

  /**
   * Beschreibung des Kunden.
   * Ist die Beschreibung mehrzeilig, so liegt sie als Array vor.
   * Die Beschreibung wird z.B. für Tooltips verwendet.
   * @var array|string
   */
  public $Description;

  /**
   * E-Mail Adresse des Kunden.
   * @var string
   */
  public $Email;

  /**
   * Telefonnummer des Kunden.
   * Die Nummer muss das internationale Format haben (z.B. +49...).
   * @var string
   */
  public $Phone;

  /**
   * Faxnummer des Kunden.
   * Die Nummer muss das internationale Format haben (z.B. +49...).
   * @var string
   */
  public $Fax;

  /**
   * Postadresse des Kunden.
   * Die Adresse wird als Array mit den einzelnen Zeilen der Postanschrift
   * abgelegt.
   * @var array
   */
  public $Address;

  /**
   * Beliebige zusätzliche Daten des Kunden.
   * @var array
   */
  public $Data;

  /**
   * Aktualisiert die Kontaktdaten des Kunden anhand der Rechnungsadresse.
   *
   * @param array $address Rechnungsadresse
   * @return null
   */
  public function setBillingAddress($address)
  {
    if(!is_array($address))
      return;

    if($address['Email'] && strpos($address['Email'], '@') > 0)
      $this->Email = $address['Email'];

    if($address['Phone'])
      $this->Phone = $address['Phone'];

    if($address['Fax'])
      $this->Fax = $address['Fax'];

    if($address['Letter.City'])
      $this->Address = GhCoreLocale::getAddressLines($address);
  }
}