<?php
/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2014-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Exception-Klasse für leere Suchergebnisse.
 *
 * @package GhAddonFramework
 * @subpackage core
 * @category PHP Addon Framework
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2014-2019 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhAddonEmptySearchException extends GhAddonException
{
  /**
   * Felder, in denen gesucht wurde.
   *
   * @var array
   */
  protected $_searchedFields;

  /**
   * Suchbegriff, nach dem gesucht wurde.
   *
   * @var string
   */
  protected $_searchTerm;

  /**
   * Hinweis, den das Addon bei leerer Suche zusätzlich anzeigen soll.
   */
  protected $_searchInfo;



  /**
   * Konstruktor.
   *
   * @param string $term Suchbegriff
   * @param array $fields Felder, in denen gesucht wurde
   * @param string $searchTerm Suchbegriff, nach dem gesucht wurde
   */
  public function __construct($term, $fields = null, $searchTerm = null, $searchInfo = null)
  {
    parent::__construct($term);

    if(is_array($fields))
      $this->setSearchedFields($fields);

    $this->setSearchTerm($searchTerm);
    $this->setSearchInfo($searchInfo);
  }

  /**
   * Setzt die Suchfelder.
   *
   * @param array $fields Felder, in denen gesucht wurde
   */
  public function setSearchedFields($fields)
  {
    if(is_array($fields))
      $this->_searchedFields = $fields;
    else
      $this->_searchedFields = array();
  }

  /**
   * Liefert die Felder, in denen gesucht wurde.
   *
   * @return array
   */
  public function getSearchedFields()
  {
    return $this->_searchedFields;
  }

  /**
   * Setzt den Suchbegriff, nach dem gesucht wurde.
   *
   * @param string $searchTerm Suchbegriff
   */
  public function setSearchTerm($searchTerm)
  {
    if(is_object($searchTerm))
      $searchTerm = get_object_vars($searchTerm);

    if(is_array($searchTerm))
    {
      foreach($searchTerm as $key => $value)
      {
        if($value)
        {
          $searchTerm = $value;
          break;
        }
      }
    }

    $this->_searchTerm = $searchTerm;
  }

  /**
   * Liefert den Suchbegriff, nach dem gesucht wurde.
   *
   * @return string
   */
  public function getSearchTerm()
  {
    return $this->_searchTerm;
  }

  /**
   * Setzt den Hinweis, den das Addon bei leerer Suche zusätzlich anzeigen soll.
   * 
   * @param string $searchInfo Hinweistext
   */
  public function setSearchInfo($searchInfo)
  {
    $this->_searchInfo = $searchInfo;
  }

  /**
   * Liefert den Hinweis, den das Addon bei leerer Suche zusätzlich anzeigen soll.
   * 
   * @return string
   */
  public function getSearchInfo()
  {
    return $this->_searchInfo;
  }
}