/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

// Hilfsfunktion zum Markieren von Textbaustein-Variablen beim Anklicken:
function selectText(element) {
  var range, selection;

  if(document.body.createTextRange) {
    range = document.body.createTextRange();
    range.moveToElementText(element);
    range.select();
  }
  else if(window.getSelection) {
    selection = window.getSelection();
    range = document.createRange();
    range.selectNodeContents(element);
    selection.removeAllRanges();
    selection.addRange(range);
  }
};

// Addon-Hilfeseite:
function ghAddonHelpPage() {
  "use strict";

  // Lokale Variablen (private)
  var _self = this;
  var _uniqueRpcId = 0;
  var _versionString  = '';

  var _hash           = {};
  var _rpc;

  var _guiId = 1000;

  //  Instanzen verwendeter Komponenten
  var _$infoView, _$errorView, _$body, _$help, _$errorOpacityLayer, _$statusBtnOK;


  //
  //  Constructor
  //
  (function constructor() {
    _hash = extractHashValues();

    _rpc = new ghAddonRpc();
    _rpc.setSettings(_hash);

    buildGUI();

    _rpc.call('getAddonInfo', [], function(response) {
      if(response && response.result) {
        _versionString = 'Version ' + response.result.version + ' build ' + response.result.build;
        _$infoView.innerHTML = _versionString;
        _rpc.settingsTimestamp = response.result.settingsTimestamp;
        document.getElementById('helpManualLink').innerHTML = '<a href="greyhound:ExecExtension?Identifier=' + response.result.webExtensionIdentifier + '&url=http%3A%2F%2Fgreyhound-software.com%2Fdocs%2Fmanual%2Ftexts%2F297_Connect-Addons">Online-Handbuch</a>';
      }
    });

    _showAddonHelp();
    _showTemplateVariables();
  })();

  function _showAddonHelp() {
    RPCCall("getAddonHelp", [], function(response) {
      var node;

      if (response && response.result && response.result.length) {
        node = document.getElementById('addonHelp');
        node.style.display = 'block';
        node.innerHTML = response.result;
      }
    }, function onError(err) { showError(err.error); });
  }

  function _showTemplateVariables() {
    RPCCall("getAddonTemplateVariables", [], _onTemplateVariables, function onError(err) { showError(err.error); });
  }

  //  Seite auf kritischen Fehler setzen:
  function showError(err, title) {
    document.getElementById("error_layerhead").innerHTML = title || "Fehler";
    document.getElementById("error_layercontent").innerHTML = htmlspecialchars(err.message);
    _$errorView.style.left = 0;
    _$errorView.style.display='block';

    _$errorView.style.left = (_$errorView.offsetWidth/2)+"px";
    _$errorOpacityLayer.style.display="block";
  }

  // Aufbau der Seite
  function buildGUI() {
    var i, p;
    var $statusBar, $statusDivider;
    var $page, $pane, $tab;

    // Gesamte Seite in einen Layer packen:
    _$body = document.createElement('DIV');
    document.body.appendChild(_$body);

    // Seiteninhalt:
    $page = document.createElement('DIV');
    $page.className = 'page help';
    _$body.appendChild($page);

    // Hilfetext:
    _$help = document.createElement('DIV');
    _$help.className = 'helpPane';

    _$help.innerHTML = '<h1>Handbuch</h1>' +
      '<p>Die Dokumentation für GREYHOUND Addons finden Sie in unserem <span id="helpManualLink">Online-Handbuch</span>.</p>' +
      '<div id="addonHelp"></div>' +
      '<h1>Variablen für Textbausteine</h1>' +
      '<p>Diese Variablen können in Textbausteinen verwendet werden um Auftragsdaten darzustellen. Sie stehen zur Verfügung, wenn aus dem Addon heraus ' +
      'eine Nachricht mit einer der Schaltflächen "Neu", "Antworten" oder "Weitergeleitet" erstellt wurde.</p>' +
      '<div id="templateVariables"></div>';

    $page.appendChild(_$help);

    // Statuszeile:
    $statusBar = document.createElement('DIV');
    $statusBar.className = 'statusbar';
    
    $statusDivider = document.createElement('DIV');
    $statusDivider.className = 'statusdivider';
    $statusBar.appendChild($statusDivider);

    _$infoView = document.createElement('DIV');
    _$infoView.className = 'info';
    _$infoView.innerHTML = _versionString;
    $statusBar.appendChild(_$infoView);

    _$statusBtnOK = document.createElement('DIV');
    _$statusBtnOK.className = 'button';
    _$statusBtnOK.innerHTML = 'OK';
    _$statusBtnOK.onclick   = function() {window.close()};
    $statusBar.appendChild(_$statusBtnOK);
    
    _$body.appendChild($statusBar);

    // errorView ist ein zusätzliches Layer:
    _$errorOpacityLayer = document.createElement('DIV');
    _$errorOpacityLayer.className = 'errorOpacityLayer';
    _$body.appendChild(_$errorOpacityLayer);

    _$errorView = document.createElement('DIV');
    _$errorView.className = 'errorView';
    _$errorView.innerHTML = '<div id="error_layerhead"></div><div id="error_layercontent"></div><div class="statusbar"><div class="statusdivider"></div><div class="button" id="error_buttonok">OK</div></div>';
    document.body.appendChild(_$errorView);
    document.getElementById('error_buttonok').onclick = function() {
      _$errorView.style.display = 'none';
      _$errorOpacityLayer.style.display = 'none';
    }
  }

  function _onTemplateVariables(response) {
    var i, j, html, row = 'odd', varName, varDesc, varList;

    html = '<table><tr><th>Variable</th><th>Beschreibung</th></tr>';

    for(i = 0; i < response.result.length; i++) {
      varName = response.result[i].name;
      varDesc = response.result[i].desc;
      varList = [];

      if(varName == 'Order.Date' || varName == 'Payment.Date' || varName == 'Shipping.Date' || varName == 'BillingAddress.Birthday' || varName == 'ShippingAddress.Birthday') {
        varList.push({name: varName, desc: varDesc + ' (Tag.Monat.Jahr)'});
        varList.push({name: varName + 'Info.D', desc: varDesc + ' (Tag, z.B. "01")'});
        varList.push({name: varName + 'Info.M', desc: varDesc + ' (Monat, z.B. "02")'});
        varList.push({name: varName + 'Info.Y', desc: varDesc + ' (Jahr, z.B. "2013")'});
        varList.push({name: varName + 'Info.Day.DE', desc: varDesc + ' (Tag auf deutsch, z.B. "Montag")'});
        varList.push({name: varName + 'Info.Day.EN', desc: varDesc + ' (Tag auf englisch, z.B. "Montag")'});
        varList.push({name: varName + 'Info.Day.ES', desc: varDesc + ' (Tag auf spanisch, z.B. "Lunes")'});
        varList.push({name: varName + 'Info.Day.FR', desc: varDesc + ' (Tag auf französisch, z.B. "Lundi")'});
        varList.push({name: varName + 'Info.Month.DE', desc: varDesc + ' (Monat auf deutsch, z.B. "Januar")'});
        varList.push({name: varName + 'Info.Month.EN', desc: varDesc + ' (Monat auf englisch, z.B. "January")'});
        varList.push({name: varName + 'Info.Month.ES', desc: varDesc + ' (Monat auf spanisch, z.B. "Enero")'});
        varList.push({name: varName + 'Info.Month.FR', desc: varDesc + ' (Monat auf französisch, z.B. "Janvier")'});
        varList.push({name: varName + 'Info.Timestamp', desc: varDesc + ' (Unix-Zeitstempel, Sekunden seit 01.01.1970)'});
      }
      else if(varName == 'Order.TotalAmount' || varName == 'Order.OpenAmount' || varName == 'Payment.Costs' || varName == 'Shipping.Costs') {
        varList.push({name: varName, desc: varDesc + ' inkl. Währung'});
        varList.push({name: varName + 'Info.Value', desc: varDesc + ' (Zahl mit Dezimalpunkt)'});
        varList.push({name: varName + 'Info.Currency', desc: varDesc + ' (Währung in drei Buchstaben)'});
      }
      else if(varName == 'BillingAddress.Letter.Country' || varName == 'ShippingAddress.Letter.Country') {
        varList.push({name: varName, desc: varDesc});
        varList.push({name: varName + 'Info.ISO2', desc: varDesc + ' (ISO3166 Länderkürzel: zwei Buchstaben)'});
        varList.push({name: varName + 'Info.ISO3', desc: varDesc + ' (ISO3166 Länderkürzel: drei Buchstaben)'});
        varList.push({name: varName + 'Info.DE', desc: varDesc + ' (deutsche Schreibweise, z.B. "Deutschland")'});
        varList.push({name: varName + 'Info.DE_UC', desc: varDesc + ' (deutsche Schreibweise, Blockbuchstaben, z.B. "DEUTSCHLAND")'});
        varList.push({name: varName + 'Info.EN', desc: varDesc + ' (englische Schreibweise, z.B. "Germany")'});
        varList.push({name: varName + 'Info.EN_UC', desc: varDesc + ' (englische Schreibweise, Blockbuchstaben, z.B. "GERMANY")'});
        varList.push({name: varName + 'Info.ES', desc: varDesc + ' (spanische Schreibweise, z.B. "Alemania")'});
        varList.push({name: varName + 'Info.ES_UC', desc: varDesc + ' (spanische Schreibweise, Blockbuchstaben, z.B. "ALEMANIA")'});
        varList.push({name: varName + 'Info.FR', desc: varDesc + ' (französische Schreibweise, z.B. "Allemagne")'});
        varList.push({name: varName + 'Info.FR_UC', desc: varDesc + ' (französische Schreibweise, Blockbuchstaben, z.B. "ALLEMAGNE")'});
      }
      else
        varList.push({name: varName, desc: varDesc});

      for(j = 0; j < varList.length; j++) {
        html += '<tr class="' + row + '"><td><div onclick="selectText(this);">[$' + varList[j].name + ']</div></td><td>' + varList[j].desc + '</td></tr>';
        row = (row == 'odd' ? 'even' : 'odd');
      }
    }

    html += '</table>';

    document.getElementById('templateVariables').innerHTML = html;
  }

  //  Sendet einen RPC-Call an das Backend
  function RPCCall(RPCMethod, RPCParams, onSuccess, onError) {
    try {
      _rpc.call(RPCMethod, RPCParams, onSuccess, onError);
    } catch(e) {
      _showError(e.message);
    }
  }

  function _showError(message) {
    _$infoView.innerHTML = message;
  }
};

var ghAddonPage = ghAddonHelpPage;

window.onload = function() {
  window.onload = null;   //  verhindern das methode mehrfach aufgerufen wird.
  try { new ghAddonHelpPage() }
  catch(e) { document.body.innerHTML = e.message };
  window.scrollTo(0,0);
};

//  Addon initalisieren sobald DOM fertig geladen; Grafiken könnten hier noch nicht komplett vorhanden sein.
if (document.addEventListener)
  document.addEventListener("DOMContentLoaded", window.onload, false);
