<?php
/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2014-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

if(GhCoreGlobals::getValue('debug'))
{
  error_reporting(E_ALL);
  ini_set('display_errors', true);
}

/**
 * Hilfeseite des Addons.
 *
 * @package GhAddonFramework
 * @subpackage page
 * @category PHP Addon Framework
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2014-2019 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhAddonHelpPage
{
  /**
   * Addon.
   * @var GhAddon
   */
  protected $_addon;

  /**
   * Seitentitel.
   * @var string
   */
  protected $_title;

  /**
   * Webseiteninhalt.
   * @var string
   */
  protected $_html;


  /**
   * Konstruktor.
   *
   * @param GhAddon $addon Addon
   * @param string  $title Seitentitel
   */
  public function __construct($addon, $title = null)
  {
    $this->_addon = $addon;
    $this->_title = $title;

    $this->_initPage();
    $this->_html .= $this->_getPageContent();
  }

  /**
   * Zeigt die Webseiteninhalte an.
   */
  public function show()
  {
    echo $this->_html;
  }

  /**
   * Diese Methode kann überladen werden um vor dem Aufbau der Webseiteninhalte
   * Initialisierungen vorzunehmen.
   */
  protected function _initPage()
  {
  }

  /**
   * Liefert den Anfang der Webseite.
   *
   * @return string
   */
  protected function _getPageContent()
  {
    $graceUi = GhCoreGlobals::get('ghbuild') >= 2453;

    return '<!DOCTYPE html>' . "\r\n" .
      '<html>' . "\r\n" .
        '<head>' . "\r\n" .
          '<meta http-equiv="X-UA-Compatible" content="IE=' . ($graceUi ? 'edge' : 'EmulateIE8') . '">' . "\r\n" .
          '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">' . "\r\n" .
          '<title>' . $this->_addon->getName() . ' Hilfe</title>' . "\r\n" .
          '<link rel="Shortcut Icon" type="image/x-icon" href="/lib/greyhound/favicon.ico">' . "\r\n" .
          '<link rel="stylesheet" type="text/css" media="screen" href="' . GhAddon::getAddonFrameworkUrl() . 'ghaddon.css?v=' . $this->_addon->getBuild() . '">' . "\r\n" .
          '<link rel="stylesheet" type="text/css" media="screen" href="/lib/greyhound/dojo/theme/ghicons' . ($graceUi ? '-graceui' : '') . '.css?v=' . $this->_addon->getBuild() . '">' . "\r\n" .
          '<script type="text/javascript">ghAddonPage="help"; ghAddonTimeout=' . ((int)ini_get('max_execution_time')) . '; ghBuildNo = ' . $this->_addon->getBuild() . '; ghFrameworkPath="' . GhAddon::getAddonFrameworkUrl() . '"; ghIconPath="' . $this->_addon->getIcon() . '";</script>' . "\r\n" .
          '<script type="text/javascript" src="' . GhAddon::getAddonFrameworkUrl() . 'components.js?v=' . $this->_addon->getBuild() . '"></script>' . "\r\n" .
          '<script type="text/javascript" src="' . GhAddon::getAddonFrameworkUrl() . 'ghaddonhelppage.js?v=' . $this->_addon->getBuild() . '"></script>' . "\r\n" .
        '</head>' . "\r\n" .
        '<body id="ghPage" class="' . ($graceUi ? 'ghGraceUI' : 'ghLegacyUI') . '"></body>' .
      '</html>';
  }
}