<?php
/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Datenklasse für Bestelldaten.
 *
 * @package GhAddonFramework
 * @subpackage core
 * @category PHP Addon Framework
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2019 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhAddonOrder
{
  /**
   * ID der Bestellung.
   * @var string
   */
  public $ID;

  /**
   * ID zur Anzeige (z.B. Bestellnummer).
   * @var string
   */
  public $OrderNumber;

  /**
   * ID aus der die Vorgangsnummer generiert wird (z.B. Bestellnummer).
   * @var string
   */
  public $TaskNumberID;

  /**
   * ID des Kunden dieser Bestellung.
   * @var string
   */
  public $CustomerID;

  /**
   * Kundennummer dieser Bestellung.
   * @var string
   */
  public $CustomerNumber;

  /**
   * ID des Kunden und der Bestellung (für die Bestellauswahlbox).
   * Diese ID wird von der Sidebar-Seite gesetzt.
   * @var string
   */
  public $CombinedID;

  /**
   * Kurzbeschreibung / Listendarstellung der Bestellung.
   * @var string
   */
  public $Name;

  /**
   * Icon für die Listendarstellung der Bestellung.
   * @var string
   */
  public $Icon = 'shoppingcart';

  /**
   * Datum der Bestellung.
   * @var GhRpcDateTime
   */
  public $Date;

  /**
   * Rechnungsdatum.
   * @var GhRpcDateTime
   */
  public $InvoiceDate;

  /**
   * Status der Bestellung.
   * @var string
   */
  public $Status;

  /**
   * Rechnungsadresse des Kunden.
   * Die Adresse wird als Array mit folgenden Schlüsseln abgelegt:
   * Company, Salutation, Title, Name, Firstname, Lastname, Street, Street2, ZipCode, City, Province, Country, Email, Phone, Fax.
   * @var array
   */
  public $BillingAddress = array(
    'Company' => '',
    'Salutation' => '',
    'Gender' => '',
    'Title' => '',
    'Name' => '',
    'Firstname' => '',
    'Lastname' => '',
    'Street' => '',
    'AdditionalInfo' => '',
    'ZipCode' => '',
    'City' => '',
    'Province' => '',
    'Country' => '',
    'CountryISO2' => '',
    'Email' => '',
    'Phone' => '',
    'Fax' => ''
  );

  /**
   * Lieferadresse des Kunden.
   * Die Adresse wird als Array mit folgenden Schlüsseln abgelegt:
   * Company, Salutation, Title, Name, Firstname, Lastname, Street, Street2, ZipCode, City, Province, Country, Email, Phone, Fax.
   * @var array
   */
  public $ShippingAddress = array(
    'Company' => '',
    'Salutation' => '',
    'Gender' => '',
    'Title' => '',
    'Name' => '',
    'Firstname' => '',
    'Lastname' => '',
    'Street' => '',
    'AdditionalInfo' => '',
    'ZipCode' => '',
    'City' => '',
    'Province' => '',
    'Country' => '',
    'CountryISO2' => '',
    'Email' => '',
    'Phone' => '',
    'Fax' => ''
  );

  /**
   * Verkaufsplattform oder Drittsystem.
   * @var string
   */
  public $Platform;

  /**
   * Benutzername oder ID in der Verkaufsplattform bzw. dem Drittsystem.
   * @var string
   */
  public $PlatformUsername;

  /**
   * Versanddatum.
   * @var GhRpcDateTime
   */
  public $ShippingDate;

  /**
   * Versanddienstleister.
   * @var string
   */
  public $ShippingProvider;

  /**
   * Versandkosten.
   * @var float
   */
  public $ShippingCosts = 0.0;

  /**
   * Rechnungsnummer.
   * @var string
   */
  public $InvoiceNumber;

  /**
   * Zahlungsart.
   * @var string
   */
  public $PaymentMethod;

  /**
   * Kosten der Zahlungsart.
   * @var float
   */
  public $PaymentCosts = 0.0;

  /**
   * Zahlungsdatum.
   * @var GhRpcDateTime
   */
  public $PaymentDate;

  /**
   * Transaktions-ID des Zahlungsanbieters.
   * @var string
   */
  public $PaymentTransactionID;

  /**
   * Zahlungsziel in Tagen.
   * @var int
   */
  public $TermOfPayment;

  /**
   * Währung der Bestellung.
   * @var string
   */
  public $Currency;

  /**
   * Gesamtbetrag der Bestellung.
   * @var float
   */
  public $TotalAmount;

  /**
   * Noch offener Betrag der Bestellung.
   * @var float
   */
  public $OpenAmount;

  /**
   * Tracking-Informationen.
   * Der Array enthält Arrays mit den Keys 'Code' (Sendungsnummer), 'Url' (Tracking-Link),
   * 'Provider' (Versanddienstleister) und 'Date' (Versanddatum: GhRpcDateTime Objekt oder null).
   * @var array
   */
  public $Tracking = array();

  /**
   * Beschreibung der Bestellung.
   * Ist die Beschreibung mehrzeilig, so liegt sie als Array vor.
   * Die Beschreibung wird z.B. für Tooltips verwendet.
   * @var array|string
   */
  public $Description;

  /**
   * Detailinformationen der Bestellung.
   * Jeder Eintrag ist ein Objekt mit den Eigenschaften 'Icon', 'Caption' und
   * 'Value'. Die Eigenschaften 'Caption' und 'Value' können ein String oder
   * ein Array von Strings sein (bei mehrzeiligen Daten).
   * @var array
   */
  public $OrderInfo = array();

  /**
   * Zusätzliche Detailinformationen der Bestellung.
   * Diese Informationen werden als GhGuiText unter dem DataGrid mit den
   * $orderInfo Einträgen angezeigt und immitieren die Darstellung im DataGrid.
   * Jeder Eintrag ist ein Objekt mit den Eigenschaften 'Icon', 'Caption' und
   * 'Value'. Die Eigenschaft 'Value' kann ein String oder ein Array von Strings
   * sein (bei mehrzeiligen Daten).
   * @var array
   */
  public $OrderTextInfo = array();

  /**
   * Bestellpositionen der Bestellung.
   * @var GhAddonOrderItem[]
   */
  public $OrderItems = array();

  /**
   * Zusatzfunktionen für die Bestellung.
   * Enthält Arrays mit den Schlüsseln 'Icon', 'Caption', 'Tooltip', 'Url', 'Enabled'.
   * @var array
   */
  public $OrderActions = array();

  /**
   * Beliebige zusätzliche Daten der Bestellung.
   * @var array
   */
  public $Data;

  /**
   * Gibt an, ob die Bestellung Detaildaten oder nur allgemeine Daten enthält.
   * @var boolean
   */
  public $HasDetails = false;

  public $OrderTaskNumberID;

  public $DebugFile;
  
  /**
   * Setzt die Auftragsdaten.
   *
   * @param mixed $data Auftragsdaten
   */
  public function setData($data)
  {
    $this->Data = $data;
  }
}