<?php
/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Datenklasse für Bestellpositionen.
 *
 * @package GhAddonFramework
 * @subpackage core
 * @category PHP Addon Framework
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2019 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhAddonOrderItem
{
  /**
   * ID der Bestellposition.
   * @var string
   */
  public $ID;

  /**
   * Artikelnummer.
   * @var string
   */
  public $ItemNumber;

  /**
   * Titel der Bestellposition (für die Darstellung).
   * @var string
   */
  public $Name;

  /**
   * Beschreibung der Bestellposition (für die Darstellung).
   * Ist die Beschreibung mehrzeilig, so liegt sie als Array vor.
   * @var string|array
   */
  public $Description;

  /**
   * Anzahl der Bestellposition.
   * @var int
   */
  public $Quantity;

  /**
   * Währung der Bestellposition.
   * @var string
   */
  public $Currency;

  /**
   * Preis einer Einheit der Bestellposition.
   * @var float
   */
  public $UnitPrice;

  /**
   * Gesamtpreis der Bestellposition (Anzahl mal Einzelpreis).
   * @var float
   */
  public $TotalPrice;

  /**
   * Unterprodukte aus Produkt-Sets.
   * @var GhAddonOrderItem[]
   */
  public $ProductSetItems = array();

  /**
   * Unterprodukte aus Varianten-Sets.
   * @var GhAddonOrderItem[]
   */
  public $VariantItems = array();

  /**
   * Beliebige zusätzliche Daten der Bestellposition.
   * @var array
   */
  public $Data;

  /**
   * Setzt die Daten für die Bestellposition.
   *
   * @param mixed $data Daten
   */
  public function setData($data)
  {
    $this->Data = $data;
  }
}