/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

// Vorschauzeile:
var ghAddonPreviewLinePage = new (function() {
  "use strict";

  var _self = this;
  var _hash = {};
  var _data = null;
  var _selectedOrderId = null;
  var _templateId = null;
  var _newButtonKind = '';
  var _answerButtonKind = 'iconAndText';
  var _forwardButtonKind = '';

  var _$contentNode = null;

  this.rpc = null;
  this.dataManager = null;
  this.beforeInit = null;
  this.afterInit = null;
  this.init = init;
  this.GHAction = GHAction;

  function init() {
    var haveError = false;

    _hash = extractHashValues();

    _self.rpc = new ghAddonRpc();
    _self.rpc.setSettings(_hash);
    _self.dataManager = new ghAddonDataManager(_self.rpc);

    if (_self.beforeInit) {
      _self.beforeInit(_self);
    }

    _$contentNode = document.getElementById('ghPreviewLineContent');
    _$contentNode.style.height = (document.documentElement.clientHeight - 6) + 'px';

    _self.rpc.call('getAddonInfo', [],
      function(response) {
        if(response && response.result) {
          _self.rpc.settingsTimestamp = response.result.settingsTimestamp;
        }
      },
      function(err) {
        haveError = true;
        _onError(err);
      }
    );

    if(!haveError) {
      _self.rpc.call('getAddonSettings', [],
        function(response) {
          //Key-Value Paare in Objekt umwandeln:
          var fixedSettings = {};

          for (var i = 0; i < response.result.length; i++) {
            fixedSettings[response.result[i].key] = response.result[i].value;
          }

          response.result = fixedSettings;

          _self.dataManager.setSettings(response.result, null);
          _newButtonKind = fixedSettings.PreviewLineNew ? fixedSettings.PreviewLineNew : 'none';
          _answerButtonKind = fixedSettings.PreviewLineAnswer ? fixedSettings.PreviewLineAnswer : 'iconAndText';
          _forwardButtonKind = fixedSettings.PreviewLineForward ? fixedSettings.PreviewLineForward : 'none';
          invalidateData();
        },
        function(err) {
          haveError = true;
          _onError(err);
        }
      );

      _self.rpc.onAddonSettingsChanged = function(settings) {
        _self.dataManager.setSettings(settings);
        _newButtonKind = settings.PreviewLineNew ? settings.PreviewLineNew : 'none';
        _answerButtonKind = settings.PreviewLineAnswer ? settings.PreviewLineAnswer : 'iconAndText';
        _forwardButtonKind = settings.PreviewLineForward ? settings.PreviewLineForward : 'none';
      };
    }

    //  Hashchange / Änderungen der Suchanfrage abfangen
    if (!haveError) {
      if (window.addEventListener) {
        window.addEventListener("hashchange", invalidateData, false);
      } else if(window.attachEvent) {
        window.attachEvent("onhashchange", invalidateData);
      } else {
        window.onhashchange = invalidateData;
      }
    }

    if (_self.afterInit) {
      _self.afterInit(_self);
    }
  }

  function _onSuccess(response)
  {
    _data = response;

    if (!response || !response.result || response.result.length == 0) {
      _$contentNode.innerHTML = '<div class="ghMessage">Es wurden keine Daten gefunden.</div>';
    } else {
      _selectedOrderId = response.result.selectedOrderId;
      _self.rpc.call('previewline', [_self.dataManager.account.id, response.result.selectedOrderId, response.result.data, _hash],
        function onSuccess(response) {
          var newButton, answerButton, forwardButton, dataNode, keyNode, valueNode;

          if (!response.result || response.result.length == 0) {
            _$contentNode.innerHTML = '<div class="ghMessage">Es wurden keine Daten gefunden.</div>';
          } else {
            while (_$contentNode.firstChild) {
              _$contentNode.removeChild(_$contentNode.firstChild);
            }

            if (_newButtonKind != 'none') {
              newButton = new ghButton(_$contentNode, {
                caption: _newButtonKind == 'iconAndText' || _newButtonKind == 'text' ? 'Neue E-Mail' : '',
                icon: _newButtonKind == 'iconAndText' || _newButtonKind == 'icon' ? 'mail_new' : '',
                className: _newButtonKind == 'icon' ? 'ghIconOnly' : '',
                onClick: function() {
                  _self.GHAction('newEmail');
                  return false;
                }
              });
            }

            if (_hash.itemID && _answerButtonKind != 'none') {
              answerButton = new ghButton(_$contentNode, {
                caption: _answerButtonKind == 'iconAndText' || _answerButtonKind == 'text' ? 'Antworten' : '',
                icon: _answerButtonKind == 'iconAndText' || _answerButtonKind == 'icon' ? 'mail_answer' : '',
                className: _answerButtonKind == 'icon' ? 'ghIconOnly' : '',
                onClick: function() {
                  _self.GHAction('replyEmail');
                  return false;
                }
              });
            }

            if (_hash.itemID && _forwardButtonKind != 'none') {
              forwardButton = new ghButton(_$contentNode, {
                caption: _forwardButtonKind == 'iconAndText' || _forwardButtonKind == 'text' ? 'Weiterleiten' : '',
                icon: _forwardButtonKind == 'iconAndText' || _forwardButtonKind == 'icon' ? 'mail_forward' : '',
                className: _forwardButtonKind == 'icon' ? 'ghIconOnly' : '',
                onClick: function() {
                  _self.GHAction('forwardEmail');
                  return false;
                }
              });
            }

            for (var i = 0; i < response.result.length; i++) {
              dataNode = document.createElement('DIV');
              dataNode.className = 'ghData';
              _$contentNode.appendChild(dataNode);

              if (response.result[i].key) {
                keyNode = document.createElement('DIV');
                keyNode.className = 'ghKey';
                keyNode.innerHTML = response.result[i].key;
                dataNode.appendChild(keyNode);
              }

              valueNode = document.createElement('DIV');
              valueNode.className = 'ghValue';
              valueNode.innerHTML = response.result[i].value;
              dataNode.appendChild(valueNode);
            }
          }
        },
        function onError(response) {
          _$contentNode.innerHTML = '<div class="ghError">Es ist ein Fehler beim Verarbeiten der Daten aufgetreten.</div>';
        }
      );
    }
  }

  function _onError(response)
  {
    if (response !== undefined && response.error !== undefined && response.error.name !== undefined && response.error.name == 'GhAddonEmptySearchException') {
      _$contentNode.innerHTML = response.error.message;
    } else {
      var msg = 'Es konnten keine Daten abgerufen werden.';

      if(response.error && response.error.message)
        msg = response.error.message;

      _$contentNode.innerHTML = '<div class="ghError">' + msg + '</div>';
    }
  }

  function invalidateData() {
   _$contentNode.innerHTML = '<div class="ghMessage">Daten werden abgerufen...</div>';

    _hash = extractHashValues();
    _self.rpc.setSettings(_hash);

    try {
      _self.dataManager.invalidateData(_hash, _onSuccess, _onError, null, null);
    } catch(e) {
      _onError(createJsonError(e.message));
    }
  }

  //  Startet eine Aktion im GH, namentlich das Öffnen einer Email/Fax/Briefes mit optional ausgewählter Textvorlage
  function GHAction(action) {
    var isReply, isForward, isReference, isCustomSearch, kind, iframe = document.createElement('IFRAME'), form = document.createElement('FORM');
    var customerEmail, customerFax, customerAddress, customerPhone, ccEmail, toEmail, attachments, i;

    function addEntry(key,value) {
      var p = document.createElement('INPUT');
      p.name = key;
      p.value = value;
      p.type = 'hidden';
      form.appendChild(p);
    };

    isCustomSearch = false;
    iframe.name = 'GHActionFrame';
    document.body.appendChild(iframe);

    form.action = 'greyhound:NewItem';
    form.method = 'POST';
    form.target = 'GHActionFrame';
    document.body.appendChild(form);

    if (action) {
      if (action.substr(0, 5) == 'reply')
        form.action = 'greyhound:AnswerItem';
      else if (action.substr(0, 7) == 'forward')
        form.action = 'greyhound:ForwardItem';
    }

    //  Optionale Parameter:
    if (action == 'newEmail') {
      if (_self.dataManager.email) {
        toEmail = _self.dataManager.email;
      } else if(_self.dataManager.origEmail) {
        toEmail = _self.dataManager.origEmail;
      } else {
        toEmail = '';
      }
    } else if (_self.dataManager.origEmail) {
      toEmail = _self.dataManager.origEmail;
    } else {
      toEmail = _self.dataManager.email ? _self.dataManager.email : (_selectedCustomer ? _selectedCustomer.Email : '');
    }

    if (_hash.itemState != RpcItemState.isOpen && _hash.itemState != RpcItemState.isDone) {
      addEntry('DisallowDone', 'True');
    }

    switch(action) {
      case 'newEmail':
      case 'replyEmail':
      case 'forwardEmail':
        isReply   = (action == 'replyEmail');
        isForward = (action == 'forwardEmail');
        isReference = !isCustomSearch || isReply || isForward;
        kind      = RpcItemKind.ikEmail;
        addEntry('Kind', 'EMail');
        addEntry('Recipients[0]', isForward ? '' : toEmail);
        addEntry('RecipientsClear', 'True');

        if(ccEmail)
          addEntry('CCs[0]', ccEmail);
        break;

      case 'newFax':
      case 'replyFax':
      case 'forwardFax':
        isReply   = (action == 'replyFax');
        isForward = (action == 'forwardFax');
        isReference = !isCustomSearch || isReply || isForward;
        kind      = RpcItemKind.ikFax;
        addEntry('Kind', 'Fax');
        addEntry('Recipients[0]', isForward ? '' : customerFax);
        addEntry('RecipientsClear', 'True');
        break;

      case 'newLetter':
      case 'replyLetter':
      case 'forwardLetter':
        isReply   = (action == 'replyLetter');
        isForward = (action == 'forwardLetter');
        isReference = !isCustomSearch || isReply || isForward;
        kind      = RpcItemKind.ikLetter;
        addEntry('Kind', 'Letter');
        addEntry('Recipients[0]', isForward ? '' : customerAddress);
        addEntry('RecipientsClear', 'True');
        break;

      case 'newCall':
      case 'replyCall':
      case 'forwardCall':
        isReply   = (action == 'replyCall');
        isForward = (action == 'forwardCall');
        isReference = !isCustomSearch || isReply || isForward;
        kind      = RpcItemKind.ikCall;
        addEntry('Kind', 'Call');
        addEntry('Recipients[0]', isForward ? '' : customerPhone);
        addEntry('RecipientsClear', 'True');
        break;
    }

    if (isReference && _hash.itemTaskNumber) {
      addEntry('Number', _hash.itemTaskNumber);
    }

    if (_self.addAttachments) {
      try {
        attachments = _self.addAttachments();

        if(Array.isArray(attachments)) {
          for(i = 0; i < attachments.length; i++) {
            addEntry('Attachments[' + i + ']', attachments[i]);
          }
        }
      } catch(e) {}
    }

    //  Original Email besorgen:
    if (_self.dataManager.account) {
      try {
        var itemId = isReply || isForward ? _hash.itemID : 0;
        var getItemParams = [_self.dataManager.account.id, itemId, kind, true, _hash.itemGroup, _selectedOrderId, _data && _data.result ? _data.result.data : null, _templateId];

        if (_self.dataManager.sendHash) {
          getItemParams.push(_hash);
        }

        _self.rpc.call(
          'getItem',
          getItemParams,
          function onItem(result) {
            var a, i, k, item = result.result, content = item.content, subject = '', signature = item.signature || '', template = '';

            if (itemId) {
              subject = item.subject;
            }

            if (item.template) {
              template = item.template.Content;
              subject = item.template.Subject || item.subject;
            }

            //  Kopfdaten hinzufügen

            if (isReference && itemId) {
              addEntry("ItemID", itemId);
            }

            addEntry("Group", _hash.itemGroup);
            addEntry("DataMode", item.isHTML ? "Html" : "Text");
            addEntry("Subject", subject);

            //  Template-Variablen
            for (k in item.templateVars) {
              addEntry(k, item.templateVars[k]);
            }

            if(itemId && item.isHTML) {
              if(content) {
                content = content.replace('<p>&nbsp;</p><div class="quoteheader"', '<div class="quoteheader"');
                template = content.replace('<div class="quoteheader"', "<p>\r\n" + (template ? template + "\r\n" : "") + "</p>" + signature + '<p>&nbsp;</p><div class="quoteheader"');
              }
            } else {
              template = template + signature + "\r\n" + content;
            }

            for (a = template.split("\n"), i = 0; i < a.length; i++) {
              addEntry("Data" + (i + 1), trim(a[i]));
            }

            try {
              form.submit();
            } catch (e) {
              debug(e);
            }
          },
          function onError(err) {
            _onError(err.error);
          },
          false
        );
      } catch (e) {}
    }
  }

})();

var ghAddonPage = ghAddonPreviewLinePage;

window.onload = function() {
  window.onload = null; // Verhindern, dass die methode mehrfach aufgerufen wird.
  try { ghAddonPreviewLinePage.init() }
  catch(e) { document.body.innerHTML = e.message };
  window.scrollTo(0,0);
};

// Addon initalisieren sobald DOM fertig geladen; Grafiken könnten hier noch nicht komplett vorhanden sein.
if (document.addEventListener) {
  document.addEventListener("DOMContentLoaded", window.onload, false);
}
