/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

// Einstellungsdialog:
var ghAddonSettingsPage = new (function () {
  "use strict";

  //  
  //  Interface (public)
  //
  this.actions = {};
  this.init                 = init;
  this.connectionRenderer   = _connectionRenderer;
  this.globalRenderer       = _globalRenderer;
  this.datamigrationRenderer= _datamigrationRenderer;
  this.previewlineRenderer  = _previewlineRenderer;
  this.trackingRenderer     = _trackingRenderer;
  this.extendedRenderer     = _extendedRenderer;
  this.RPCCall              = RPCCall;
  this.sendDebugLog         = sendDebugLog;
  this.components = {};

  //
  //  Lokale Variablen (private)
  //
  var _self = this;
  var _uniqueRpcId = 0;
  var _versionString  = '';

  var _hash           = {};
  var _rpc;
  var _settings       = {};
  var _accounts       = [];     //  Konfigurierte Accounts/Verbindungen
  var _accountOffset  = 0;      //  Offset auf Account
  var _previewLines   = [];     //  e.g. [ { id: 0, Var: 'Order.OrderNumber', Caption: 'Bestellnummer'} ... ]
  var _userFieldLines = [];     //  e.g. [ { id: 0, Var: "Order.InvoiceNumber", Field: "6"} ... ]
  var _trackingLinks  = [];
  var _connectionListItems = {};

  var _globalOptions;
  var _accountOptions;
  var _extendedOptions;
  var _accountDefaults;
  var _templateVariableDescriptions;
  var _userDefinedFields;
  var _guiId = 1000;

  var _panes = {
    'connect':  {'Label': 'Verbindung',    'Element': null, 'Active': false, Visible: true, 'Class': 'connect',  'Tab': null, 'Renderer': 'connectionRenderer'},
    'global':   {'Label': 'Global',        'Element': null, 'Active': false, Visible: true, 'Class': 'global',   'Tab': null, 'Renderer': 'globalRenderer'},
    'migration':{'Label': 'Datenübernahme','Element': null, 'Active': false, Visible: true, 'Class': 'migration','Tab': null, 'Renderer': 'datamigrationRenderer'},
    'preview':  {'Label': 'Vorschauzeile', 'Element': null, 'Active': false, Visible: true, 'Class': 'preview',  'Tab': null, 'Renderer': 'previewlineRenderer'},
    'tracking': {'Label': 'Tracking',      'Element': null, 'Active': false, Visible: true, 'Class': 'tracking', 'Tab': null, 'Renderer': 'trackingRenderer'},
    'extended': {'Label': 'Erweitert',     'Element': null, 'Active': false, Visible: true, 'Class': 'extended', 'Tab': null, 'Renderer': 'extendedRenderer'}
  };
  var _panesOrder = ['connect', 'global', 'migration', 'preview', 'tracking', 'extended'];
  var _activeTab  = 'connect';

  //  Instanzen verwendeter Komponenten
  var _$infoView, _$errorView, _$body, _$errorOpacityLayer, _$tabbar, _$statusBtnOK, _$statusBtnCancel;


  //
  //  Constructor
  //
  function init() {
    _hash = extractHashValues();

    _rpc = new ghAddonRpc();
    _rpc.setSettings(_hash);

    _rpc.call('getAddonInfo', [], function(response) {
      if(response && response.result) {
        _versionString = 'Version ' + response.result.version + ' build ' + response.result.build;
        _$infoView.innerHTML = _versionString;
        _rpc.settingsTimestamp = response.result.settingsTimestamp;
      }
    });

    buildGUI();
    getAddonSettings();
  }

  function getAddonSettings() {
    //  Abruf der AddonSettings
    RPCCall("getAddonSettingsOptions", [],
      function onAccounts(response) {
        var key, tmp, i, id, val, accountData = {}, previewData = {}, userFieldData = {}, trackingData = {};
        //  Initialisierung Klassenlokaler Variablen
        _accounts       = []; 
        _accountOffset  = 0;  
        _previewLines   = []; 
        _userFieldLines = []; 
        _trackingLinks  = [];

        if (response.result && response.result.pages) {
          _panesOrder = response.result.pages || [];

          if (_panesOrder.indexOf(_activeTab) < 0 && _panesOrder.length > 0) {
            _activeTab = _panesOrder[0];
          }

          for (key in _panes) {
            if (_panes.hasOwnProperty(key) && _panesOrder.indexOf(key) < 0) {
              _panes[key].Visible = false;

              if (_panes[key].Tab) {
                _panes[key].Tab.style.display = 'none';
              }
            }
          }
        }

        //Key-Value Paare in Objekt umwandeln:
        var fixedSettings = {};

        for(i = 0; i < response.result.settings.length; i++)
          fixedSettings[response.result.settings[i].key] = response.result.settings[i].value;

        response.result.settings = fixedSettings;

        _globalOptions    = response.result.global || [];
        _accountOptions   = response.result.account || [];
        _extendedOptions  = response.result.extended || [];
        _accountDefaults  = response.result.accountDefaults || {};
        _accountDefaults.recipients = _accountDefaults.Recipient0 || '*';

        _templateVariableDescriptions = response.result.templateVariableDescriptions || {};
        _userDefinedFields= response.result.userDefinedFields || [];

        for(key in response.result.settings)
        {
          val = response.result.settings[key];
          if (key.match(/^Account\.(\d+)\.(.*)$/))
          {
            tmp = key.match(/^Account\.(\d+)\.(.*)$/);
            id  = tmp[1];
            key = tmp[2];
            
            if(!accountData[id]) 
              accountData[id] = { Activated: false, Caption: 'Neue Verbindung', id: id, recipients: '' }; //  Default erstellen

            if(key.match(/^Recipient\d+$/))
            {
              accountData[id].recipients = trim(accountData[id].recipients + '\r\n' + val + '\r\n') ;
            }
            else
            {
              accountData[id][key] = val;            
            }
          }
          else if (key.match(/^PreviewLineField\.(\d+)\.(.*)$/))
          {
            tmp = key.match(/^PreviewLineField\.(\d+)\.(.*)$/);
            id  = tmp[1];
            key = tmp[2];

            if (!previewData[id])
              previewData[id] = {id: id};

            previewData[id][key] = val;
          }
          else if (key.match(/^UserFieldVar\.(\d+)\.(.*)$/))
          {
            tmp = key.match(/^UserFieldVar\.(\d+)\.(.*)$/);
            id  = tmp[1];
            key = tmp[2];

            if (!userFieldData[id])
              userFieldData[id] = {id: id};

            userFieldData[id][key] = val;
          }
          else if (key.match(/^TrackingLink\.(\d+)\.(.*)$/))
          {
            tmp = key.match(/^TrackingLink\.(\d+)\.(.*)$/);
            id  = tmp[1];
            key = tmp[2];

            if (!trackingData[id])
              trackingData[id] = {id: id};

            trackingData[id][key] = val;
          }
          else
          {
            _settings[key] = val;
          }
        }
    
        //  Accountdaten durchgehen; inaktive löschen und leere Recipientsfilter behandeln
        for(key in accountData) 
        {
          
          if(accountData[key].recipients !== undefined && !accountData[key].recipients.length)
            accountData[key].recipients = '*';

          _accounts.push(accountData[key]);
        }

        //  ggf. einen Default-Account anlegen
        if(_accounts.length == 0)
          _newAccount();

        for(key in previewData)
          _previewLines.push( previewData[key] );

        for(key in userFieldData)
          _userFieldLines.push( userFieldData[key] );

        for(key in trackingData)
          _trackingLinks.push( trackingData[key] );

        _$tabbar.style.display = 'block';

        _$statusBtnOK.style.display = 'inline-block';

        // Den 1. Reiter aktivieren
        _panes[_activeTab].Tab.click();
      }, 
      function onError(err) {
        showError(err.error);
        _$statusBtnOK.style.display = 'none';
      }
    );
  };

  //  Fügt neuen Account hinzu und setzt passenden AccountOffset
  function _newAccount(account) {
    account = createOptions(account || _accountDefaults);
    account.id = _accounts.length;
    _accounts.push( account );       
    _accountOffset = _accounts.length - 1;   
    _connectionRenderer();
  };

  //  Seite auf kritischen Fehler setzen:
  function showError(err, title) {
    document.getElementById("error_layerhead").innerHTML = title || "Fehler";
    document.getElementById("error_layercontent").innerHTML = htmlspecialchars(err.message);
    _$errorView.style.left = 0;
    _$errorView.style.display='block';

    _$errorView.style.left = (_$errorView.offsetWidth/2)+"px";
    _$errorOpacityLayer.style.display="block";
  };

  //  Aufbau Accountinformationen
  function buildGUI() {
    var i, p;
    var $statusBar, $statusDivider;
    var $page, $pane, $tab;

    //  Gesamte Seite in einem Layer packen:
    _$body = document.createElement("DIV");
    document.body.appendChild(_$body);

    //  Reiterbox
    $page = document.createElement('DIV');
    $page.className = 'page';
    _$tabbar = document.createElement('DIV');
    _$tabbar.className = 'tabbar';
    _$tabbar.style.display = 'none';
    $page.appendChild(_$tabbar);
    _$body.appendChild($page);

    //  Panes und Tabs aufbauen
    for(i=0; i< _panesOrder.length; i++) {
      p = _panes[_panesOrder[i]];

      $pane = document.createElement('DIV');
      $pane.className = 'pane ' + p.Class;
      $page.appendChild($pane);
      p.Element = $pane;
      p.Active  = false;

      $tab = document.createElement('DIV');
      $tab.className = 'tab';
      $tab.innerHTML = p.Label;
      p.Tab  = $tab;

      _$tabbar.appendChild($tab);

      (function(pane) {
        $tab.onclick = function() {
          for(var k in _panes)
          {
            if (pane == _panes[k]) {
              _panes[k].Active = true;
              _panes[k].Element.className     = 'pane ' + _panes[k].Class + ' active';
              _panes[k].Tab.className         = 'tab active';
              _activeTab = k;
            } else {
              _panes[k].Active = false;
              _panes[k].Element.className     = 'pane ' + _panes[k].Class;
              _panes[k].Tab.className         = 'tab';
            }
          }
    
          //  "Pane" mit Leben füllen (kann überschrieben werden)
          try {
            _self[pane.Renderer]();
          } catch(e) {
            pane.Element.innerHTML = e.message;
          }
        }
      })(p);
    }

    //  Statusbar
    $statusBar = document.createElement('DIV');
    $statusBar.className = 'statusbar';
    
    $statusDivider = document.createElement('DIV');
    $statusDivider.className = 'statusdivider';
    $statusBar.appendChild($statusDivider);

    _$infoView = document.createElement('DIV');
    _$infoView.className = 'info';
    _$infoView.innerHTML = _versionString;
    $statusBar.appendChild(_$infoView);

    _$statusBtnCancel = document.createElement('DIV');
    _$statusBtnCancel.className = 'button';
    _$statusBtnCancel.innerHTML = 'Abbrechen';
    _$statusBtnCancel.onclick   = function() {window.close()};
    $statusBar.appendChild(_$statusBtnCancel);

    _$statusBtnOK = document.createElement('DIV');
    _$statusBtnOK.className = 'button';
    _$statusBtnOK.style.display = 'none';
    _$statusBtnOK.innerHTML = 'OK';
    _$statusBtnOK.onclick   = _saveSettings;
    $statusBar.appendChild(_$statusBtnOK);
    
    _$body.appendChild($statusBar);

    //  errorView ist neu ein Layer
    _$errorOpacityLayer = document.createElement("DIV");
    _$errorOpacityLayer.className="errorOpacityLayer";
    _$body.appendChild(_$errorOpacityLayer);

    _$errorView = document.createElement('DIV');
    _$errorView.className = 'errorView';
    _$errorView.innerHTML = '<div id="error_layerhead"></div><div id="error_layercontent"></div><div class="statusbar"><div class="statusdivider"></div><div class="button" id="error_buttonok">OK</div></div>';
    document.body.appendChild(_$errorView);
    document.getElementById("error_buttonok").onclick = function() {
      _$errorView.style.display="none";
      _$errorOpacityLayer.style.display="none";
    }
  };

  //  Sendet einen RPC-Call an das Backend
  function RPCCall(RPCMethod, RPCParams, onSuccess, onError) {
    try {
      _rpc.call(RPCMethod, RPCParams, onSuccess, onError);
    } catch(e) {
      _showError(e.message);
    }
  };

  function _showError(message) {
    _$infoView.innerHTML = message;
  };

  //
  //  Implementation
  //
  function _saveSettings() {
    var settings, i, k, r, x, y;

    //  Kopie interner settings:
    settings = createOptions(_settings);

    //    Account.X.Y       [hier muss zudem noch Recipient0 etc behandelt werden]
    for(i = 0; i < _accounts.length; i++)
    {
      for(k in _accounts[i])
      {
        if (k != 'id' && k != 'recipients')
          settings['Account.'+i+'.'+k] = _accounts[i][k];
        else if(k=='recipients') {
          for(r = _accounts[i][k].split('\n'), y = x = 0; x < r.length; x++)
            if(trim(r[x]))
              settings['Account.' + i + '.Recipient' + y++] = r[x];
        }
      }
    }

    //    UserFieldVar.X.Y
    for(i = 0; i < _userFieldLines.length; i++)
    {
      settings['UserFieldVar.'+i+'.Var']    = _userFieldLines[i].Var;
      settings['UserFieldVar.'+i+'.Field']  = _userFieldLines[i].Field;
    }

    //    PreviewLineField.X.Y
    for(i = 0; i < _previewLines.length; i++)
    {
      settings['PreviewLineField.'+i+'.Var']      = _previewLines[i].Var;
      settings['PreviewLineField.'+i+'.Caption']  = _previewLines[i].Caption || '';
    }

    //    TrackingLink.X.Y
    for(i = 0; i < _trackingLinks.length; i++)
    {
      settings['TrackingLink.'+i+'.Prefix']   = _trackingLinks[i].Prefix;
      settings['TrackingLink.'+i+'.Provider'] = _trackingLinks[i].Provider;
      settings['TrackingLink.'+i+'.Url']      = _trackingLinks[i].Url;
    }

    RPCCall(
      'saveSettings', 
      settings,
      function onSuccess(response) {
        if(response.result)
          _rpc.settingsTimestamp = response.result;

        window.close();
      }, 
      function onError(err) {
        showError(err.error, "Sicherung der Einstellungen");
      }
    );
  };

  //  Führt eine "button-action" aus
  function callAction(elem, property, value) {

    function onError(data) {
      waitIcon.className = 'icon right ghGuiDefaultIcons forbidden2';
      elem.onClick = clickHandler;
      showError(data.error, elem.innerHTML);
    }

    var waitIcon, clickHandler;

    if (elem.domNode.firstChild.nodeName == 'SPAN') {
      waitIcon = elem.domNode.firstChild;
    } else {
      waitIcon = document.createElement('SPAN');
      elem.domNode.insertBefore(waitIcon, elem.domNode.firstChild);
    }

    waitIcon.className = "icon right ghSpin";

    clickHandler = elem.onClick;
    elem.onClick = null;

    if (typeof _self.actions[value] == 'function') {
      _self.actions[value](function() {
        waitIcon.className = 'icon right ghGuiDefaultIcons check2';
        elem.onClick = clickHandler;
      }, onError, {
        accountId: _accountOffset,
        settings: _accounts[_accountOffset]
      });
    } else {
      switch (value) {
        
        case 'checkConnection':
          RPCCall("checkConnection",
            [_accounts[_accountOffset]], 
            function onSuccess(result) {
              waitIcon.className = 'icon right ghGuiDefaultIcons check2';
              elem.onClick = clickHandler;
            }, 
            onError
          );
          break;

        case 'createUserIdUserField':
          RPCCall("createUserIdUserField",
            [], 
            function() {
              getAddonSettings();
              elem.onClick = clickHandler;
            }, 
            onError
          );
          break;

        case 'sendDebugLog':
          RPCCall('sendDebugLog',
            [],
            function onSuccess(result) {
              sendDebugLog(result.result);
              waitIcon.className = 'icon right ghGuiDefaultIcons check2';
              elem.onClick = clickHandler;
            },
            onError
          );
          break;
        
        default:
          onData({result: 'invalid action called! ('+value+')'});
          break;
      }
    }
  }

  function sendDebugLog(data) {
    var iframe, form, lines, i;

    function addEntry(key,value) {
      var p = document.createElement('INPUT');
      p.name = key;
      p.value = value;
      p.type = 'hidden';
      form.appendChild(p);
    };

    iframe = document.createElement('IFRAME');
    iframe.name = 'GHActionFrame';
    document.body.appendChild(iframe);

    form = document.createElement('FORM');
    form.action = 'greyhound:NewItem';
    form.method = 'POST';
    form.target = 'GHActionFrame';
    document.body.appendChild(form);

    addEntry('DisallowDone', 'True');
    addEntry('Kind', 'EMail');
    addEntry('DataMode', 'Html');
    addEntry('Subject', data.subject + '');
    addEntry('Recipients[0]', data.email + '');
    addEntry('RecipientsClear', 'True');

    if(data.attachments && data.attachments.length > 0) {
      addEntry('Attachments[0]', data.attachments[0]);
    }

    addEntry('Data1', 'Achtung: das beigefügte Debug-Log kann personenbezogene Daten enthalten. Es darf daher nur über hinreichend gesicherte E-Mail Konten versendet werden (SSL/TLS Verschlüsselung)!');

    if(data.body && data.body.length > 0) {
      for(i = 0; i < data.body.length; i++) {
        addEntry('Data' + (i + 2), data.body[i]);
      }
    }

    try {
      form.submit();
    }
    finally {
      document.body.removeChild(form);
      document.body.removeChild(iframe);
    }
  }

  // Rendert eine Option-Zeile
  function renderoption($container, option, _settings) {
    var a,i,combotree, $e, $d;

    //  Konvertiert eine geschachtelte Struktur in ein sequentielles Array.
    function _convertRecursiveArray(p, a) {
      if(a === undefined)
        return [];

      var i, r = [];
      for(i = 0; i < a.length; i++)
      {
        r.push({
          ID: a[i].id,
          Name: a[i].caption, 
          icon: a[i].data.icon,
          parent: p,
          data: a[i].data
        });

        if(a[i].children.length)
          r = r.concat(_convertRecursiveArray(a[i].id||888, a[i].children));
      }
      return r;
    };

    if(option !== undefined) {
      switch(option.type) {
        case 'combotree':
          combotree = new ghDropdown($container, this);
          combotree.setRenderer(ghTreeview);

          if (option.property) {
            _self.components[option.property] = combotree;
          }

          // Textbausteine so sortieren, dass "nicht verwenden" und "Alle Textbausteine" oben stehen:
          switch(option.sortOrder) {
            case 'asc':
              combotree.sortOrder = ghSortOrder.ascending;
              break;

            case 'desc':
              combotree.sortOrder = ghSortOrder.descending;
              break;

            case 'ascAndId':
              combotree.sortOrder = ghSortOrder.ascendingAndId;
              break;

            case 'descAndId':
              combotree.sortOrder = ghSortOrder.descendingAndId;
              break;

            case 'ascId':
              combotree.sortOrder = ghSortOrder.ascendingId;
              break;

            case 'descId':
              combotree.sortOrder = ghSortOrder.descendingId;
              break;

            default:
              if(option.property == 'TextBlockRoot')
                combotree.sortOrder = ghSortOrder.ascendingAndId;
              break;
          }

          //  Falls option.options ein object ist, wird es wohl ein treeview sein!
          if(!(option.options instanceof Array)) {
            option.options = _convertRecursiveArray(0, option.options.children);
            combotree.setRenderer(ghTreeview);
          }

          for(i = 0; i < option.options.length; i++) {
            combotree.addEntry(
              option.options[i].ID,
              {
                id: option.options[i].ID,
                label: option.options[i].Name,
                icon: 'ghGuiDefaultIcons ' + option.options[i].icon,
                parent: option.options[i].parent || option.options[i].ParentRef || 0,
                data: option.options[i].data
              },
              option.options[i].ID == _settings[option.property]
            );
          }
          (function (property) {
            combotree.onChange = function (idx, value, data) {
              _settings[property] = value;
            };
          })(option.property);
          break;

        case 'checkbox':
          var checkboxId = _guiId++;

          $d = document.createElement('LABEL');
          $d.className = "checkbox";
          $d.setAttribute('for', 'gh' + checkboxId);

          if(option.label)
            $d.innerHTML = option.label;

          (function (option) {
            $e = new ghCheckbox($container, null, {
              id: 'gh' + checkboxId,
              checked: _settings[option.property] !== undefined ? ~~_settings[option.property] != 0 : (option['default'] ? true : false),
              onChange: function(checked) {
                _settings[option.property] = checked ? '1' : '0';
              }
            });

            if (option.property) {
              _self.components[option.property] = $e;
            }
          })(option);

          $container.appendChild($d);
          break;

        case undefined:
        case 'password':
        case 'number':
        case 'text':
          var v = _settings[option.property];

          if(!v) {
            if(option.type == 'number')
              v = option['default'] || 0;
            else
              v = '';
          }

          $e = new ghInput($container, {
            type: (option.type == 'password') ? 'password' : 'text',
            value: v,
            placeholder: option.placeholder ? option.placeholder : ''
          });
          (function (property) {
            $e.onValueChange = function (value) {
              _settings[property] = value;
              debug("setting property", property, _settings[property]);

              if (property === 'Caption') {
                if (_connectionListItems['connection' + _accountOffset]) {
                  _connectionListItems['connection' + _accountOffset].innerHTML = value;
                }
              }
            };
          })(option.property);

          if (option.property) {
            _self.components[option.property] = $e;
          }
          break;

        case 'textarea':
          $d = document.createElement('DIV');
          $d.className = "textarea";
          $e = document.createElement('TEXTAREA');

          if (option.property) {
            _self.components[option.property] = $e;
          }

          (function(node, container) {
            addEvent(node, 'focus', function() {
              addCssClass(container, 'ghFocus');
            });
            addEvent(node, 'blur', function() {
              removeCssClass(container, 'ghFocus');
            });
          })($e, $d);

          if(_settings[option.property] === undefined || _settings[option.property] === null)
            $e.value = '';
          else
            $e.value = _settings[option.property];  //  REMX: Scheint Recipient0 default zu sein. ggf auch den default vor dem accountparser positionieren?

          if(option.height) {
            $e.style.height = option.height;
            $d.style.height = option.height;
          }

          $d.appendChild($e);
          $container.appendChild($d);
          (function (property) {
            $e.onchange = function () {
              if(this.value === undefined)
                _settings[property] = '';
              else
                _settings[property] = this.value;
              debug("setting property", property, _settings[property]);
            };
          })(option.property);
          break;

        case 'button':
          $e = new ghButton($container, {
            caption: option.caption
          });

          if (option.property) {
            _self.components[option.property] = $e;
          }

          (function (property, value) {
            $e.onClick = function () {
              callAction($e, property, value);
            };
          })(option.property, option.value);
          break;

        case 'separator':
          break;

        case 'caption':
          $container.innerHTML = option.value;
          break;
        
        case 'html':
          var v;

          if (option.value === undefined && option.property) {
            v = _settings[option.property];
          }

          if (v === undefined) {
            v = '';
          }

          $e = document.createElement('DIV');
          $e.className = 'ghHtmlOption';
          $e.innerHTML = v;
          $container.appendChild($e);

          if (option.property) {
            _self.components[option.property] = new (function(property, domNode) {
              this.domNode = domNode;
              this.getValue = function() {
                return _settings[property];
              };
              this.setValue = function(value) {
                _settings[property] = (value === undefined ? null : value);
                domNode.innerHTML = (value === undefined ? '' : value);
              };
            })(option.property, $e);
          }
          break;

        case 'hidden':
          if (option.property) {
            _self.components[option.property] = new (function(property) {
              this.getValue = function() {
                return _settings[property];
              };
              this.setValue = function(value) {
                _settings[property] = value;
              };
            })(option.property);
          }
          break;

        default:
          $container.innerHTML = option.type + "n/a";
      }
    }
  };

  function _connectionRenderer() {
    var $table, $tr, $td, $leftSide, $rightSide, $connectionList, $connectionBtns, $list, $title, $button;
    var i, option, options = _accountOptions.slice();

    if(options !== undefined) {
      var elem = _panes['connect'].Element;
      elem.innerHTML = '';

      $leftSide = document.createElement('DIV');
      $leftSide.className = "leftSide";
      elem.appendChild($leftSide);
      $rightSide = document.createElement('DIV');
      $rightSide.className = "rightSide";
      elem.appendChild($rightSide);

      $table = document.createElement('TABLE');
      $rightSide.appendChild($table);

      //  Verbindungsliste linke Seite
      $connectionList  = document.createElement('DIV');
      $connectionList.className = 'connectionList';
      $leftSide.appendChild($connectionList);

      $connectionBtns  = document.createElement('DIV');
      $connectionBtns.className = 'connectionButtons';
      $leftSide.appendChild($connectionBtns);

      $button = document.createElement('DIV');
      $button.className = 'button left';
      $button.innerHTML = '<span class="button ghGuiDefaultIcons add"></span>';
      $button.onclick   = function(e) {
        _newAccount();
      };
      $connectionBtns.appendChild($button);


      $button = document.createElement('DIV');
      $button.className = 'button left';
      $button.innerHTML = '<span class="button ghGuiDefaultIcons delete"></span>';
      $button.onclick   = function(e) {
        _accounts.splice(_accountOffset, 1);
        _accountOffset = Math.min(_accountOffset, _accounts.length -1);
        if(_accounts.length == 0)
          _newAccount();
        _connectionRenderer();
      };
      $connectionBtns.appendChild($button);

      $button = document.createElement('DIV');
      $button.className = 'button middle';
      $button.innerHTML = '<span class="button ghGuiDefaultIcons copy"></span>';
      $button.onclick   = function(e) {
        _newAccount(_accounts[_accountOffset]);
        _accounts[_accountOffset].Caption += ' (Kopie)';
        _connectionRenderer();
      };
      $connectionBtns.appendChild($button);

      $button = document.createElement('DIV');
      $button.className = 'button right' + ((_accountOffset == _accounts.length - 1) ? ' disabled' : '');
      $button.innerHTML = '<span class="button ghGuiDefaultIcons arrow_down_blue"></span>';
      $button.onclick   = function(e) {

        if (_accountOffset >= _accounts.length -1)
          return;

        _accounts.splice(_accountOffset++, 0, _accounts.splice(_accountOffset, 1)[0]);
        _connectionRenderer();
      };
      $connectionBtns.appendChild($button);

      $button = document.createElement('DIV');
      $button.className = 'button right' + ((_accountOffset == 0) ? ' disabled' : '');
      $button.innerHTML = '<span class="button ghGuiDefaultIcons arrow_up_blue"></span>';
      $button.onclick   = function(e) {

        if (_accountOffset <= 0)
          return;

        _accounts.splice(_accountOffset--, 0, _accounts.splice(_accountOffset, 1)[0]);
        _connectionRenderer();
      };
      $connectionBtns.appendChild($button);

      $title  = document.createElement('DIV');
      $title.innerHTML = 'Verbindungen';
      $title.className = 'headline title';
      $connectionList.appendChild($title);

      $list  = document.createElement('UL');    //  Liste statt <select> wegen IE
      $connectionList.appendChild($list);

      //  Vorhandene Verbindungen listen:
      _connectionListItems = {};

      for(i = 0; i < _accounts.length; i++)
      {
        (function(i, account) {
          var $li = document.createElement('LI');
          _connectionListItems['connection' + i] = $li;
          $li.innerHTML = account.Caption;
          $li.className = ((i%2) ? 'even' : 'odd') + ((i==_accountOffset) ? ' selected' : '');
          $li.onclick   = function(e) {
            _accountOffset = i;
            _connectionRenderer();
          };
          $list.appendChild($li);
        })(i, _accounts[i]);
      }


      //  Verbindungseinstellungen rechte Seite
      $tr = document.createElement('TR');
      $td = document.createElement('TH');
      $td.setAttribute("colspan", "2");
      $td.innerHTML = '<b>Verbindungs-Einstellungen</b>';
      $tr.appendChild($td);
      $table.appendChild($tr);

      while(options.length) {
        option = options.shift();

        if (option.type == 'hidden') {
          if (option.property) {
            renderoption(null, option, _accounts[_accountOffset]);
          }
          continue;
        }

        $tr = document.createElement('TR');
        $td = document.createElement('TD');

        switch (option.type) {
          case 'button':
            $td.innerHTML = '';
            break;

          case 'caption':
            $td.colSpan = 2;
            $td.innerHTML = '<span class="ghLabel">' + option.caption + '</span>';
            break;

          default:
            $td.innerHTML = '<span class="ghLabel">' + option.caption + '</span>';
            break;
        }

        $tr.appendChild($td);

        if (option.type != 'caption') {
          $td = document.createElement('TD');

          if (option.type) {
            $td.className += 'ghOption_' + option.type;
          }

          renderoption($td, option, _accounts[_accountOffset]);
          $td.style.width = '100%';
          $tr.appendChild($td);
        }

        if (option.help) {
          $td = document.createElement('TD');
          $td.innerHTML = '<div class="ghGuiDefaultIcons help"></div>';
          $tr.appendChild($td);
          new ghTooltip($td, option.help, {item: $td.firstChild});
        }
        $table.appendChild($tr);
      }
    }
  };

  function _globalRenderer() {
    var $table, $tr, $td;
    var option, options = _globalOptions.slice();

    if(options !== undefined) {
      var elem = _panes['global'].Element;
      elem.innerHTML = '';

      $table = document.createElement('TABLE');
      elem.appendChild($table);

      while(options.length) {
        option = options.shift();

        $tr = document.createElement('TR');
        $td = document.createElement('TD');
        $td.innerHTML = (option.type != 'button') ? '<span class="ghLabel">' + option.caption + '</span>' : '';
        $tr.appendChild($td);

        $td = document.createElement('TD');
        renderoption($td, option, _settings);
        $td.style.width = '100%';
        $tr.appendChild($td);

        if(option.help) {
          $td = document.createElement('TD');
          $td.innerHTML = '<div class="ghGuiDefaultIcons help"></div>';
          $tr.appendChild($td);
          new ghTooltip($td, option.help);
        }

        $table.appendChild($tr);
      }
    }
  };
  
  function _datamigrationRenderer() {
    var $table, $tr, $td, $div, $e;
    var i, o, Var, Detail, line, option, optionNewVar, optionNewField, btnDelete, btnDrag, elem;

    elem = _panes['migration'].Element;
    elem.innerHTML = '<p>Hier können Auftragsdaten benutzerdefinierten Feldern zugeordnet werden. Das Addon befüllt diese benutzerdefinierten Felder dann mit den abgerufenen Daten.</p>';

    $table = document.createElement('TABLE');
    elem.appendChild($table);
    
    $tr = document.createElement('TR');
    $table.appendChild($tr);

    $td = document.createElement('TD');
    $td.innerHTML = 'Daten auch für folgende Vorgangsnummern-Präfixe übernehmen:';
    $tr.appendChild($td);

    $td = document.createElement('TD');
    $e = createInput($td, {
      type: 'text',
      value: _settings['AssignUserFieldsTaskNumberPrefixes'] || ''
    });
    (function (property) {
      $e.onchange = function () {
        if (''.toUpperCase && this.value) {
          this.value = this.value.toUpperCase().replace(/[^A-Z,*]/, '');
        }

        _settings[property] = this.value;
        debug("setting property", property, _settings[property]);
      };
    })('AssignUserFieldsTaskNumberPrefixes');
    $tr.appendChild($td);

    $tr = document.createElement('TR');
    $table.appendChild($tr);
    $td = document.createElement('TD');
    $td.style.height = '1px';
    $tr.appendChild($td);

    $tr = document.createElement('TR');
    $td = document.createElement('TH');
    $td.innerHTML = 'Addon-Variable';
    $tr.appendChild($td);

    $td = document.createElement('TH');
    $td.innerHTML = 'Benutzerdef. Feld';
    $tr.appendChild($td);

    $table.appendChild($tr);

    //  Vorhandene Felder aufbauen
    for(i = 0; i < _userFieldLines.length; i++)
    {
      line = _userFieldLines[i];

      $tr = document.createElement('TR');
      $tr.className = "moveable";
      $td = document.createElement('TD');
      $td.setAttribute('width', '50%');
      option = new ghDropdown($td, this);
      for(Var in _templateVariableDescriptions)
      {
        option.addEntry(Var, {label: _templateVariableDescriptions[Var], icon: 'ghGuiDefaultIcons document'}, Var == line.Var);
      }
      (function(line) {
        option.onChange = function(idx, val, data) {
          line.Var = val.toString();
        };
      })(line);
      $tr.appendChild($td);

      $td = document.createElement('TD');
      $td.setAttribute('width', '50%');
      option = new ghDropdown($td, this);
      for(o = 0; o < _userDefinedFields.length; o++)
      {
        Detail = _userDefinedFields[o];
        option.addEntry(Detail.ID, {label: Detail.Description, icon: 'ghGuiDefaultIcons document'}, Detail.ID == line.Field);
      }
      (function(line) {
        option.onChange = function(idx, val, data) {
          line.Field = val.toString();
        };
      })(line);
      $tr.appendChild($td);

      $td = document.createElement('TD');

      $div = document.createElement('DIV');
      $div.className = 'previewLineButtons';
      $td.appendChild($div);

      //  Löschbutton
      btnDelete = document.createElement('DIV');
      btnDelete.className = 'button';
      btnDelete.title = 'Entfernen';
      btnDelete.innerHTML = '<span class="ghGuiDefaultIcons delete"></span>';
      $div.appendChild(btnDelete);

      //  Dragbutton
      btnDrag = document.createElement('DIV');
      btnDrag.className = 'dragbtn';
      $div.appendChild(btnDrag);

      (function(rowIndex, $tr) {
       
        btnDelete.onclick = function(e) 
        {
          _userFieldLines.splice(rowIndex, 1);
          _datamigrationRenderer();
        };

        btnDrag.onmousedown = function(e) {
          e=e||event;

          //  Platzhalter erzeugen zur Visualisierung:
          var dropRow = document.createElement('tr');
          var dropLine= document.createElement('td');
          dropRow.className = "moveable";
          dropRow.appendChild(dropLine);
          dropLine.className="dropArea";
          dropLine.setAttribute('colspan','3');

          $table.insertBefore(dropRow, $tr);
          $table.removeChild($tr);

          //  Kopie der Dragbaren Zeile anfertigen
          var dragRow = document.createElement('table');
          dragRow.className = "dragRow";
          dragRow.appendChild($tr);
          elem.appendChild(dragRow);
          dragRow.style.width = $table.offsetWidth + 'px';

          $table.className = "dragmove";

          //  Body eine drag-Klasse geben um Textselektion zu verhindern
          document.body.className = (document.body.className || '').replace(/\s?dragmove\s?/,'') + ' dragmove';

          //  Funktion zum Setzen der Visualisierung
          function _mousemove(e) {
            e = e||event;
            //  Childs entfernen
            var n = $table.childNodes;
            var top = $table.offsetTop;
            var row;
            for(var i = 0; i < n.length; i++)
            {
              if(e.clientY >= n[i].offsetTop + top && n[i].className == 'moveable') 
              {
                row = n[i]; 
                $table.insertBefore(dropRow, row);
              }
            }
            dragRow.style.top = dropRow.offsetTop + top + 'px';
          }

          function _mouseup(e) {
            var n = $table.childNodes;
            var newRowPos=0;
            for(var i = 0; i < n.length; i++)
            {
              if(n[i] == dropRow) 
                break;
             
              if(n[i].className == 'moveable') 
                newRowPos++;
            }

            _userFieldLines.splice(newRowPos, 0, _userFieldLines.splice(rowIndex, 1)[0]);

            removeEvent(document, "mousemove", _mousemove);
            removeEvent(document, "mouseup", _mouseup);

            document.body.className = (document.body.className || '').replace(/\s?dragmove\s?/,'');
            _datamigrationRenderer();
          };

          addEvent(document, "mousemove", _mousemove);
          addEvent(document, "mouseup", _mouseup);

          _mousemove(e);
          return false;
        }
      })(i, $tr);

      $tr.appendChild($td);

      $table.appendChild($tr);
    }

    if(_userFieldLines.length) {
      $tr = document.createElement('TR');
      $tr.className='moveable';
      $td = document.createElement('TD');
      $td.setAttribute('colspan',3);
      $td.innerHTML = '&nbsp;';
      $tr.appendChild($td);
      $table.appendChild($tr);
    }

    //  Hinzufügen
    $tr = document.createElement('TR');
    $td = document.createElement('TD');
    $td.setAttribute('width', '50%');
    optionNewVar = new ghDropdown($td, this);
    optionNewVar.addEntry(0, {label: ''});
    for(Var in _templateVariableDescriptions)
    {
      optionNewVar.addEntry(Var, {label: _templateVariableDescriptions[Var], icon: 'ghGuiDefaultIcons document'});
    }
    $tr.appendChild($td);

    $td = document.createElement('TD');
    $td.setAttribute('width', '50%');
    optionNewField = new ghDropdown($td, this);
    optionNewField.addEntry(0, {label: ''});
    for(o = 0; o < _userDefinedFields.length; o++)
    {
      Detail = _userDefinedFields[o];
      optionNewField.addEntry(Detail.ID, {label: Detail.Description, icon: 'ghGuiDefaultIcons document'});
    }
    $tr.appendChild($td);

    $td = document.createElement('TD');
    $div=document.createElement('DIV');
    $div.className = 'previewLineButtons';
    $td.appendChild($div);
    $div.innerHTML = '<div class="button" title="Hinzufügen"><span class="ghGuiDefaultIcons add"></span></div>';
    $tr.appendChild($td);
    $td.onclick = function(e) {
      var Var = optionNewVar.getSelected();
      var Field = optionNewField.getSelected();
      
      if(!Var || !Field) return;

      _userFieldLines.push( {id: _userFieldLines.length.toString(), Var: Var, Field: Field.toString()} );
      _datamigrationRenderer();
    };

    $table.appendChild($tr);
  };

  function _previewlineRenderer() {
    var $table, $tr, $td, $div, $input;
    var i, o, Var, Detail, line, option, optionNewVar, btnDelete, btnDrag;

    var elem = _panes['preview'].Element;

    elem.innerHTML = '';

    $table = document.createElement('TABLE');
    elem.appendChild($table);

    $tr = document.createElement('TR');
    $table.appendChild($tr);
    $td = document.createElement('TD');
    $tr.appendChild($td);
    $div = document.createElement('DIV');
    $div.innerHTML = '"Neue E-Mail"-Button:';
    $div.style.paddingTop = '3px';
    $td.appendChild($div);
    $td = document.createElement('TD');
    $tr.appendChild($td);
    option = new ghDropdown($td, this);
    option.addEntry('iconAndText', {label: 'Icon und Text'}, _settings.PreviewLineNew == 'iconAndText');
    option.addEntry('icon', {label: 'Nur Icon'}, _settings.PreviewLineNew == 'icon');
    option.addEntry('text', {label: 'Nur Text'}, _settings.PreviewLineNew == 'text');
    option.addEntry('none', {label: 'Kein "Neue E-Mail"-Button'}, !_settings.PreviewLineNew || _settings.PreviewLineNew == 'none');
    option.onChange = function(idx, value, data) {
      _settings.PreviewLineNew = value;
    };

    $tr = document.createElement('TR');
    $table.appendChild($tr);
    $td = document.createElement('TD');
    $tr.appendChild($td);
    $div = document.createElement('DIV');
    $div.innerHTML = 'Antworten-Button:';
    $div.style.paddingTop = '3px';
    $td.appendChild($div);
    $td = document.createElement('TD');
    $tr.appendChild($td);
    option = new ghDropdown($td, this);
    option.addEntry('iconAndText', {label: 'Icon und Text'}, !_settings.PreviewLineAnswer || _settings.PreviewLineAnswer == 'iconAndText');
    option.addEntry('icon', {label: 'Nur Icon'}, _settings.PreviewLineAnswer == 'icon');
    option.addEntry('text', {label: 'Nur Text'}, _settings.PreviewLineAnswer == 'text');
    option.addEntry('none', {label: 'Kein Antworten-Button'}, _settings.PreviewLineAnswer == 'none');
    option.onChange = function(idx, value, data) {
      _settings.PreviewLineAnswer = value;
    };

    $tr = document.createElement('TR');
    $table.appendChild($tr);
    $td = document.createElement('TD');
    $tr.appendChild($td);
    $div = document.createElement('DIV');
    $div.innerHTML = 'Weiterleiten-Button:';
    $div.style.paddingTop = '3px';
    $td.appendChild($div);
    $td = document.createElement('TD');
    $tr.appendChild($td);
    option = new ghDropdown($td, this);
    option.addEntry('iconAndText', {label: 'Icon und Text'}, _settings.PreviewLineForward == 'iconAndText');
    option.addEntry('icon', {label: 'Nur Icon'}, _settings.PreviewLineForward == 'icon');
    option.addEntry('text', {label: 'Nur Text'}, _settings.PreviewLineForward == 'text');
    option.addEntry('none', {label: 'Kein Weiterleiten-Button'}, !_settings.PreviewLineForward || _settings.PreviewLineForward == 'none');
    option.onChange = function(idx, value, data) {
      _settings.PreviewLineForward = value;
    };

    $tr = document.createElement('TR');
    $table.appendChild($tr);
    $td = document.createElement('TD');
    $td.colSpan = '2';
    $tr.appendChild($td);
    $div = document.createElement('P');
    $div.innerHTML = 'Hier können die Daten festgelegt werden, die in der Vorschauzeile angezeigt werden:';
    $td.appendChild($div);
    
    $table = document.createElement('TABLE');
    elem.appendChild($table);
    
    $tr = document.createElement('TR');
    $td = document.createElement('TH');
    $td.innerHTML = 'Beschriftung';
    $tr.appendChild($td);

    $td = document.createElement('TH');
    $td.innerHTML = 'Addon-Variable';
    $tr.appendChild($td);

    $table.appendChild($tr);

    //  Vorhandene Felder aufbauen
    for(i = 0; i < _previewLines.length; i++)
    {
      line = _previewLines[i];

      $tr = document.createElement('TR');
      $tr.className = "moveable";

      $td = document.createElement('TD');
      $td.setAttribute('width', '50%');
      $input = createInput($td, {value: line.Caption || ''});

      (function(line) {
        $input.onchange = function() {
          line.Caption = this.value;
        };
      })(line);

      $tr.appendChild($td);

      $td = document.createElement('TD');
      $td.setAttribute('width', '50%');
      option = new ghDropdown($td, this);
      for(Var in _templateVariableDescriptions)
      {
        option.addEntry(Var, {label: _templateVariableDescriptions[Var], icon: 'ghGuiDefaultIcons document'}, Var == line.Var);
      }
      (function(line) {
        option.onChange = function(idx, val, data) {
          line.Var = val.toString();
        };
      })(line);
      $tr.appendChild($td);

      $td = document.createElement('TD');
      $div = document.createElement('DIV');
      $div.className = 'previewLineButtons';
      $td.appendChild($div);

      //  Löschbutton
      btnDelete = document.createElement('DIV');
      btnDelete.className = 'button adjustTop';
      btnDelete.title = 'Entfernen';
      btnDelete.innerHTML = '<span class="ghGuiDefaultIcons delete"></span>';
      $div.appendChild(btnDelete);

      //  Dragbutton
      btnDrag = document.createElement('DIV');
      btnDrag.className = 'dragbtn';
      $div.appendChild(btnDrag);

      (function(rowIndex, $tr) {
       
        btnDelete.onclick = function(e) 
        {
          _previewLines.splice(rowIndex, 1);
          _previewlineRenderer();
        };

        btnDrag.onmousedown = function(e) {
          e=e||event;

          //  Platzhalter erzeugen zur Visualisierung:
          var dropRow = document.createElement('tr');
          var dropLine= document.createElement('td');
          dropRow.className = "moveable";
          dropRow.appendChild(dropLine);
          dropLine.className="dropArea";
          dropLine.setAttribute('colspan','3');

          $table.insertBefore(dropRow, $tr);
          $table.removeChild($tr);

          //  Kopie der Dragbaren Zeile anfertigen
          var dragRow = document.createElement('table');
          dragRow.className = "dragRow";
          dragRow.appendChild($tr);
          elem.appendChild(dragRow);
          dragRow.style.width = $table.offsetWidth + 'px';

          $table.className = "dragmove";

          //  Body eine drag-Klasse geben um Textselektion zu verhindern
          document.body.className = (document.body.className || '').replace(/\s?dragmove\s?/,'') + ' dragmove';

          //  Funktion zum Setzen der Visualisierung
          function _mousemove(e) {
            e = e||event;
            //  Childs entfernen
            var n = $table.childNodes;
            var top = $table.offsetTop;
            var row;
            for(var i = 0; i < n.length; i++)
            {
              if(e.clientY >= n[i].offsetTop + top && n[i].className == 'moveable') 
              {
                row = n[i]; 
                $table.insertBefore(dropRow, row);
              }
            }
            dragRow.style.top = dropRow.offsetTop + top + 'px';
          }

          function _mouseup(e) {
            var n = $table.childNodes;
            var newRowPos=0;
            for(var i = 0; i < n.length; i++)
            {
              if(n[i] == dropRow) 
                break;
             
              if(n[i].className == 'moveable') 
                newRowPos++;
            }

            _previewLines.splice(newRowPos, 0, _previewLines.splice(rowIndex, 1)[0]);

            removeEvent(document, "mousemove", _mousemove);
            removeEvent(document, "mouseup", _mouseup);

            document.body.className = (document.body.className || '').replace(/\s?dragmove\s?/,'');
            _previewlineRenderer();
          };

          addEvent(document, "mousemove", _mousemove);
          addEvent(document, "mouseup", _mouseup);

          _mousemove(e);
          return false;
        }
      })(i, $tr);

      $tr.appendChild($td);

      $table.appendChild($tr);
    }

    if(_previewLines.length) {
      $tr = document.createElement('TR');
      $tr.className='moveable';
      $td = document.createElement('TD');
      $td.setAttribute('colspan',3);
      $td.innerHTML = '&nbsp;';
      $tr.appendChild($td);
      $table.appendChild($tr);
    }

    //  Hinzufügen
    $tr = document.createElement('TR');
    $td = document.createElement('TD');
    $td.setAttribute('width', '50%');
    $input = createInput($td);
    $tr.appendChild($td);

    $td = document.createElement('TD');
    $td.setAttribute('width', '50%');
    optionNewVar = new ghDropdown($td, this);
    optionNewVar.addEntry(0, {label: ''});
    for(Var in _templateVariableDescriptions)
    {
      optionNewVar.addEntry(Var, {label: _templateVariableDescriptions[Var], icon: 'ghGuiDefaultIcons document'});
    }
    $tr.appendChild($td);

    $td = document.createElement('TD');
    $div= document.createElement('DIV');
    $div.className = 'previewLineButtons';
    $td.appendChild($div);
    $div.innerHTML = '<div class="button" title="Hinzufügen"><span class="ghGuiDefaultIcons add"></span></div>';
    $tr.appendChild($td);
    $div.firstChild.onclick = function(e) {
      var Caption = $input.value;
      var Var = optionNewVar.getSelected();
      
      if(!Var) return;

      _previewLines.push( {id: _previewLines.length.toString(), Var: Var.toString(), Caption: Caption || ''} );
      _previewlineRenderer();
    };

    $table.appendChild($tr);
  };

  function _trackingRenderer() {
    var $table, $tr, $td, $div, $input, $inputPrefix, $inputProvider, $inputUrl;
    var i, line, btnDelete, btnDrag;

    var elem = _panes['tracking'].Element;
    elem.innerHTML = '<p>Ordnet das Addon einer Paketnummer den falschen (oder gar keinen) Link zu, so können hier die Links zur Sendungsverfolgung für einzelne Nummernkreise hinterlegt werden. Für Paketnummern, die mit dem angegebenen Nummernkreis beginnen, wird dann der angegebene Versanddienstleister angezeigt und die Tracking-URL als Link verwendet. In der Tracking-URL wird die Variable "[$Nr]" durch die Paketnummer und die Variable "[$Zip]" durch die Postleitzahl des jeweiligen Auftrags ersetzt.</p>';

    $table = document.createElement('TABLE');
    elem.appendChild($table);
    
    $tr = document.createElement('TR');

    $td = document.createElement('TH');
    $td.innerHTML = 'Nummernkreis';
    $td.setAttribute('width', '33%');
    $tr.appendChild($td);

    $td = document.createElement('TH');
    $td.innerHTML = 'Versanddienstleister';
    $td.setAttribute('width', '33%');
    $tr.appendChild($td);

    $td = document.createElement('TH');
    $td.innerHTML = 'Tracking-URL';
    $td.setAttribute('width', '34%');
    $tr.appendChild($td);

    $td = document.createElement('TH');
    $td.innerHTML = '';
    $tr.appendChild($td);

    $table.appendChild($tr);

    //  Aufbau vorhandener TrackingLinks
    //  Vorhandene Felder aufbauen
    for(i = 0; i < _trackingLinks.length; i++)
    {
      line = _trackingLinks[i];

      $tr = document.createElement('TR');
      $tr.className='moveable';

      $td = document.createElement('TD');
      $td.setAttribute('width', '30%');
      $input = createInput($td, {value: line.Prefix});

      (function(line) {
        $input.onchange = function() {
          line.Prefix = this.value;
        };
      })(line);

      $tr.appendChild($td);

      $td = document.createElement('TD');
      $td.setAttribute('width', '30%');
      $input = createInput($td, {value: line.Provider});

      (function(line) {
        $input.onchange = function() {
          line.Provider = this.value;
        };
      })(line);

      $tr.appendChild($td);

      $td = document.createElement('TD');
      $td.setAttribute('width', '30%');
      $input = createInput($td, {value: line.Url});

      (function(line) {
        $input.onchange = function() {
          line.Url = this.value;
        };
      })(line);

      $tr.appendChild($td);

      $td = document.createElement('TD');
      $td.setAttribute('width', '10%');
      $div = document.createElement('DIV');
      $div.className = 'previewLineButtons';
      $td.appendChild($div);

      //  Löschbutton
      btnDelete = document.createElement('DIV');
      btnDelete.className = 'button';
      btnDelete.title = 'Entfernen';
      btnDelete.innerHTML = '<span class="ghGuiDefaultIcons delete"></span>';
      $div.appendChild(btnDelete);

      //  Dragbutton
      btnDrag = document.createElement('DIV');
      btnDrag.className = 'dragbtn';
      $div.appendChild(btnDrag);

      (function(rowIndex, $tr) {
       
        btnDelete.onclick = function(e) 
        {
          _trackingLinks.splice(rowIndex, 1);
          _trackingRenderer();
        };

        btnDrag.onmousedown = function(e) {
          e=e||event;

          //  Platzhalter erzeugen zur Visualisierung:
          var dropRow = document.createElement('tr');
          var dropLine= document.createElement('td');
          dropRow.className = "moveable";
          dropRow.appendChild(dropLine);
          dropLine.className="dropArea";
          dropLine.setAttribute('colspan','4');

          $table.insertBefore(dropRow, $tr);
          $table.removeChild($tr);

          //  Kopie der Dragbaren Zeile anfertigen
          var dragRow = document.createElement('table');
          dragRow.className = "dragRow";
          dragRow.appendChild($tr);
          elem.appendChild(dragRow);
          dragRow.style.width = $table.offsetWidth + 'px';

          $table.className = "dragmove";

          //  Body eine drag-Klasse geben um Textselektion zu verhindern
          document.body.className = (document.body.className || '').replace(/\s?dragmove\s?/,'') + ' dragmove';

          //  Funktion zum Setzen der Visualisierung
          function _mousemove(e) {
            e = e||event;
            //  Childs entfernen
            var n = $table.childNodes;
            var top = $table.offsetTop;
            var row;
            for(var i = 0; i < n.length; i++)
            {
              if(e.clientY >= n[i].offsetTop + top && n[i].className == 'moveable') 
              {
                row = n[i]; 
                $table.insertBefore(dropRow, row);
              }
            }
            dragRow.style.top = dropRow.offsetTop + top + 'px';
          }

          function _mouseup(e) {
            var n = $table.childNodes;
            var newRowPos=0;
            for(var i = 0; i < n.length; i++)
            {
              if(n[i] == dropRow) 
                break;
             
              if(n[i].className == 'moveable') 
                newRowPos++;
            }

            _trackingLinks.splice(newRowPos, 0, _trackingLinks.splice(rowIndex, 1)[0]);

            removeEvent(document, "mousemove", _mousemove);
            removeEvent(document, "mouseup", _mouseup);

            document.body.className = (document.body.className || '').replace(/\s?dragmove\s?/,'');
            _trackingRenderer();
          };

          addEvent(document, "mousemove", _mousemove);
          addEvent(document, "mouseup", _mouseup);

          _mousemove(e);
          return false;
        }
      })(i, $tr);

      $tr.appendChild($td);

      $table.appendChild($tr);
    }

    if(_trackingLinks.length) {
      $tr = document.createElement('TR');
      $tr.className='moveable';
      $td = document.createElement('TD');
      $td.setAttribute('colspan', 4);
      $td.innerHTML = '&nbsp;';
      $tr.appendChild($td);
      $table.appendChild($tr);
    }

    //  Hinzufügen-Zeile
    $tr = document.createElement('TR');

    $td = document.createElement('TD');
    $inputPrefix = createInput($td);
    $tr.appendChild($td);

    $td = document.createElement('TD');
    $inputProvider = createInput($td);
    $tr.appendChild($td);

    $td = document.createElement('TD');
    $inputUrl = createInput($td);
    $tr.appendChild($td);

    $td = document.createElement('TD');
    $div=document.createElement('DIV');
    $div.className = 'previewLineButtons';
    $td.appendChild($div);
    $div.innerHTML = '<div class="button" title="Hinzufügen"><span class="ghGuiDefaultIcons add"></span></div>';

    (function(i){
      $td.onclick = function(e) {
        
        if(!$inputPrefix.value || !$inputProvider.value || !$inputUrl.value)
          return;

        _trackingLinks.push( {Prefix: $inputPrefix.value, Provider: $inputProvider.value, Url: $inputUrl.value} );
        _trackingRenderer();
      }
    })(i);
    $tr.appendChild($td);

    $table.appendChild($tr);
  };

  function _extendedRenderer() {
    var $table, $tr, $td;
    var option, options = _extendedOptions.slice();

    if(options !== undefined) {
      var elem = _panes['extended'].Element;
      elem.innerHTML = '';

      $table = document.createElement('TABLE');
      elem.appendChild($table);

      while(options.length) {
        option = options.shift();

        $tr = document.createElement('TR');
        $td = document.createElement('TD');
        $td.innerHTML = (option.type != 'button') ? '<span class="ghLabel">' + option.caption + '</span>' : '';
        $tr.appendChild($td);

        $td = document.createElement('TD');
        renderoption($td, option, _settings);
        $td.style.width = '100%';
        $tr.appendChild($td);

        $td = document.createElement('TD');
        $td.innerHTML = '<div class="ghGuiDefaultIcons help"></div>';
        $tr.appendChild($td);
        new ghTooltip($td, option.help);

        $table.appendChild($tr);
      }
    }
  };

})();

var ghAddonPage = ghAddonSettingsPage;

window.onload = function() {
  window.onload = null;   //  verhindern das methode mehrfach aufgerufen wird.
  try { ghAddonSettingsPage.init() }
  catch(e) { document.body.innerHTML = e.message };
  window.scrollTo(0,0);
};

//  Addon initalisieren sobald DOM fertig geladen; Grafiken könnten hier noch nicht komplett vorhanden sein.
if (document.addEventListener) {
  document.addEventListener("DOMContentLoaded", window.onload, false);
}
