/*
 * GREYHOUND Addon Framework
 *
 * Copyright (c) 2011-2019 GREYHOUND Software GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND Addon Framework and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

// Kompatiblität mit älteren Browsern:
if(!Array.isArray) { Array.isArray = function(arg) { return Object.prototype.toString.call(arg) === '[object Array]'; }; }

// Seitenleiste:
var ghAddonSidebarPage = new (function() {
  "use strict";

  //
  //  Lokale Variablen (private)
  //
  var _self = this;

  var _templateId;
  var _hash = {};         //  Kapselt übergebene item-Parameter
  var _data;              //  Kapselt Ergebnis der letzten Suche.
  var _addon = {name: '', version: '', build: 0};
  var _remoteServiceName = '';
  var _searchTooltip = '';
  var _emptySearchMessage = '';

  var _settings = {};
  var _userSettings = {};

  var _customers = [];    //  Liste mit potenziellen Kundenaccounts
  var _customerId;
  var _selectedCustomer;
  
  var _orders = [];       //  Liste mit Bestellungen
  var _orderId;
  var _selectedOrder;
  var _splitterChanged = false;

  var isCustomSearch;     //  Wird gesetzt sobald manuell gesucht wird.

  var _$accountView, _$infoWrapper, _$infoView, _$templateView, _$actionsView, _$orderDetailView, _$orderItemsView;
  var _$searchInput, _$customerRow, _$orderRow, _$assignTasknumberRow, _$templatesRow, _$accountSelectCell, _$orderRow_Count, _$customerRow_Count, _$taskNumberButton;
  var _$dialog;

  //  Instanzen verwendeter Komponenten
  var IAccounts, ICustomers, IOrders, ITemplates, IOrderDetails, ISplitter;
  var newButton, replyButton, forwardButton;

  //
  //  Interface (public)
  //
  this.rpc = null;
  this.dataManager = null;
  this.init               = init;
  this.refresh            = refresh;      //  Neuzeichnen der GUI
  this.beforeInit = null;
  this.afterInit = null;
  this.addSearchButtons = null;
  this.addCommunicationButtons = null;
  this.commAction = GHAction;
  this.showHistory = showHistory;
  this.getAttachments = null;


  //  Blendet eine Info ein wie z.b. "loading"
  function showInfo(content) {
    if(content)
    {
      _$infoView.style.left = 0;
      _$infoView.innerHTML = content;
      document.body.className = (document.body.className||'').replace(/\s?info/,'') + " info";
      _$infoView.style.left = (document.body.offsetWidth - _$infoView.offsetWidth) / 2 + 'px';
      _$infoWrapper.style.display='block';
    }
    else
    {
      _$infoView.innerHTML = '';
      document.body.className = (document.body.className||'').replace(/\s?info/,'');
      _$infoWrapper.style.display='';
    }
  };

  function init() {
    _hash = extractHashValues();

    _self.rpc = new ghAddonRpc();
    _self.rpc.setSettings(_hash);
    _self.rpc.onAddonSettingsChanged = _onSettingsChanged;

    _self.dataManager = new ghAddonDataManager(_self.rpc);
    _self.dataManager.onAccountChanged = _onAccountRefreshed;

    if (_self.beforeInit) {
      _self.beforeInit(_self);
    }

    //  GUI Splitter
    document.body.innerHTML = '';

    _$dialog = new ghDialog({width:"80%"});

    ISplitter = new ghSplitter( document.body );
    ISplitter.onSplitterMoved = onSplitterMoved;
    _$accountView = ISplitter.addPanel({"class": "ghAddonUser", id: "accounts"});
    _$templateView = ISplitter.addPanel({"class": "ghAddonTemplates", id: "templates"});
    _$actionsView = ISplitter.addPanel({"class": "ghAddonActions", id: "actions"});
    _$orderDetailView = ISplitter.addPanel({"class": "ghAddonOrderDetails", id: "order", height: 0.75});
    _$orderItemsView = ISplitter.addPanel({"class": "ghAddonOrderItems", id: "orderItems", resizable: true, height: 0.25});

    _$infoWrapper    = document.createElement("DIV");
    _$infoWrapper.className = "ghInfoWrapper";
    document.body.appendChild(_$infoWrapper);
    _$infoView       = document.createElement("DIV");
    _$infoView.className = "ghInfoPopup";
    document.body.appendChild(_$infoView);

    // Template-/Vorlagen-Bereich bauen:
    var $table = document.createElement("TABLE");
    _$templateView.appendChild($table);

    //  Template-Dropdown
    var $tr = document.createElement("TR");
    var $td1= document.createElement("TD");
    $tr.appendChild($td1);
    $table.appendChild($tr);
    ITemplates = new ghDropdown($td1, this);
    ITemplates.setRenderer(ghTreeview);
    ITemplates.onChange = function(idx, id, data) {
      _templateId = id;
    };

    _$templatesRow = $tr;
    _$templatesRow.style.display = "none";

    //  Antworten-Buttons
    $tr = document.createElement("TR");
    $td1= document.createElement("TD");
    $tr.appendChild($td1);
    $table.appendChild($tr);
    $td1.className = "actionButtons";
    $td1.innerHTML = '<div class="buttons"></div>';

    newButton     = new ghDropdown($td1.firstChild, _self, {buttonMode: true, iconInRepresentation: true});
    newButton.onChange = onNewButton;
    replyButton   = new ghDropdown($td1.firstChild, _self, {buttonMode: true, iconInRepresentation: true});
    replyButton.onChange = onReplyButton;
    forwardButton = new ghDropdown($td1.firstChild, _self, {buttonMode: true, iconInRepresentation: true});
    forwardButton.onChange = onForwardButton;

    if(_self.addCommunicationButtons)
      _self.addCommunicationButtons($td1.firstChild);

    // Ein 1px breites div einfügen, welches nicht floatet, damit der Bereich nicht Höhe 0 hat:
    var spacer = document.createElement('DIV');
    spacer.className = 'spacer';
    $td1.firstChild.appendChild(spacer);

    // GUI aufbauen / aktualisieren:
    buildGUI();
    refresh();

    // Basisinfos abrufen:
    _self.rpc.call('getAddonInfo', [],
      function(response) {
        if (response.result) {
          _addon.name = response.result.name;
          _addon.version = response.result.version;
          _addon.build = response.result.build;
          _remoteServiceName = response.result.remoteServiceName;
          _searchTooltip = response.result.searchTooltip;
          _emptySearchMessage = response.result.emptySearchMessage;
          _$searchInput.setTitle(_searchTooltip);
          _self.rpc.settingsTimestamp = response.result.settingsTimestamp;
        }
      },
      function(err) {
        criticalError(err.error);
      },
      false
    );

    //  Abruf der AddonSettings
    _self.rpc.call("getAddonSettings", [],
      function(response){
        if (response && response.result) {
          //Key-Value Paare in Objekt umwandeln:
          var fixedSettings = {};

          for (var i = 0; i < response.result.length; i++) {
            fixedSettings[response.result[i].key] = response.result[i].value;
          }

          response.result = fixedSettings;
          _onSettingsChanged(response.result);
        }
      },
      function(err) {
        criticalError(err.error);
      },
      false
    );

    _self.rpc.call("getAddonUserSettings", [],
      function(response) {
        var height, i, panels, panel;

        _userSettings = response && response.result ? response.result : {};
        height = _userSettings.ghOrderItemsSplitter || 0.25;

        if (height < 0.1) {
          height = 0.1;
        }

        panels = ISplitter.getPanels();

        for (i = 0; i < panels.length; i++) {
          panel = panels[i];

          if (panel.options) {
            if (panel.options.id == 'order') {
              panel.height = 1 - height;
              panel.options.height = 1 - height;
              _splitterChanged = true;
            } else if (panel.options.id == 'orderItems') {
              panel.height = height;
              panel.options.height = height;
              _splitterChanged = true;
            }
          }
        }

        ISplitter.resize();
      }
    );

    //  Hashchange / Änderungen der Suchanfrage abfangen
    addEvent(window, 'hashchange', invalidateData);
    addEvent(window, 'resize', function() {
      ISplitter.resize();
    });

    if (_self.afterInit) {
      _self.afterInit(_self);
    }
  };

  function onNewButton(index, id, data) {
    GHAction(data.value); // value in [newEmail, newFax... replyEmail...forwardEmail]
  }

  function onReplyButton(index, id, data) {
    replyButton.setInactive(true);
    
    try {
      GHAction(data.value); // value in [newEmail, newFax... replyEmail...forwardEmail]
    } finally {
      replyButton.setInactive(false);
    }
  }

  function onForwardButton(index, id, data) {
    GHAction(data.value); // value in [newEmail, newFax... replyEmail...forwardEmail]
  }

  function _onSettingsChanged(settings) {
    var key, val;

    for (key in settings) {
      val = settings[key];

      if (!key.match(/^Account\.(\d+)\.(.*)$/)) {
        _settings[key] = val;
      }
    }

    _self.dataManager.setSettings(settings, _onAccountRefreshed);

    if (_self.dataManager.accounts.length == 0) {
      return criticalError( {code: 0, message: 'Es wurden keine gültigen Verbindungsdaten hinterlegt, oder die Verbindungen sind inaktiv.'} );
    }

    ITemplates.sortOrder = (_settings.SortMailTemplates) ? parseInt(_settings.SortMailTemplates) : ghSortOrder.ascending;

    invalidateData();
    refresh();
  };

  //  Seite auf kritischen Fehler setzen:
  function criticalError(err) {
    var sError = '<b>Es ist ein Fehler aufgetreten:</b> ' + err.message;

    try {
      resetData();
      showInfo(sError);
      _$infoWrapper.style.display='';
    } catch(e) {
      document.getElementById("ghPage").innerHTML = sError + '<br>' + e.message;
      debug(err);
    }
  };

  // Splitter wurde bewegt:
  function onSplitterMoved(panels) {
    var i, panel;

    for (i = 0; i < panels.length; i++) {
      panel = panels[i];

      if (panel.options && panel.options.id == 'orderItems') {
        _self.rpc.call("setAddonUserSetting", ["ghOrderItemsSplitter", panel.height], function() {
        }, function() {
        });
        break;
      }
    }
  }

  //  Aufbau Accountinformationen
  function buildGUI() {
    var $table, $tr, $td1, $td2, $inputWrapper, $inputFrame;

    //  AccountView aufbauen:
    $table = document.createElement("TABLE");

    //  Suche
    $tr = document.createElement("TR");
    $td1= document.createElement("TD");
    $td2= document.createElement("TD");
    $tr.appendChild($td1);
    $tr.appendChild($td2);
    $table.appendChild($tr);
    $td1.setAttribute("width", "32");
    $td1.innerHTML = '<div class="find"><div class="ghGuiDefaultIcons find"></div></div>';
    $td2.className = "selectNone";
    _$accountSelectCell = $td2;

    //  Inputfeld gewrapped aus Layoutgründen:
    _$searchInput = new ghInput($td2, {placeholder: 'In ' + ghRemoteServiceName + ' suchen...', title: _searchTooltip, selectOnFocus: true});

    _$searchInput.inputNode.onkeydown = function(e) {
      e = e || event;

      if (e.keyCode == 13) {
        isCustomSearch = true;
        RPCSearch("searchBySearchTerm", _$searchInput.getValue());
      }        
    };
    
    //  SelectBox für Accounts aufbauen:
    IAccounts = new ghDropdown($td2, this);
    IAccounts.onChange = function(idx, id, data) {
      //  Account geändert == Suche wiederholen!
      if (_self.dataManager.account && id == _self.dataManager.account.id) {
        return;
      }
      
      _self.dataManager.setAccountId(id);

      showInfo('<span class="ghSpin"></span> Initialisiere...');
      resetData();

      _self.dataManager.repeatSearch(
        id,
        function onSuccess(response, rpcMethod, rpcParams) {
          resetData();
          showInfo();
          setDataFromSearchResult(response, rpcParams);
        },
        function onError(response, rpcMethod, rpcParams){
          resetData();
          showInfo();
          showError(response);
        },
        function onSearch(rpcMethod, rpcParams) {
          showInfo('<span class="ghSpin"></span> Die Suche wird durchgeführt...');
        }
      );
    };

    if (_self.addSearchButtons) {
      var $td3 = document.createElement("TD");
      $tr.appendChild($td3);
      $td3.style.verticalAlign = "bottom";
      _self.addSearchButtons($td3);
    }

    //  Customerwahl:
    $tr = document.createElement("TR");
    $td1= document.createElement("TD");
    $td2= document.createElement("TD");
    $tr.appendChild($td1);
    $tr.appendChild($td2);
    $table.appendChild($tr);
    $td1.setAttribute("width", "32");
    _$customerRow_Count = $td1;
    _$customerRow_Count.innerHTML = '<div class="ghCustomersCount"><span class="ghGuiDefaultIcons businessman"></span><span class="count">' + _customers.length + '</span></div>';
    //  SelectBox für Kunden aufbauen:
    ICustomers = new ghDropdown($td2, this);
    ICustomers.onChange = function(idx, id, data) {
      _customerId = id;
      filterOrdersByCustomerId(id);

      //  Erste Bestellung aktivieren
      if (_orders.length && _orders[0].ID) {
        setSelectedOrder( _orders[0].ID );
      }

      refresh();
    };

    //  Gesamte Zeile zwischenspeichern um sie ggf auszublenden!
    _$customerRow = $tr;
    _$customerRow.style.display = "none";

    //  Auswahl Bestellung:
    $tr = document.createElement("TR");
    $td1= document.createElement("TD");
    $td2= document.createElement("TD");
    $tr.appendChild($td1);
    $tr.appendChild($td2);
    $table.appendChild($tr);
    $td1.setAttribute("width", "32");
    _$orderRow_Count = $td1;
    _$orderRow_Count.innerHTML = '<div class="ghOrdersCount"><span class="ghGuiDefaultIcons shoppingcart"></span><span class="count">' + _orders.length + '</span></div>';
    //  SelectBox für Bestellungen aufbauen:
    IOrders = new ghDropdown($td2, this);
    IOrders.onChange = function(idx, id, data) {
      setSelectedOrder(id);
      refresh();
    };
    //  Gesamte Zeile zwischenspeichern um sie ggf auszublenden!
    _$orderRow = $tr;
    _$orderRow.style.display="none";

    //  Option zum manuellen Setzen einer Vorgangsnummer:
    $tr = document.createElement("TR");
    $td1= document.createElement("TD");
    $td2= document.createElement("TD");
    $tr.appendChild($td1);
    $tr.appendChild($td2);
    $table.appendChild($tr);
    $td1.setAttribute("width", "32");
    $td1.setAttribute("align", "center");
    $td1.setAttribute("valign", "top");
    $td1.innerHTML = '<span class="ghGuiDefaultIcons barcode"></span>';
    //  Gesamte Zeile zwischenspeichern um sie ggf auszublenden!
    _$assignTasknumberRow = $tr;
    _$assignTasknumberRow.style.display="none";

    _$taskNumberButton = document.createElement("DIV");
    _$taskNumberButton.className = 'button';
    _$taskNumberButton.innerHTML = ghRemoteServiceName + ' Vorgang zuweisen';
    _$taskNumberButton.title = 'Ordnet das ausgewählte Element dem angezeigten ' + ghRemoteServiceName + ' Vorgang zu.';
    $td2.appendChild(_$taskNumberButton);

    //  Hinzufügen
    _$accountView.innerHTML = '';
    _$accountView.appendChild($table);

    IOrderDetails = new ghOrderDetails( _$orderDetailView );
    IOrderDetails.addColumn('Icon',    {'type': 'icon',   'width': '24px', 'fixed': true});
    IOrderDetails.addColumn('Caption', {'type': 'string', 'label': 'Bestellung'});
    IOrderDetails.addColumn('Value',   {'type': 'html',   'label': 'x', 'width': '100%'}); // default ist html da das php bereits die daten escaped haben sollte!
  };

  //  Erstellt action-Buttons
  function _buildCommunicationActions()
  {
    var haveItem, canReply, canForward, emailNewTooltip, emailReplyTooltip, emailForwardTooltip;

    haveItem = _hash.itemID ? true : false;
    canReply = true;
    canForward = true;

    if(!_selectedCustomer || !_selectedCustomer.Email) {
      emailNewTooltip = 'Es liegt keine E-Mail Adresse des Kunden vor.';
      emailReplyTooltip = 'Es liegt keine E-Mail Adresse des Kunden vor.';
      emailForwardTooltip = 'Per E-Mail weiterleiten.';
    }
    else {
      emailNewTooltip = 'Neue E-Mail an ' + htmlspecialchars(_selectedCustomer.Email) + ' verfassen.';
      emailReplyTooltip = 'Antwort per E-Mail an ' + htmlspecialchars(_selectedCustomer.Email) + ' verfassen.';
      emailForwardTooltip = 'Per E-Mail weiterleiten.';
    }

    if(!haveItem) {
      canReply = false;
      canForward = false;
      emailReplyTooltip = 'Es kann keine Antwort per E-Mail verfasst werden, da kein Element ausgewählt ist.';
      emailForwardTooltip = 'Es kann keine Weiterleitung per E-Mail verfasst werden, da kein Element ausgewählt ist.';
    }

    //  Headlines-Setup
    newButton.clear();
    newButton.addEntry(0, {
      label: 'Neu', 
      icon:  'ghGuiDefaultIcons mail_add',
      value: 'newEmail',
      tooltip: emailNewTooltip,
      inactive: false

    });
    replyButton.clear();
    replyButton.addEntry(0, {
      label: 'Antworten', 
      icon:  'ghGuiDefaultIcons mail_answered',
      value: 'replyEmail',
      tooltip: emailReplyTooltip,
      inactive: !canReply
    });
    forwardButton.clear();
    forwardButton.addEntry(0, {
      label: 'Weiterleiten', 
      icon:  'ghGuiDefaultIcons mail_forwarded',
      value: 'forwardEmail',
      tooltip: emailForwardTooltip,
      inactive: !canForward
    });

    // E-Mail:
    newButton.addMenuItem('E-Mail', 'mail_new', 'newEmail', emailNewTooltip, false);
    replyButton.addMenuItem('E-Mail', 'mail_answered', 'replyEmail', emailReplyTooltip, !canReply);
    forwardButton.addMenuItem('E-Mail', 'mail_forwarded', 'forwardEmail', emailForwardTooltip, !canForward);

    // Fax:
    if(!_selectedCustomer || !_selectedCustomer.Fax)
    {
      newButton.addMenuItem('Fax', 'fax_new', 'newFax', 'Es liegt keine Faxnummer des Kunden vor.');
      replyButton.addMenuItem('Fax', 'fax_answered', 'replyFax', 'Es liegt keine Faxnummer des Kunden vor.', !canReply);
    }
    else {
      newButton.addMenuItem('Fax', 'fax_new', 'newFax', 'Neues Fax an ' + htmlspecialchars(_selectedCustomer.Fax) + ' verfassen.');
      replyButton.addMenuItem('Fax', 'fax_answered', 'replyFax', 'Antwort per Fax an ' + htmlspecialchars(_selectedCustomer.Fax) + ' verfassen.', !canReply);
    }

    forwardButton.addMenuItem('Fax', 'fax_forwarded', 'forwardFax', 'Per Fax weiterleiten.', !canForward);


    // Brief:
    if(!_selectedCustomer || !Array.isArray(_selectedCustomer.Address) || !_selectedCustomer.Address.length)
    {
      newButton.addMenuItem('Brief', 'letter_new', 'newLetter', 'Es liegt keine Anschrift des Kunden vor.');
      replyButton.addMenuItem('Brief', 'letter_answered', 'replyLetter', 'Es liegt keine Anschrift des Kunden vor.', !canReply);
    }
    else {
      newButton.addMenuItem('Brief', 'letter_new', 'newLetter', 'Neuen Brief an ' + htmlspecialchars(_selectedCustomer.Address.join(", ")) + ' verfassen.');
      replyButton.addMenuItem('Brief', 'letter_answered', 'replyLetter', 'Antwort per Brief an ' + htmlspecialchars(_selectedCustomer.Address.join(", ")) + ' verfassen.', !canReply);
    }

    forwardButton.addMenuItem('Brief', 'letter_forwarded', 'forwardLetter', 'Per Brief weiterleiten.', !canForward);


    // Telefonnotiz:
    if(!_selectedCustomer || !_selectedCustomer.Phone)
    {
      newButton.addMenuItem('Telefonnotiz', 'call_new', 'newCall', 'Es liegt keine Telefonnummer des Kunden vor.');
      replyButton.addMenuItem('Telefonnotiz', 'call_answered', 'replyCall', 'Es liegt keine Telefonnummer des Kunden vor.', !canReply);
    }
    else
    {
      newButton.addMenuItem('Telefonnotiz', 'call_new', 'newCall', 'Neue Telefonnotiz für ' + htmlspecialchars(_selectedCustomer.Phone) + ' anlegen.');
      replyButton.addMenuItem('Telefonnotiz', 'call_answered', 'replyCall', 'Antwort als Telefonnotiz an ' + htmlspecialchars(_selectedCustomer.Phone) + ' anlegen.', !canReply);
    }

    forwardButton.addMenuItem('Telefonnotiz', 'call_forwarded', 'forwardCall', 'Per Telefonnotiz weiterleiten.', !canForward);

  };

  //  Aufbau GUI
  function refresh() {
    var $table, $tr, $td1, $td2, matchingTaskNumber, taskNumberPrefix, prefixes, i, j;

    //  Accounts neu aufbauen:
    IAccounts.clear();

    for(i = 0; i < _self.dataManager.accounts.length; i++) {
      var account = _self.dataManager.accounts[i];
      IAccounts.addEntry(account.id, {
        label: account.settings.Caption,
        icon: 'ghGuiDefaultIcons server'
      }, _self.dataManager.account && _self.dataManager.account.id == account.id);
    }

    _$accountSelectCell.className = (_self.dataManager.accounts.length > 1) ? 'select40' : 'selectNone';

    //  Kundenliste
    ICustomers.clear();
    if(_customers.length > 1) 
    {
      ICustomers.addEntry(null, {label: "Kunde auswählen"}, _customerId == null);

      for(i=0;i<_customers.length; i++)
        ICustomers.addEntry(_customers[i].ID, {label: _customers[i].Name, icon: 'ghGuiDefaultIcons businessman', tooltip: _customers[i].Description}, _customerId == _customers[i].ID);

      _$customerRow.style.display="";
      _$customerRow_Count.innerHTML = '<div class="ghCustomersCount"><span class="ghGuiDefaultIcons businessman"></span><span class="count">' + _customers.length + '</span></div>';
    }
    else
    {
      _$customerRow.style.display = "none";
    }

    //  Vorgangsnummer zuweisen (ich verlasse mich darauf das das AddOn in getOrder bereits die "TaskNumberID" gesetzt hat):
    _$assignTasknumberRow.style.display = "none";

    if(_selectedOrder && _selectedOrder.ID && _settings.TaskNumberPrefix && _settings.TaskNumberPrefix != "0" && _settings.TaskNumberPrefix != "T" && _hash.itemID && _hash.itemTaskNumber && _hash.itemTaskNumber != _selectedOrder.OrderTaskNumberID)
    {
      // Bei Suchergebnissen wird die Option angeboten, die Vorgangsnummer explizit zu setzen:
      if (isCustomSearch || !_settings.TaskNumberAutoAssign)
      {
        _$assignTasknumberRow.style.display = "";
        _$taskNumberButton.onclick = function() {
          _self.rpc.call("setTaskNumber", [_hash.itemID, _selectedOrder.OrderTaskNumberID],
            function(response) {
              _self.rpc.call('setUserFields', [_self.dataManager.account.id, _hash.itemID, _selectedOrder.ID, _data.result.data],
                function(response) {
                  _$assignTasknumberRow.style.display = "none";
                },
                function(err) {
                  showInfo();
                  _$dialog.show("Datenübernahme fehlgeschlagen", "Die automatische Datenübernahme in benutzerdefinierte Felder ist fehlgeschlagen. " + err.error.message);
                }
              );
            },
            function(err) {
              showInfo();
              _$dialog.show("Zuordnung fehlgeschlagen", err.error.message);
            },
            false
          );
        };
      }
      // Andernfalls wird die Vorgangsnummer automatisch gesetzt, sofern dies in den Einstellungen so festgelegt wurde:
      else if(_hash.itemKind != RpcItemKind.ikContact && _settings.TaskNumberAutoAssign === "1" && _hash.itemTaskNumber != _selectedOrder.OrderTaskNumberID && (_hash.itemTaskNumber[0] == 'T' || (_hash.itemTaskNumber[0] == _settings.TaskNumberPrefix /* && $action == 'selectOrder'*/ ) ))
      {
        var newTaskNumberID = _selectedOrder.OrderTaskNumberID;

        _self.rpc.call("setTaskNumber", [_hash.itemID, newTaskNumberID],
          function(response) {
            _hash.itemTaskNumber = newTaskNumberID;
          },
          function(err) {
            _$assignTasknumberRow.style.display = "";
            _$taskNumberButton.onclick = function() {
              _self.rpc.call("setTaskNumber", [_hash.itemID, _selectedOrder.OrderTaskNumberID],
                function(response) {
                  _self.rpc.call('setUserFields', [_self.dataManager.account.id, _hash.itemID, _selectedOrder.ID, _data.result.data],
                    function(response) {
                      _$assignTasknumberRow.style.display = "none";
                    },
                    function(err) {
                      showInfo();
                      _$dialog.show("Datenübernahme fehlgeschlagen", "Die automatische Datenübernahme in benutzerdefinierte Felder ist fehlgeschlagen. " + err.error.message);
                    }
                  );
                },
                function(err) {
                  showInfo();
                  _$dialog.show("Zuordnung fehlgeschlagen", err.error.message);
                },
                false
              );
            };
            _$taskNumberButton.title = "Ordnet das ausgewählte Element dem angezeigten " + ghRemoteServiceName + " Vorgang zu.\nDer Vorgang konnte nicht automatisch zugeordnet werden, da das Element zurzeit gesperrt ist oder nicht bearbeitet werden kann.";
            showInfo();
          },
          false
        );
      }
    }

    // Datenübernahme:

    if (_self.dataManager.account && _selectedOrder && _selectedOrder.ID && _hash.itemID && !isCustomSearch) {
      taskNumberPrefix = _hash.itemTaskNumber !== undefined && _hash.itemTaskNumber.length > 0 ? _hash.itemTaskNumber[0] : '';
      matchingTaskNumber = false;
  
      if (!matchingTaskNumber && (taskNumberPrefix == 'T' && (_settings.TaskNumberPrefix == '0' || _settings.TaskNumberPrefix == 'T')) || (taskNumberPrefix == _settings.TaskNumberPrefix)) {
        matchingTaskNumber = true;
      } else {
        prefixes = (_settings.AssignUserFieldsTaskNumberPrefixes || '').split(',');

        for (i = 0; i < prefixes.length; i++) {
          if (prefixes[i] == '*' || prefixes[i] == taskNumberPrefix) {
            matchingTaskNumber = true;
            break;
          }
        }
      }

      if (matchingTaskNumber) {
          _self.rpc.call('setUserFields', [_self.dataManager.account.id, _hash.itemID, _selectedOrder.ID, _data.result.data],
          function(response) {},
          function(err) {
            showInfo();
            _$dialog.show("Datenübernahme fehlgeschlagen", "Die automatische Datenübernahme in benutzerdefinierte Felder ist fehlgeschlagen. " + err.error.message);
          }
        );
      }
    }

    //  Bestellungen
    IOrders.clear();
    if(_selectedOrder)
    {

      if(_orders.length > 1 )
        IOrders.addEntry(null, {label: "Bestellung auswählen"}, _orderId==null);

      for(i = 0; i < _orders.length; i++)
      {
        IOrders.addEntry(_orders[i].ID, {
          label: _orders[i].Name, 
          icon: 'ghGuiDefaultIcons shoppingcart',
          tooltip: _orders[i].Description
        }, _orderId == _orders[i].ID);
      }
      _$orderRow.style.display="";
      _$orderRow_Count.innerHTML = '<div class="ghOrdersCount"><span class="ghGuiDefaultIcons shoppingcart"></span><span class="count">' + _orders.length + '</span></div>';
      ISplitter.setPanelVisible(_$templateView,  true);
    }
    else
    {
      ISplitter.setPanelVisible(_$templateView,  false);
      _$orderRow.style.display="none";
    }

    //  AddonActions vorhanden?
    if(_selectedOrder && _selectedOrder.OrderActions && _selectedOrder.OrderActions.length)
    {
      _$actionsView.innerHTML = '';
      ISplitter.setPanelVisible(_$actionsView,  true);

      for(i=0; i<_selectedOrder.OrderActions.length;i++)
      {

        //  Closure:
        (function(action) {
          var b= document.createElement('DIV');
          b.className = 'button ' + (action.Enabled ? '' : 'disabled');
          b.innerHTML = action.Caption;
          b.title = action.Tooltip;
          
          if(action.Enabled) {
            b.onclick = function() {
              var myFrame = document.createElement("IFRAME");
              document.body.appendChild(myFrame);
              myFrame.src = action.Url;
            };
          };
        
          var ico = document.createElement('SPAN');
          ico.className = 'icon ghGuiDefaultIcons ' + action.Icon;
          b.appendChild(ico);
          
          _$actionsView.appendChild(b);
        
        })(_selectedOrder.OrderActions[i]);
      }
      var b= document.createElement('DIV');
      b.className = 'clear';
      _$actionsView.appendChild(b);
    }
    else
    {
      ISplitter.setPanelVisible(_$actionsView,  false);
    }
      

    //  Orderdetails setzen
    IOrderDetails.clear();
  
    if(_selectedOrder && !_selectedOrder.HasDetails) {
      _selectedOrder.HasDetails = true; // Damit keine weitere Anforderung verschickt wird.

      showInfo('<span class="ghSpin"></span> Daten werden abgerufen...');

      var orderId = _selectedOrder.ID;

      if(_self.dataManager.account) {
        _self.rpc.call("getOrderDetails", [_self.dataManager.account.id, orderId, _data ? _data.result.data : null],
          function (response) {
            response.result.HasDetails = true;

            patchOrderDetails(response.result);
            showInfo();
            refresh();
          },
          function (err) {
            showInfo();
            showError(err);
          },
          true
        );
      }
    }

    if (_selectedOrder && _selectedOrder.OrderInfo) {
      IOrderDetails.setColumnLabel('Value', _selectedOrder.ShortName);

      for (i = 0; i<_selectedOrder.OrderInfo.length; i++) {
        IOrderDetails.addEntry(_selectedOrder.OrderInfo[i]);
      }

      for (i = 0; i < _selectedOrder.OrderTextInfo.length; i++) {
        var tmpOrderTextInfo = _selectedOrder.OrderTextInfo[i];

        if (tmpOrderTextInfo && tmpOrderTextInfo.Value) {
          if (Array.isArray(tmpOrderTextInfo.Value)) {
            for (j = 0; j < tmpOrderTextInfo.Value.length; j++) {
              tmpOrderTextInfo.Value[j] = '<div style="white-space:normal">' + tmpOrderTextInfo.Value[j] + '</div>';
            }
          }
          else {
            tmpOrderTextInfo.Value = '<div style="white-space:normal">' + tmpOrderTextInfo.Value + '</div>';
          }
        }

        IOrderDetails.addEntry(tmpOrderTextInfo);
      }

      ISplitter.setPanelVisible(_$orderItemsView,  true);
      ISplitter.setPanelVisible(_$orderDetailView, true);

      if (_splitterChanged) {
        _splitterChanged = false;
        ISplitter.resize();
      }

      IOrderDetails.render();
    } 
    else
    {
      ISplitter.setPanelVisible(_$orderItemsView,  false);
      ISplitter.setPanelVisible(_$orderDetailView, false);
    }

    //  Bestellliste kommt ohne Komponente aus (einfache Tabelle)
    if(_selectedOrder && _selectedOrder.OrderItems)
    {
      var x,r=0,order,table = '<table class="orderlist">';
      table += '<tr>';
      table += '<th class="col1"></th>';
      table += '<th class="col2">Artikel</th>';
      table += '<th class="col3">Preis</th>';
      table += '<th class="col4">Gesamt</th>';
      table += '</tr>';

      for (i = 0; i < _selectedOrder.OrderItems.length; i++) {
        order = _selectedOrder.OrderItems[i];
        table += '<tr class="' + ((r%2)?'even':'odd') + (i > 0 ? ' separate' : '') + '">';
        table += '<td class="col1">' + order.Quantity + 'x</td>';
        table += '<td class="col2">' + order.Name + '</td>';
        table += '<td class="col3">' + (order.UnitPrice !== undefined ? Number(order.UnitPrice).toFixed(2) + ' ' + order.Currency : '') + '</td>';
        table += '<td class="col4">' + (order.TotalPrice !== undefined ? Number(order.TotalPrice).toFixed(2) + ' ' + order.Currency : '') + '</td>';
        table += '</tr>';
        r++;

        if(order.Description) {
          for(x = 0; x < order.Description.length; x++) {
            table += '<tr class="'+((r%2)?'even':'odd')+'"><td class="col1"></td><td colspan="3" class="col2">' + order.Description[x]+ '</td></tr>';
            r++;
          }
        }
      }

      _$orderItemsView.innerHTML = table + '</table>';
    }
    else
    {
      _$orderItemsView.innerHTML = '<span></span>';
    }

    //  Höhe vom Splitter anpassen sodass min. alles angezeigt wird!
    if(_$orderDetailView.offsetHeight < IOrderDetails.getHeight()) {
      _$orderDetailView.style.height = Math.min(
        document.body.offsetHeight - _$orderDetailView.offsetTop,
        IOrderDetails.getHeight() + 20
      ) + 'px';
    }

    //  Buttons anpassen für Vorlagen:
    _buildCommunicationActions();    

    ISplitter.resize();
  };

  function _onAccountRefreshed() {
    //  Templates
    ITemplates.clear();

    //  Rootknoten patchen:
    if (_self.dataManager.account && _self.dataManager.account.templates.length)
    {
      ITemplates.addEntry(0, {label: 'Textbaustein auswählen', parent: null, _icon: 'ghGuiDefaultIcons forbidden2'});

      //  Restliche Nodes hinzufügen
      for(var i = 0; i < _self.dataManager.account.templates.length; i++) {
        var node = _self.dataManager.account.templates[i];
        ITemplates.addEntry(node.id, {label: node.Name, parent: node.ParentRef||0, icon: 'ghGuiDefaultIcons text', tooltip: node.Description, subject: node.Subject});
      }
      _$templatesRow.style.display = "";
      ISplitter.resize();
    }
    else
      _$templatesRow.style.display = "none";
  };

  //  Bestellansichten zurücksetzen
  function resetData() {
    _customers = [];
    _customerId= null;
    _selectedCustomer = null;
    _orders    = [];
    _orderId   = null;
    _selectedOrder = null;
    _data      = null;
    _templateId= null;

    //  Aktuelle Darstellung entfernen
    refresh();
  };

  //  Bereits vorhandene Daten als ungültig markieren und anhand der Suchparameter neu anfragen
  function invalidateData() {
    try {
      _hash = extractHashValues();

      _self.rpc.setSettings(_hash);

      if(_hash.search) { // Übergebenen Suchbegriff verwenden
        isCustomSearch = true;
        _hash.search = trim(_hash.search.replace(/\+/g, ' '));
        _$searchInput.setValue(_hash.search);
        _$searchInput.focus();
        RPCSearch("searchBySearchTerm", _hash.search);
      } else {
        isCustomSearch = false;
        _self.dataManager.invalidateData(_hash,
          function onSuccess(response, rpcMethod, rpcParams) {
            resetData();

            // Ergebnis ignorieren da ggf veraltet:
            if(response.id && response.id != _self.dataManager.rpc.getRpcId())
              return;

            showInfo();
            setDataFromSearchResult(response, rpcParams);
          },
          function onError(response, rpcMethod, rpcParams){
            resetData();

            // Ergebnis ignorieren da ggf veraltet:
            if(response.id && response.id != _self.rpc.getRpcId())
              return;

            showInfo();
            showError(response);
          },
          function onSearchTerm(searchTerm) {
            _$searchInput.setValue(searchTerm);
          },
          function onSearch(rpcMethod, rpcParams) {
            showInfo('<span class="ghSpin"></span> Die Suche wird durchgeführt...');
          }
        );
      }
    }
    catch(e) {
      criticalError(e);
    }
  };

  function RPCSearch(rpcMethod, rpcParams) {
    try {
      _self.dataManager.search(rpcMethod, rpcParams, _hash,
        function onSuccess(response, rpcMethod, rpcParams) {
          resetData();
          showInfo();
          setDataFromSearchResult(response, rpcParams);
        },
        function onError(response, rpcMethod, rpcParams){
          resetData();
          showInfo();
          showError(response);
        },
        function onSearch(rpcMethod, rpcParams) {
          showInfo('<span class="ghSpin"></span> Die Suche wird durchgeführt...');
        },
        function onSearchTerm(searchTerm) {
          _$searchInput.setValue(searchTerm);
        }
    );

    }
    catch(e) {
      showInfo(e.message);
      _$infoWrapper.style.display='';
    }
  };

  //  Startet eine Aktion im GH, namentlich das Öffnen einer Email/Fax/Briefes mit optional ausgewählter Textvorlage
  function GHAction(action) {
    var isReply, isForward, isReference, kind, iframe = document.createElement('IFRAME'), form = document.createElement('FORM');
    var customerEmail, customerFax, customerAddress, customerPhone, ccEmail, toEmail, attachments, i;

    function addEntry(key,value) {
      var p = document.createElement('INPUT');
      p.name = key;
      p.value = value;
      p.type = 'hidden';
      form.appendChild(p);
    };

    iframe.name = 'GHActionFrame';
    document.body.appendChild(iframe);

    form.action = 'greyhound:NewItem';
    form.method = 'POST';
    form.target = 'GHActionFrame';
    document.body.appendChild(form);

    if(action) {
      if(action.substr(0, 5) == 'reply')
        form.action = 'greyhound:AnswerItem';
      else if (action.substr(0, 7) == 'forward')
        form.action = 'greyhound:ForwardItem';
    }

    //  Optionale Parameter:
    if(_selectedCustomer)
    {
      customerEmail = _selectedCustomer.Email;
      customerFax = normalizePhoneNumber(_selectedCustomer.Fax);
      customerPhone = normalizePhoneNumber(_selectedCustomer.Phone);
      customerAddress = _selectedCustomer.Address ? _selectedCustomer.Address.join(', ') : '';
    }

    if(!customerPhone)
      customerPhone = 'Unbekannt <0>';

    if(!customerFax)
      customerFax = 'Unbekannt <0>';

    if(isCustomSearch)
      toEmail = _selectedCustomer ? _selectedCustomer.Email : '';
    else
    {
      if(action == 'newEmail') {
        if(_self.dataManager.email)
          toEmail = _self.dataManager.email;
        else if(_selectedCustomer && _selectedCustomer.Email)
          toEmail = _selectedCustomer.Email;
        else if(_self.dataManager.origEmail)
          toEmail = _self.dataManager.origEmail;
        else
          toEmail = '';
      }
      else if(_self.dataManager.origEmail)
        toEmail = _self.dataManager.origEmail;
      else
        toEmail = _self.dataManager.email ? _self.dataManager.email : (_selectedCustomer ? _selectedCustomer.Email : '');

      //  ccEmail
      if(_settings.CustomerCC == "1" && _selectedCustomer && _selectedCustomer.Email && _selectedCustomer.Email.toLowerCase() != toEmail.toLowerCase())
        ccEmail = _selectedCustomer.Email;
    }

    if(_hash.itemState != RpcItemState.isOpen && _hash.itemState != RpcItemState.isDone)
      addEntry('DisallowDone', 'True');

    switch(action)
    {
      case 'newEmail':
      case 'replyEmail':
      case 'forwardEmail':
        isReply   = (action == 'replyEmail');
        isForward = (action == 'forwardEmail');
        isReference = !isCustomSearch || isReply || isForward;
        kind      = RpcItemKind.ikEmail;
        addEntry('Kind', 'EMail');
        addEntry('Recipients[0]', isForward ? '' : toEmail);
        addEntry('RecipientsClear', 'True');

        if(ccEmail)
          addEntry('CCs[0]', ccEmail);
        break;

      case 'newFax':
      case 'replyFax':
      case 'forwardFax':
        isReply   = (action == 'replyFax');
        isForward = (action == 'forwardFax');
        isReference = !isCustomSearch || isReply || isForward;
        kind      = RpcItemKind.ikFax;
        addEntry('Kind', 'Fax');
        addEntry('Recipients[0]', isForward ? '' : customerFax);
        addEntry('RecipientsClear', 'True');
        break;

      case 'newLetter':
      case 'replyLetter':
      case 'forwardLetter':
        isReply   = (action == 'replyLetter');
        isForward = (action == 'forwardLetter');
        isReference = !isCustomSearch || isReply || isForward;
        kind      = RpcItemKind.ikLetter;
        addEntry('Kind', 'Letter');
        addEntry('Recipients[0]', isForward ? '' : customerAddress);
        addEntry('RecipientsClear', 'True');
        break;

      case 'newCall':
      case 'replyCall':
      case 'forwardCall':
        isReply   = (action == 'replyCall');
        isForward = (action == 'forwardCall');
        isReference = !isCustomSearch || isReply || isForward;
        kind      = RpcItemKind.ikCall;
        addEntry('Kind', 'Call');
        addEntry('Recipients[0]', isForward ? '' : customerPhone);
        addEntry('RecipientsClear', 'True');
        break;
    }

    if(isReference && _hash.itemTaskNumber)
      addEntry('Number', _hash.itemTaskNumber);

    if(_self.addAttachments) {
      try {
        attachments = _self.addAttachments();

        if(Array.isArray(attachments)) {
          for(i = 0; i < attachments.length; i++)
            addEntry('Attachments[' + i + ']', attachments[i]);
        }
      }
      catch(e) {
        showInfo('Es ist ein Fehler beim Hinzufügen von Anhängen aufgetreten: ' + e);
      }
    }

    //  Original Email besorgen:
    if(_self.dataManager.account) {
      try {
        var itemId = isReply || isForward ? _hash.itemID : 0;
        var getItemParams = [_self.dataManager.account.id, itemId, kind, true, _hash.itemGroup, _selectedOrder.ID, _data.result.data, _templateId];

        if(_self.dataManager.sendHash)
          getItemParams.push(_hash);

        _self.rpc.call(
          'getItem',
          getItemParams,
          function onItem(result) {
            var a, i, k, item = result.result, content = item.content, subject = '', signature = item.signature || '', template = '';

            if(itemId)
              subject = item.subject;

            if(item.template) {
              template = item.template.Content;
              subject = item.template.Subject || item.subject;
            }

            //  Kopfdaten hinzufügen

            if(isReference && itemId)
              addEntry("ItemID", itemId);

            addEntry("Group", _hash.itemGroup);
            addEntry("DataMode", item.isHTML ? "Html" : "Text");
            addEntry("Subject", subject);

            //  Template-Variablen
            for(k in item.templateVars)
              addEntry(k, item.templateVars[k]);

            if (itemId && item.isHTML) {
              if (content) {
                content = content.replace('<p>&nbsp;</p><div class="quoteheader"', '<div class="quoteheader"');
                // Das "<p>&nbsp;\r\n" wird benötigt, damit der Editor dort bevorzugt den Cursor platziert (und nicht z.B. in der Signatur): (id#8906)
                template = content.replace('<div class="quoteheader"', "<p>&nbsp;\r\n" + (template ? template + "\r\n" : "") + "</p>" + signature + '<p>&nbsp;</p><div class="quoteheader"');
              }
            } else {
              template = template + signature + "\r\n" + content;

              // Bei Text-Nachrichten mindestens zwei Leerzeilen am Anfang der Nachricht: (id#8905)
              if (!item.isHTML) {
                if (template[0] != '\n') {
                  template = '\n' + template;
                }

                if (template[1] != '\n') {
                  template = '\n' + template;
                }
              }
            }

            for(a = template.split("\n"), i = 0; i < a.length; i++)
              addEntry("Data" + (i + 1), trim(a[i]));

            try {
              form.submit();
            } catch (e) {
              debug(e);
            }
          },
          function onError(err) {
            showInfo('Es ist ein Fehler aufgetreten: ' + err.error.message);
            _$infoWrapper.style.display='';
          },
          false
        );
      } catch (e) {
        debug(e);
      }
    }
  };

  function showError(err) {
    if(err !== undefined && err.error !== undefined && err.error.name !== undefined && err.error.name == 'GhAddonEmptySearchException')
    {
      showInfo(err.error.message);
      _$infoWrapper.style.display=''; //  panel-blocker freigeben in diesem fall!
      resetData();
    }
    else {
      //  Error bauen und einblenden
      var message = '';

      if(!err.error.level || err.error.level != 'info')
        message = 'Es ist ein Fehler aufgetreten: ';

      message += err.error.message;

      showInfo(message);
      _$infoWrapper.style.display = '';
      resetData();
    }
  };

  //  Verarbeitet die empfangenen Daten aus einer Sucheanfrage
  function setDataFromSearchResult(data, searchParams) {
    _data = data;

    if (_data.result) {
      if (_data.result.searchedTerm) {
        _$searchInput.setValue(_data.result.searchedTerm);
      }

      _customers = _data.result.customers || [];
      _customerId= null;
      _selectedCustomer = null;

      if (_customers.length == 1) {
        _customerId = _customers[0].ID;     //  Nur setzen wenn eindeutig!
        _selectedCustomer = _customers[0];
      }

      filterOrdersByCustomerId(_customerId); // _data.result.orders || [];
    } else {
      _customers = [];
      _customerId= null;
      _selectedCustomer = null;

      _orders = [];
      _orderId= null;
      _selectedOrder = null;
    }

    //  Evtl. keine Daten gefunden?
    if (!_selectedOrder) {
      showInfo(_emptySearchMessage.replace('%SEARCHTERM%', _$searchInput.getValue()));
      _$infoWrapper.style.display=''; //  panel-blocker freigeben in diesem fall!
    }

    //  GUI ggf neu zeichnen:
    refresh();
  };

  function filterOrdersByCustomerId(id, selectedOrderId)
  {
    var tmpOrders, i;

    _orders = [];
    tmpOrders  = _data.result.orders || [];

    // Falls eine Bestellung vorausgewählt ist, dann wird deren Kunde der vorausgewählte Kunde:
    if(!_customerId && _data.result.selectedOrderId) {
      for(i = 0; i < tmpOrders.length; i++) {
        if(tmpOrders[i].ID == _data.result.selectedOrderId) {
          _customerId = tmpOrders[i].CustomerID;
          break;
        }
      }
    }

    if(_customerId) {
      for(i = 0; i < _customers.length; i++) {
        if(_customers[i].ID == _customerId)
          _selectedCustomer = _customers[i];
      }
    }

    for(i = 0; i < tmpOrders.length; i++)
      if(id == null || tmpOrders[i].CustomerID == id)
        _orders.push(tmpOrders[i]);

    if(_orders.length > 0)
    {
      //  Jetzt noch die ausgewählte (oder die erste) Bestellung als aktiv markieren:
      if(_data && _data.result && _data.result.selectedOrderId)
        setSelectedOrder(_data.result.selectedOrderId);
      else
        setSelectedOrder( _orders.length ? _orders[0].ID : null );
    }
    else
      setSelectedOrder(null);
  };

  function setSelectedOrder(id) 
  {
    var i;

    if(id===null) {
      _orderId = null;
      return;
    }

    for(i=0;i<_orders.length;i++)
    {
      if(_orders[i].ID == id)
      {
        _selectedOrder = _orders[i];
        _orderId = id;
        return;
      }
    }

    // Fallback: falls keine Bestellung exakt passt, dann werden führende Nullen aus der ID entfernt, da z.B. eine
    // ID aus einer Vorgangsnummer führende Nullen haben kann:
    id = id.toString().replace(/^0+/g, '');

    for(i=0;i<_orders.length;i++)
    {
      if(_orders[i].ID == id)
      {
        _selectedOrder = _orders[i];
        _orderId = id;
        return;
      }
    }

    _orderId = null; // nicht gefunden.
  };

  function patchOrderDetails( order ) {
    //  Details zur Order "orderId" setzen
    var i, tmpOrders  = [];

    if(_data && _data.result && _data.result.orders)
      tmpOrders = _data.result.orders;

    for(i = 0; i < tmpOrders.length; i++)
      if(tmpOrders[i].ID == order.ID)
        for(var k in order)     
          tmpOrders[i][k] = order[k];
  };

  function showHistory(data) {
    var iframe, form;

    function addEntry(key,value) {
      var p = document.createElement('INPUT');
      p.name = key;
      p.value = value;
      p.type = 'hidden';
      form.appendChild(p);
    };

    function showHistoryInternal(response) {
      iframe = document.createElement('IFRAME');
      iframe.name = 'GHHistoryFrame';
      document.body.appendChild(iframe);

      form = document.createElement('FORM');
      form.action = 'greyhound:Search';
      form.method = 'POST';
      form.target = 'GHHistoryFrame';
      document.body.appendChild(form);

      addEntry('NewTab', 'True');
      addEntry('Colors[0]', 'All');
      addEntry('Filters[0]', 'All');
      addEntry('Groups[0]', 'All');
      addEntry('Topics[0]', 'All');
      addEntry('Users[0]', 'All');

      if(!response || !response.result) {
        addEntry('SearchFor', ('' + data.email + ' ' + data.phone).trim());
        addEntry('SearchFields', 'Address');
      }
      if(response.result.contactRef)
        addEntry('ContactRef', '' + response.result.contactRef);
      else {
        addEntry('SearchFor', '' + response.result.search);
        addEntry('SearchFields', 'Address');
      }

      try {
        form.submit();
      }
      finally {
        document.body.removeChild(form);
        document.body.removeChild(iframe);
      }
    }

    _self.rpc.call("getContactRef", [data],
      showHistoryInternal,
      showHistoryInternal,
      true
    );

    return false;
  }
})();

var ghAddonPage = ghAddonSidebarPage;


window.onload = function() {
  window.onload = null;   //  verhindern das methode mehrfach aufgerufen wird.
  try { ghAddonSidebarPage.init() }
  catch(e) { document.body.innerHTML = e.message };
  window.scrollTo(0,0);
};

//  Addon initalisieren sobald DOM fertig geladen; Grafiken könnten hier noch nicht komplett vorhanden sein.
if (document.addEventListener)
  document.addEventListener("DOMContentLoaded", window.onload, false);
